
/**
  * @author THOMAS Alexandre
  * @author AUDRU Cedric
  */

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import java.lang.Integer;
import javax.swing.tree.*;





/**
  * Class permettant de creer une nouvelle tache sous forme d'une boite de dialogue /
  */
public class GanttDialogNewTask extends JDialog implements ActionListener
{

	/** JTextField pour le nom de la tache a modifier. */
	private JTextField jtfname;
	/** JConboBox pour l'edition du pere de la tache. */
	private JComboBox jcbfather;
	/** JTextField pour la date de debut de la tache. */
	private JTextField jtfbegin;
	/** JTextField pour la duree de la tache. */
	private JTextField jtflength;
	/** JList pour les dependances de la tache. */
	private JList jldepend;
	/** JSlider pour l'etat d'avancement de la tache. */
	private JSlider spercent;
	/** JCheckBox pour editer si c'est un point bilan. */
	private JCheckBox jcbbilan;

	/** Pointeur sur le GanttGraphicArea de l'application. */
	private GanttGraphicArea aarea;
	/** Pointeur sur le GanttTree de l'application. */
	private GanttTree ttree;
	/** Boutton pour le choix de la date */
	private JButton button;
	/** La langue utilisee. */
	GanttLanguage lang;

	/** champ static pour editer un numero pour les nouvelles taches */
	static int tasknumber=0;

	/** Construteur de l'application */
	public GanttDialogNewTask(JFrame parent,GanttTree tree,GanttGraphicArea area,GanttLanguage language)
	{

		super(parent, language.newTask(),true);
		setResizable(false);
		this.ttree = tree;
		this.aarea = area;
		this.lang = language;

		final String [] lot = tree.getAllTaskString(null);

		Box b1 = Box.createVerticalBox();

		//Le nom de la tache
		b1.add(new JLabel(language.name()));
		jtfname = new JTextField (language.newTask()+(++tasknumber));
		b1.add(jtfname);

		//Le mere de la tache
		b1.add(new JLabel(language.motherTask()));
		jcbfather = new JComboBox(lot);
		GanttTask current = tree.getSelectedTask();
		if(current==null) jcbfather.setSelectedItem(language.none());
		else jcbfather.setSelectedItem(tree.getSelectedTask().toString());
		b1.add(jcbfather);

		//La date de debut de la tache
		b1.add(new JLabel(language.dateOfBegining()));
		GanttCalendar today = new GanttCalendar();
		if(current==null) jtfbegin=new JTextField(today.toString());
		else jtfbegin=new JTextField(current.getStart().toString());
		jtfbegin.setEnabled(false);
		Box hb1 = Box.createHorizontalBox();
		hb1.add(jtfbegin);
		button = new JButton(new ImageIcon("icons/calendar.gif"));
		button.addActionListener(this);
		hb1.add(button);
		b1.add(hb1);

		//La duree de la tache
		b1.add(new JLabel(language.length()));
		jtflength=new JTextField("10");
		JScrollPane scrollpane = new JScrollPane(jtflength);
		b1.add(scrollpane);

		//pour savoir si c'est une tache bilan
		jcbbilan = new JCheckBox(language.meetingPoint());
		jcbbilan.setSelected(false);
		b1.add(jcbbilan);
		/*jcbbilan.addActionListener(new ActionListener()
		 	{	public void actionPerformed(ActionEvent evt)
				{
					//System.out.println("CheckBox press");
				}
			});*/


		Box b2 = Box.createVerticalBox();

		//La liste des dependances
		b2.add(new JLabel(language.depends()));
		String lot2 [] = new String[lot.length-1];
		for(int i=1;i<lot.length;i++) lot2[i-1]=lot[i];
		jldepend = new JList(lot2);
		JScrollPane scrollPane = new JScrollPane(jldepend);
		b2.add(scrollPane);

		//L'etat d'avancement de la tache sera souvent 0 mais on ne sais jamais
		b2.add(new JLabel(language.advancement()));
		spercent = new JSlider(JSlider.HORIZONTAL, 0, 100, 0) ;
		spercent.setPaintLabels(true);
		spercent.setPaintTicks(true);
		spercent.setPaintTrack(true);
		b2.add(spercent);

		getContentPane().add(b1,"West");

		getContentPane().add(b2,"East");


		JPanel p = new JPanel();
		JButton ok = new JButton(language.getOk());
		getRootPane().setDefaultButton(ok);
		p.add(ok);

		final JDialog theDialog = this;

		//Listener sur le bouton ok
		ok.addActionListener(new ActionListener()
		 	{	public void actionPerformed(ActionEvent evt)
				{

					ArrayList alt = ttree.getArryListTaskString(null);
					String theName = jtfname.getText();

					//on verifie qu'il n'existe pas d'autre tache avec le meme nom
					if(!alt.contains((Object)(theName)))
					{
						setVisible (false);
						//On modifie les parametres de la tache

						GanttCalendar cal = new GanttCalendar(jtfbegin.getText());
						Integer length = new Integer(jtflength.getText());

						//Creation de la nouvelle tache
						GanttTask task = new GanttTask(theName, cal , length.hashCode());
						task.setPercent(spercent.getValue());
						task.setBilan(jcbbilan.isSelected());
						if(task.getBilan()) task.setLength(1);


						//Affectation des dependances
						int [] indices = jldepend.getSelectedIndices();
						for(int i=0;i<indices.length;i++)
							task.setDepend(lot[indices[i]]);

						//on ajoute la nouvelle tache dans l'arbre
						DefaultMutableTreeNode father = ttree.getNode((String)jcbfather.getSelectedItem());
						ttree.addObject(task, father);


						GanttTask taskFather = new GanttTask("__Bidon__",null,0);
						//On fait cette boucle pour rafraichir toutes les taches mere du point de vue de leur date
						while(ttree.getNode(task.toString()).isRoot()==false)
						{
							father = ttree.getFatherNode(task);
							taskFather = (GanttTask)father.getUserObject();
							taskFather.refreshDateAndAdvancement(ttree);
							father.setUserObject(taskFather);
							task = taskFather;
						}

						//Rafraichissement de la partie graphique
						aarea.repaint();
					}
					else
					{
						JOptionPane.showConfirmDialog(theDialog,
							lang.propertiesMsg(theName), lang.error(),
							JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE );
					}
				}
			});
		//bouton cancel
		JButton cancel = new JButton(language.getCancel());
		p.add(cancel);
		cancel.addActionListener(new ActionListener()
		 	{	public void actionPerformed(ActionEvent evt)
				{ setVisible (false); }
			});

		getContentPane().add(p,BorderLayout.SOUTH);

		pack();
		setResizable(false);
		Point point = parent.getLocationOnScreen();
		int x = (int)point.getX() + parent.getWidth()/2;
		int y = (int)point.getY() + parent.getHeight()/2;
		setLocation(x - getWidth()/2, y-getHeight()/2);
	}

	/** Cette fonction gere les evenements. */
	public void actionPerformed (ActionEvent evt)
	{
		//Quand on clique sur un menu */
		if(evt.getSource() == button)
		{
			GanttCalendar date = new GanttCalendar(jtfbegin.getText());
			GanttDialogDate dd = new GanttDialogDate(this,date,lang);
			dd.show();
			jtfbegin.setText(dd.getDate().toString());
		}
	}

}



