
/**
  * @author THOMAS Alexandre
  * @author AUDRU Cedric
  */

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import java.lang.Integer;
import javax.swing.tree.*;


/**
  * Classe permettant l'edition des proprietes d'une tache sous forme de boite de dialogue.
  */
public class GanttDialogProperties extends JDialog implements ActionListener
{
	/** JTextField pour le nom de la tache a modifier. */
	private JTextField jtfname;
	/** JConboBox pour l'edition du pere de la tache. */
	private JComboBox jcbfather;
	/** JTextField pour la date de debut de la tache. */
	public JTextField jtfbegin;
	/** JTextField pour la duree de la tache. */
	private JTextField jtflength;
	/** JList pour les dependances de la tache. */
	private JList jldepend;
	/** JSlider pour l'etat d'avancement de la tache. */
	private JSlider spercent;
	/** JCheckBox pour editer si c'est un point bilan. */
	private JCheckBox jcbbilan;	
	/** Boolean qui permet de sire si l'utilisateur est autorise  modifier
	    le parametre d'etat d'avancement de la tache */
	boolean haschild=false;

	/** Pointeur sur le GanttTree de l'application. */
	private GanttTree ttree;
	/** Pointeur sur la GanttTask de l'application. */
	private GanttTask ttask;
	/** Pointeur sur le GanttGraphicArea de l'application. */
	private GanttGraphicArea aarea;
	/** Boutton pour le choix de la date */
	private JButton button;
	/** Valeur pour sauvegarder la valeur de l'etat d'avancement comme on peut la modifier en direct */
	int percentValue;
	/** La langue utilisee. */
	GanttLanguage lang;

	/** Constructeur  de la classe */
	public GanttDialogProperties(JFrame parent, GanttTree tree, GanttTask task,GanttGraphicArea area,
		GanttLanguage language)
	{
		super(parent, language.propertiesFor()+task.toString(),true);
		
		this.ttree = tree;
		this.ttask = task;
		this.aarea = area;
		this.percentValue = task.getPercent();
		this.lang = language;

		if(tree.getAllChildTask(task.toString()).size()!=0)
			haschild=true;

		final String [] lot = tree.getAllTaskString(task.toString());
		Box b1 = Box.createVerticalBox();

		//Le nom de la tache
		b1.add(new JLabel(language.name()));
		jtfname = new JTextField (task.toString());
		b1.add(jtfname);

		//Le pere de la tache
		/*b1.add(new JLabel("The father"));
		jcbfather = new JComboBox(lot);
		jcbfather.setSelectedItem(tree.getFatherNode(task).toString());
		b1.add(jcbfather);*/

		//La date de debut de la tache
		b1.add(new JLabel(language.dateOfBegining()));
		jtfbegin=new JTextField(task.getStart().toString());
		jtfbegin.setEnabled(false);
		if(haschild)jtfbegin.setEnabled(false);
		
		Box hb1 = Box.createHorizontalBox();
		hb1.add(jtfbegin);
		button = new JButton(new ImageIcon("icons/calendar.gif"));
		button.addActionListener(this);
		hb1.add(button);
		b1.add(hb1);
		if(haschild)button.setEnabled(false);

		//La duree de la tache
		b1.add(new JLabel(language.length()));
		jtflength=new JTextField(new Integer(task.getLength()).toString());
		if(haschild)jtflength.setEnabled(false);
		b1.add(jtflength);

		//pour savoir si c'est une tache bilan
		jcbbilan = new JCheckBox(language.meetingPoint());
		jcbbilan.setSelected(task.getBilan());
		b1.add(jcbbilan);
		/*jcbbilan.addActionListener(new ActionListener()
		 	{	public void actionPerformed(ActionEvent evt)
				{ 
					//System.out.println("CheckBox press");
				}
			});*/

		Box b2 = Box.createVerticalBox();

		//La liste des dependances de la tache
		b2.add(new JLabel(language.depends()));
		final String lot2 [] = new String[lot.length-1];
		for(int i=1;i<lot.length;i++) lot2[i-1]=lot[i];
		jldepend = new JList(lot2);
		ArrayList depend = task.getDepend();
		int [] index = new int [depend.size()];

		for(int i=0;i<depend.size();i++){
			index[i] = findTheIndex(depend.get(i), lot2);
			//System.out.println(depend.get(i));
		}
		jldepend.setSelectedIndices(index);

		JScrollPane scrollPane = new JScrollPane(jldepend);
		b2.add(scrollPane);

		//L'etant d'avancement de la tache		
		b2.add(new JLabel(language.advancement()));
		spercent = new JSlider(JSlider.HORIZONTAL, 0, 100, task.getPercent()) ;
		spercent.setPaintLabels(true);
		spercent.setPaintTicks(true);
		spercent.setPaintTrack(true);
		spercent.addChangeListener(new ChangeListener ()
			{
				public void stateChanged(ChangeEvent e) 
				{
					ttask.setPercent(spercent.getValue());
					DefaultMutableTreeNode node = ttree.getNode(ttask.toString());
					node.setUserObject(ttask);
					
					DefaultMutableTreeNode father = ttree.getFatherNode(ttask);
					GanttTask taskFather = (GanttTask)father.getUserObject();
					taskFather.refreshDateAndAdvancement(ttree);
					father.setUserObject(taskFather);
					
					aarea.repaint();
				}
			});
		
		if(haschild)spercent.setEnabled(false);
		b2.add(spercent);


		getContentPane().add(b1,"West");
		getContentPane().add(b2,"East");

		JPanel p = new JPanel();
		JButton ok = new JButton(language.getOk());
		getRootPane().setDefaultButton(ok);
		p.add(ok);
		
		final JDialog theDialog = this;
		
		//Listener sur le bouton ok
		ok.addActionListener(new ActionListener()
		 	{	public void actionPerformed(ActionEvent evt)
				{
					ArrayList alt = ttree.getArryListTaskString(ttask.toString());
					String theNewName = jtfname.getText();
					
					//on verifie qu'il n'existe pas d'autre tache avec le meme nom
					if(!alt.contains((Object)(theNewName)))
					{					
						//on cache la fenetre de propriete
						setVisible (false);
					
						//Modification des parametre de la tache
					
						ttask.setStart(new GanttCalendar(jtfbegin.getText()));
						ttask.setBilan(jcbbilan.isSelected());
						if(ttask.getBilan())
							ttask.setLength(1);
						else ttask.setLength(new Integer(jtflength.getText()).hashCode());
						ttask.setPercent(spercent.getValue());
					
						
						//Avant de modifier le nom on va regarder dans les taches et modifier dans le tableau de dependance
						//par le nouveau nom					
						ArrayList lgt = ttree.getAllTasks();
						
						for(int i=0;i<lgt.size();i++)
						{
							GanttTask tmpTask = (GanttTask) ((DefaultMutableTreeNode)lgt.get(i)).getUserObject();
							tmpTask.checkDepend(ttask.toString(), theNewName);
						}
															
						//maintenant on peu changer de nom
						ttask.setName(theNewName);
	
	
	
						int [] indices = jldepend.getSelectedIndices();					
						ttask.clearDepend();
						for(int i=0;i<indices.length;i++)
						{
							ttask.setDepend(lot2[indices[i]]);
							//System.out.println("########   "+indices[i]);
						}
	
						//DefaultMutableTreeNode theNewFather = ttree.getNode((String)jcbfather.getSelectedItem());
						DefaultMutableTreeNode node = ttree.getNode(ttask.toString());
						node.setUserObject(ttask);
						//node.setParent((MutableTreeNode) theNewFather);
	
	
	
						DefaultMutableTreeNode father = ttree.getFatherNode(ttask);
						GanttTask taskFather = (GanttTask)father.getUserObject();
						taskFather.refreshDateAndAdvancement(ttree);
						father.setUserObject(taskFather);

						aarea.repaint();
					} 
					else
					{
						JOptionPane.showConfirmDialog(theDialog,
							lang.propertiesMsg(theNewName), lang.error(),
							JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE );					
					}
				}
			});

		//bouton cancel
		JButton cancel = new JButton(language.getCancel());
		p.add(cancel);
		cancel.addActionListener(new ActionListener()
		 	{	public void actionPerformed(ActionEvent evt)
				{ 
					setVisible (false); 
					ttask.setPercent(percentValue);
					DefaultMutableTreeNode node = ttree.getNode(ttask.toString());
					node.setUserObject(ttask);
					aarea.repaint();
				}
			});

		getContentPane().add(p,BorderLayout.SOUTH);

		pack();
		
		
		
		setResizable(false);
		Point point = parent.getLocationOnScreen();
		int x = (int)point.getX() + parent.getWidth()/2;
		int y = (int)point.getY() + parent.getHeight()/2;
		setLocation(x-getWidth()/2, y-getHeight()/2);		
	}

	/** Fonction qui recherche la chaine s dans le tableau et retourne son indice */
	private int findTheIndex(Object s, String []lot)
	{
		for (int i=0;i<lot.length;i++)
			if(s.toString()==lot[i]) return i;
		return 0;
	}
	
	
	/** Cette fonction gere les evenements. */
	public void actionPerformed (ActionEvent evt)
	{
		//Quand on clique sur un menu */
		if(evt.getSource() == button)
		{
			GanttCalendar date = new GanttCalendar(jtfbegin.getText());
			GanttDialogDate dd = new GanttDialogDate(this,date,lang);
			dd.show();
			jtfbegin.setText(dd.getDate().toString());
		}
	}
			
	
	
}






