
/**
  * @author THOMAS Alexandre
  * @author AUDRU Cedric
  */

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Calendar;
import java.lang.String;
import javax.swing.tree.*;
import java.util.ArrayList;
import java.lang.Math;
import java.awt.print.*;



/**
  * Classe for the graphic part of the soft
  */
public class GanttGraphicArea extends JPanel implements Printable
{

	/** La date de debut du dessin. */
	GanttCalendar date;

	/** Un pointeur sur le GanttTree de l'application */
	GanttTree tree;

	/** La veleur du zoom
	  * 0 - par semaine
	  * 1 - par mois
	  * 2 - par 3 mois
	  * 3 - par annee
	  * 4 - par 3 ans */
	
	
	/** Zoom  sur une semaine */
	public final int ONE_WEEK=0;     
	
	/** Zoom  sur une semaine */
	public final int TWO_WEEK=1;     
	
	/** Zoom  sur une semaine */
	public final int ONE_MONTH=2;     
	
	/** Zoom  sur une semaine */
	public final int TWO_MONTH=3;     
	
	/** Zoom  sur une semaine */
	public final int THREE_MONTH=4; 
	
	/** Zoom  sur une semaine */
	public final int SIX_MONTH=5;     
	
	/** Zoom  sur une semaine */
	public final int ONE_YEAR=6;     
	
	/** Zoom  sur une semaine */
	public final int TWO_YEAR=7;     
	
	/** Zoom  sur une semaine */
	public final int THREE_YEAR=8;     
	
	
	/** La veleur du zoom
	  * 0 - par 1 semaine
	  * 1 - par 2 semaines
	  * 1 - par mois
	  * 2 - par 3 mois
	  * 3 - par annee
	  * 4 - par 3 ans */  
	/* La valeur du zoom */ 	  
	int zoomValue;

	/**Tableau pour stoquer les differents parametres d'affichage de chaque tache */
	ArrayList listOfParam = new ArrayList();

	/** Variable qui sert  reperer l'ecart quand on bouge le margYollbar du JTree */
	int margY;

	/** Point en haut a gauche pour les marge d'impression */
	Point upperLeft;

	/** Render on the window or for printing*/
	boolean printRendering=false;

	/** Le language utilise */
	GanttLanguage language;

	/*! L'application principale */
	GanttProject appli;

	/*! Ancienne position de la souris en X au moment ou on a cliqu dessus. */
	int oldX;

	/** Constructeur principal. */
	public GanttGraphicArea (GanttProject app, GanttTree tree, GanttLanguage language)
	{
		date = new GanttCalendar();
		date.setDay(1);
		this.tree = tree;
		zoomValue = ONE_MONTH;	//zoom par moi par default
		
		margY = 0;
		this.language = language;
		appli = app;

		//On ajoute un listener lors d'un mouvement de la molette
		this.addMouseWheelListener(new MouseWheelListener (){
			public void mouseWheelMoved(MouseWheelEvent e)
			{
				zoomValue += e.getWheelRotation();
				if(zoomValue<0) zoomValue=0;
				if(zoomValue>8) zoomValue=8;

				zoomToBegin();

				appli.bZoomIn.setEnabled(zoomValue==0?false:true);
				appli.bZoomOut.setEnabled(zoomValue==8?false:true);

				repaint();
			}
		});


		//Ajout d'un listener lors d'un clic souris
		this.addMouseListener(new MouseListener (){
			public void mouseClicked(MouseEvent e){}
          	public void mouseEntered(MouseEvent e){}
          	public void mouseExited(MouseEvent e){}
          	public void mousePressed(MouseEvent e)
			{
				oldX = e.getX();
				setCursor(new Cursor(Cursor.HAND_CURSOR));
			}
          	public void mouseReleased(MouseEvent e)
			{
				setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
				int f = 1;
				int x = oldX - e.getX();
				if(x>2)f=-1;
				else if(x<-2)f=1;
				else return;	//eviter de bouger le calendrier si on clique un petit coup sans le faire expret

				switch(zoomValue)
				{
					case ONE_WEEK :date.go(Calendar.WEEK_OF_YEAR,1*f); break;
//					case TWO_WEEK :date.go(Calendar.WEEK_OF_YEAR,1*f); break;
					case TWO_WEEK:date.go(Calendar.DATE,14*f); break;
					case ONE_MONTH:date.go(Calendar.MONTH,1*f); break;
					case TWO_MONTH:date.go(Calendar.MONTH,1*f); break;
					case THREE_MONTH:date.go(Calendar.MONTH,1*f); break;
					case SIX_MONTH:date.go(Calendar.MONTH,2*f); break;
					case ONE_YEAR:date.go(Calendar.YEAR,1*f);break;
					case TWO_YEAR:date.go(Calendar.YEAR,1*f);break;
					case THREE_YEAR:date.go(Calendar.YEAR,1*f);break;
				}

				repaint();
			}
		});

	}


	/** La taille par defaut du panel. */
	public Dimension getPreferredSize()
	{
		return new Dimension(465, 600);
	}

	/** Fonction qui sera appelee quand on devra changer de date */
	public void changeDate (boolean next)
	{
		int f=1;
		if(!next)f=-1;

		switch(zoomValue)
		{
			case ONE_WEEK:date.go(Calendar.WEEK_OF_YEAR,1*f); break;
			case TWO_WEEK:date.go(Calendar.DATE,14*f); break;
			case ONE_MONTH:case TWO_MONTH:case THREE_MONTH:date.go(Calendar.MONTH,1*f); break;
			case SIX_MONTH:date.go(Calendar.MONTH,2*f); break;
			case ONE_YEAR:case TWO_YEAR:case THREE_YEAR:date.go(Calendar.YEAR,1*f);break;
		}
	}

	/** Fonction qui sera lorsque l'on zoom on veut reinitialise au debut. */
	public void zoomToBegin ()
	{
		switch(zoomValue)
		{
			case ONE_WEEK: case TWO_WEEK:
					String d = language.getDay(date.getDayWeek());
					while(!d.equals(language.getDay(2)))
					{
						date.add(1);
						d = language.getDay(date.getDayWeek());
					}
					
					break;
			case ONE_MONTH:case TWO_MONTH:case THREE_MONTH:case SIX_MONTH:date.setDay(1);break;
			case ONE_YEAR:case TWO_YEAR:case THREE_YEAR:date.setMonth(0);
				   date.setDay(1);
				   break;
		}
	}


	/** dessinne le panel */
	public void paintComponent(Graphics g)
	{
		//Super paint component!!!!!!!!!!
		super.paintComponent(g);

		if(printRendering)
			g.translate((int)upperLeft.getX(),(int)upperLeft.getY());

		//dessin des barres verticales
		paintCalendar1(g);
		//Dessin des taches
		paintTasks(g);
		//dessin des depances
		paintDepend(g);
		//dessin de la legende
		paintCalendar2(g);
				
	}


	/** Retourne si la tache passee est visible dans le JTree. */
	public boolean isVisible (GanttTask thetask)
	{

		boolean res = true;
		ArrayList expand = tree.getExpand();
		DefaultMutableTreeNode father = tree.getFatherNode(thetask);

		//la tache root n'est pas visible
		if(father==null) return false;

		while(father!=null)
		{
			if(!expand.contains(father.toString())) res=false;
			GanttTask t = tree.getTask(father.toString());
			father = tree.getFatherNode(t);

		}

		return res;

	}


	/** Recherche le coeff dans le tableau de parametre */
	public int indexOf(ArrayList listOfParam, String coef)
	{
		for(int i=0;i<listOfParam.size();i++)
			if(coef==listOfParam.get(i).toString())
				return i;
		return -1;
	}

	/** Modification de la valeur du margYollbar. */
	public void setScrollBar (int v) { margY = v; }

	/** Retourne la valeur du margYollbar */
	public int getScrollBar () { return margY; }

	/** Cette fonction permet de changer la valeur du zoom */
	public void setZoom (int z) { zoomValue = z; }

	/** Cette fonction retourne la valeur du zoom */
	public int getZoom () { return zoomValue; }

	/** Fonction permettant de modifier la valeur de la date de debut du calendrier */
	public void setDate(GanttCalendar d) { date = d; }

	/** Fonction qui retourne la date de dbut du calendrier */
	public GanttCalendar getDate() { return date ; }

	/** On reaffecte la valeur du language */
	public void setLanguage (GanttLanguage language) { this.language = language; }

	/** Cette fonction retourne la granularite en fonction de la valeur de zoom */
	public int getGranit(boolean day)
	{
		GanttCalendar cal;
		int res=7;	//les 7 jours de la semaines par default la valeur de zoomValue=ONE_WEEK
		switch(zoomValue)
		{
			case ONE_WEEK: res=7; break;
		
			case TWO_WEEK:res=14;break;
		
			case ONE_MONTH:res=date.getNumberOfDay();break;
			
			case TWO_MONTH:cal = date.Clone();
				   res=cal.getNumberOfDay();
				   cal.goNextMonth();
				   res+=cal.getNumberOfDay();
				   break;

			case THREE_MONTH:cal = date.Clone();
				res=0;
				for(int i=0;i<3;i++)
				{
					res+=cal.getNumberOfDay();
					cal.goNextMonth();
				}break;
			
			case SIX_MONTH:cal = date.Clone();
				res=0;
				for(int i=0;i<6;i++)
				{
					res+=cal.getNumberOfDay();
					cal.goNextMonth();
				}break;

			case ONE_YEAR:if(!day) res=12;
				   else res=(date.getYear()%4==0)?366:365;
				   break;

			case TWO_YEAR:if(!day) res=12*2;
				   else
				   {
				   	cal = date.Clone();
					res=0;
					for(int i=0;i<2;i++)
					{
				   		if(cal.getYear()%4==0) res+=366;
						else res+=365;
						cal.go(Calendar.YEAR,1);
					}
				   }
				   break;

			case THREE_YEAR:if(!day) res=12*3;
				   else
				   {
				   	cal = date.Clone();
					res=0;
					for(int i=0;i<3;i++)
					{
				   		if(cal.getYear()%4==0) res+=366;
						else res+=365;
						cal.go(Calendar.YEAR,1);
					}
				   }
				   break;
		}
		return res;
	}


	/** Cette fonction retourne le pas d'avance (en nombre de jours) */
	public int getFoot()
	{
		int res=1;
		switch(zoomValue)
		{
			case ONE_YEAR:case TWO_YEAR:case THREE_YEAR:res=date.getNumberOfDay();break;
			//default:res=1;break;
		}
				
		return res;
	}

	/** Fonction qui dessine le calendrier */
	public void paintCalendar1(Graphics g)
	{
		int sizex = getWidth();
		int sizey = getHeight();
		int headery = 45;
		int gra = sizex / getGranit(false);		//la granularite
		int gra2 = getGranit(false);
		float fgra = (float)sizex / (float)getGranit(false);	


		g.setFont(new Font("SansSerif",Font.BOLD,12));

		//Dessin de la legende en haut
		g.setColor(Color.white);
		g.fillRect(0,0,sizex,sizey);


		GanttCalendar tmpdate;
		if(zoomValue==ONE_WEEK || zoomValue==TWO_WEEK) tmpdate = date.Clone();
		else tmpdate = new GanttCalendar(date.getYear(), date.getMonth(), 1);
		g.setFont(new Font("SansSerif",Font.PLAIN,10));

		//Affichage des barres verticales
		for(int i=0;i<gra2;i++)
		{
			String sDay = tmpdate.getdayWeek(language);
			String sMonth = tmpdate.getdayMonth(language);

			//Affichage des barres verticales
			if(zoomValue<6) {
				if(sDay==language.getDay(6)){
					g.setColor(new Color((float)0.905,(float)0.905,(float)0.905));
					g.fillRect((int)(fgra*i),headery,gra+1/*(int)(fgra+0.5)*/,sizey);

					g.setColor(new Color((float)0.745,(float)0.745,(float)0.745));
					g.drawLine((int)(fgra*i),headery,(int)(fgra*i),sizey+headery);

				} else if(sDay==language.getDay(0)) {
					g.setColor(new Color((float)0.905,(float)0.905,(float)0.905));
					g.fillRect((int)(fgra*i),headery,gra+1/*(int)(fgra+0.5)*/,sizey);
					if(zoomValue==ONE_WEEK || zoomValue==TWO_WEEK){
						g.setColor(new Color((float)0.745,(float)0.745,(float)0.745));
						g.drawLine((int)(fgra*i),headery,(int)(fgra*i),sizey+headery);
					}
				}


			} else {

				if(sMonth==language.getMonth(0))
				{
					g.setColor(new Color((float)0.905,(float)0.905,(float)0.905));
					g.fillRect((int)(fgra*i),headery,(int)fgra,sizey);
				}
				g.setColor(new Color((float)0.745,(float)0.745,(float)0.745));
				g.drawLine((int)(fgra*i),headery,(int)(fgra*i),sizey+headery);

			}




			//On dessinne le jour courant en bleut
			if(zoomValue<6) {
				if(tmpdate.compareTo(new GanttCalendar())==0)
				{
					g.setColor(new Color((float)0.843,(float)0.890,(float)0.910));
					g.fillRect((int)(fgra*i),headery,(int)fgra,sizey);
					g.setColor(new Color((float)0.722,(float)0.765,(float)0.780));
					g.drawLine((int)(fgra*i),headery,(int)(fgra*i),sizey+headery);
				}
			} else {
				GanttCalendar today=new GanttCalendar();
				if(tmpdate.getYear()==today.getYear() && tmpdate.getMonth()==today.getMonth())
				{
					g.setColor(new Color((float)0.843,(float)0.890,(float)0.910));
					float s = fgra/(float)tmpdate.getNumberOfDay();
					g.fillRect((int)(fgra*i)+(int)(s*today.getDay()),headery,((s<2)?2:(int)s),sizey);
					g.setColor(new Color((float)0.722,(float)0.765,(float)0.780));
					g.drawLine((int)(fgra*i)+(int)(s*today.getDay()),headery,(int)(fgra*i)+(int)(s*today.getDay()),sizey+headery);
				}
			}

			tmpdate.add(getFoot());
		}
	}


	/** Fonction qui dessine la legende du calendrier */
	public void paintCalendar2(Graphics g)
	{
		int sizex = getWidth();
		int sizey = getHeight();
		int headery = 45;
		//int gra = sizex / getGranit(false);		//la granularite
		int gra2 = getGranit(false);
		float fgra=(float)(sizex) / (float)getGranit(false);		//la granularite
		

		g.setFont(new Font("SansSerif",Font.BOLD,12));

		//affichage du rectangle gris et des bordure en relief
		g.setColor(new Color((float)0.905,(float)0.905,(float)0.905));
		g.fillRect(0,0,sizex,headery);
		g.setColor(new Color((float)0.482,(float)0.482,(float)0.482));
		g.drawRect(0,0, sizex-1, headery/2);
		g.drawRect(0,headery/2, sizex-1, headery/2);
		g.setColor(new Color((float)0.807,(float)0.807,(float)0.807));
		g.drawLine(1,headery/2-1,sizex-2,headery/2-1);
		g.drawLine(0,headery-2,sizex-2,headery-2);
		g.setColor(Color.white);
		g.drawLine(1,1,sizex-2,1);
		g.drawLine(0,headery/2+1,sizex-2,headery/2+1);

		g.setColor(Color.black);
		//Le texte dans la barre tout en haut
		float posX;
		GanttCalendar cal;
		switch(zoomValue)
		{
			//on affiche juste le numero de semaine et le mois correspondant
			case ONE_WEEK: g.drawString(language.week()+date.getWeek()+" - "+date.getdayMonth(language)+"  "+date.getYear(),2,headery/2-5);
					break;

			case TWO_WEEK:
						g.drawString(language.week()+(date.getWeek())+" - "+date.getdayMonth(language)+"  "+date.getYear(),2,headery/2-5);
						g.drawString(language.week()+(date.getWeek()+1)+" - "+date.getdayMonth(language)+"  "+date.getYear(),(int)(fgra*7)+2,headery/2-5);
				break;

			//On affiche le mois et l'annee
			case ONE_MONTH: g.drawString(date.getdayMonth(language)+"  "+date.getYear(),2,headery/2-5);
					break;

			// Affichage de deux mois
			case TWO_MONTH:cal = date.Clone();
					posX = 0;
					for(int i=0;i<2;i++)
					{
						g.drawString(cal.getdayMonth(language)+"  "+cal.getYear(),(int)posX+2/*i*sizex/3+2*/,headery/2-5);
						posX+=((float)cal.getNumberOfDay()*fgra);
						cal.goNextMonth();
					}
				break;

			//Affichage des trois mois
			case THREE_MONTH:cal = date.Clone();
					posX = 0;
					for(int i=0;i<3;i++)
					{
						g.drawString(cal.getdayMonth(language)+"  "+cal.getYear(),(int)posX+2/*i*sizex/3+2*/,headery/2-5);
						posX+=((float)cal.getNumberOfDay()*fgra);
						cal.goNextMonth();
					}
					break;

			// Affichage de six mois
			case SIX_MONTH:cal = date.Clone();
					posX = 0;
					for(int i=0;i<6;i++)
					{
						g.drawString(cal.getdayMonth(language).substring(0,(sizex>480)?3:1)+"  "+new
							Integer(cal.getYear()).toString().substring(2,4),(int)posX+2/*i*sizex/3+2*/,headery/2-5);
						posX+=((float)cal.getNumberOfDay()*fgra);
						cal.goNextMonth();
					}
				break;

			//Affichage de l'annee
			case ONE_YEAR: g.drawString(""+date.getYear(),2,headery/2-5);
					break;

			// Affichage de deux annees
			case TWO_YEAR:int dy2 = date.getYear();
					for(int i=0;i<2;i++,dy2++)
					{
						g.setColor(Color.black);
						g.drawString(""+dy2,i*sizex/2+2,headery/2-5);
					}
				break;

			//Affichage des 3 annees
			case THREE_YEAR: int dy = date.getYear();
					for(int i=0;i<3;i++,dy++)
					{
						g.setColor(Color.black);
						g.drawString(""+dy,i*sizex/3+2,headery/2-5);
					}
					break;
		}

		GanttCalendar tmpdate;
		if(zoomValue==ONE_WEEK || zoomValue==TWO_WEEK) tmpdate = date.Clone();
		else tmpdate = new GanttCalendar(date.getYear(), date.getMonth(), 1);
		g.setFont(new Font("SansSerif",Font.PLAIN,10));

		//Affichage des jours ou mois
		for(int i=0;i<gra2;i++)
		{
			String sDay = tmpdate.getdayWeek(language);
			String sMonth = tmpdate.getdayMonth(language);

			g.setColor(Color.black);
			switch(zoomValue)
			{
				case ONE_WEEK: g.drawString(sDay.substring(0,(sizex>300)?3:1)+" "+tmpdate.getDate(),(int)(fgra*i)+2,headery-7);
						g.setColor(new Color((float)0.482,(float)0.482,(float)0.482));
						g.drawLine((int)(fgra*i),headery/2,(int)(fgra*i),headery-1);
						break;
						
				case TWO_WEEK:g.drawString(sDay.substring(0,1)+" "+tmpdate.getDate(),(int)(fgra*i)+2,headery-7);
						g.setColor(new Color((float)0.482,(float)0.482,(float)0.482));
						g.drawLine((int)(fgra*i),headery/2,(int)(fgra*i),headery-1);
						break;
						
				case ONE_MONTH: g.drawString(""+tmpdate.getDate(),(int)(fgra*i)+3,headery-7);
						g.setColor(new Color((float)0.482,(float)0.482,(float)0.482));
						g.drawLine((int)(fgra*i),headery/2,(int)(fgra*i),headery-1);
						break;
				
				// Affichage de plusieur mois
				case TWO_MONTH:case THREE_MONTH:
					 	if(sDay==language.getDay(1)) g.drawString(""+tmpdate.getDate(),(int)(fgra*i)+1,headery-7);
							break;
				case SIX_MONTH:
						if(sDay==language.getDay(1)) g.drawString(""+tmpdate.getDate(),(int)(fgra*i)+1,headery-7);
							break;
				//Affichage d'une annee
				case ONE_YEAR: g.drawString(sMonth.substring(0,(sizex>300)?3:1),(int)(fgra*i)+2,headery-7);
						g.setColor(new Color((float)0.482,(float)0.482,(float)0.482));
						g.drawLine((int)(fgra*i),headery/2,(int)(fgra*i),headery-1);
						break;
						
				// Affichage de plusieurs annees
				case TWO_YEAR: case THREE_YEAR: g.drawString(sMonth.substring(0,1),(int)(fgra*i)+2,headery-7);
						g.setColor(new Color((float)0.482,(float)0.482,(float)0.482));
						g.drawLine((int)(fgra*i),headery/2,(int)(fgra*i),headery-1);
						break;
			}

			tmpdate.add(getFoot());
		}
	}

	/** Fonction qui dessine toutes les taches  */
	public void paintTasks(Graphics g)
	{
		int sizex = getWidth();
		int sizey = getHeight();
		int headery = 45;
		//int gra = sizex / getGranit(true);		//la granularite
		float fgra = (float)sizex / (float)getGranit(true);

		g.setFont(new Font("SansSerif",Font.PLAIN,9));

		//On recupere toute les taches
		ArrayList listOfTask = tree.getAllTasks();

		//Pourra etre optimiser pour savoir si on reaffiche tjs la meme chose
		listOfParam.clear();

		int y=0;

		//On va calculer tous les parametres
		for(int i=0;i<listOfTask.size();i++)
		{
			//On recupere la tache
			GanttTask task =(GanttTask) ((DefaultMutableTreeNode)listOfTask.get(i)).getUserObject();

			//Seulement si la tache est visible (visible au sens de l'empilement dans le JTree)
			if(isVisible(task))
			{
				int x1=-10, x2=sizex+10;
				int e1; //ecart entre la date de debut de la tache et la date du debut du calendrier
				int fois;
				y++;

				//on calcul l'ecart entre la date de debut de la tache et la date de debut du celendrier
				e1 = date.diff(task.getStart());

				//Calcul des parametre x1 et x2 d'affichage de la tache
				/*if(zoomValue<5) {

					x1 = e1 * gra * ((date.compareTo(task.getStart())==1)?-1:1);
					x2 = x1 + task.getLength() * gra;

				} else {	//Cas de l'affichage par annee
*/
					//on calcul ien float pour sinon pb avec les divisions entieres
					float fx1, fx2;

					if(task.getBilan())
					{
							fx1 = (float)e1 * fgra * ((date.compareTo(task.getStart())==1)?-1:1);
							x1 = (int)fx1;
					}
					else
					{
						fx1 = (float)e1 * fgra * ((date.compareTo(task.getStart())==1)?-1:1);
						fx2 = fx1 + (float)task.getLength() * fgra;

						x1 = (int) fx1;
						x2 = (int) fx2;
					}
				//}


				//si c'est une tache bilan alors on ne fait que ca
				if(task.getBilan()) {
					paintATaskBilan(g, x1, y,task.toString());
				}
				//Esct ce que c'est une tache mere
				else if(tree.getAllChildTask(task.toString()).size()!=0){// || tree.getFatherNode(task).isRoot()) {
					paintATaskFather(g, x1, x2, y,task.toString());
					paintAdvancement(g, x1, x2, y, task.getPercent(), true);
				}
				//C'est une tache fille
				else {
					paintATaskChild(g, x1, x2, y,task.toString());
					paintAdvancement(g, x1, x2, y, task.getPercent(), false);
				}

				//On ajoute dans le tableau de parametres
				listOfParam.add(new GanttPaintParam(task.toString(), x1, x2, y));
			}
		}
	}



	/** Fonction qui dessine une tache mere*/
	public void paintATaskFather(Graphics g, int x1, int x2, int y, String taskName)
	{
		//le rectangle de base
		g.setColor(Color.black);
		g.fillRect(x1, y*20+41-margY, x2-x1,3 );

		//on trace les petits triangles
		int xPoints [] = new int[3];
		int yPoints [] = new int[3];
		xPoints[0] = x1;
		xPoints[1] = x1+5;
		xPoints[2] = x1;
		yPoints[0] = y*20+44-margY;
		yPoints[1] = y*20+44-margY;
		yPoints[2] = y*20+48-margY;
		g.fillPolygon(xPoints, yPoints, 3);
		xPoints[0] = x2;
		xPoints[1] = x2-5;
		xPoints[2] = x2;
		g.fillPolygon(xPoints, yPoints, 3);

		if(printRendering)
		{
			g.setColor(Color.black);
			g.drawString(taskName, x2+40,y*20+50-margY);
		}
	}


	/** Fonction qui dessine une tache fille */
	public void paintATaskChild(Graphics g, int x1, int x2, int y, String taskName)
	{
		//On trace le rectangle de base
		g.setColor(new Color((float)0.549,(float)0.713,(float)0.807));
		g.fillRect(x1, y*20+40-margY, (x2-x1-1),12 );

		//les bordures en plus pour faire joli...
		g.setColor(Color.black);
		g.drawLine(x1, y*20+40-margY, (x2-1), y*20+40-margY);
		g.drawLine(x1, y*20+52-margY, (x2-1), y*20+52-margY);
		g.drawLine(x1, y*20+40-margY, x1, y*20+52-margY);
		g.drawLine((x2-1), y*20+40-margY, (x2-1), y*20+52-margY);

		g.setColor(new Color((float)0.741,(float)0.745,(float)0.741));
		g.drawLine(x1+1, y*20+41-margY, (x2-1)-1, y*20+41-margY);
		g.drawLine(x1+1, y*20+41-margY, x1+1, y*20+51-margY);

		g.setColor(new Color((float)0.388,(float)0.396,(float)0.388));
		g.drawLine(x1+2, y*20+51-margY, (x2-1)-2, y*20+51-margY);
		g.drawLine(x2-2, y*20+42-margY, x2-2, y*20+51-margY);

		if(printRendering)
		{
			g.setColor(Color.black);
			g.drawString(taskName, x2+40,y*20+50-margY);
		}
	}


	/** Fonction qui dessinne une tache bilan */
	public void paintATaskBilan(Graphics g, int x1, int y, String taskName)
	{

		int gra = getWidth() / getGranit(false);		//la granularite
		float fgra = (float)getWidth() / (float)getGranit(true);
		//on trace le petit losange
		int xPoints [] = new int[4];
		int yPoints [] = new int[4];

		if(zoomValue<5)
		{
			xPoints[0] = x1+gra/2;
			xPoints[1] = x1+gra/2-5;
			xPoints[2] = x1+gra/2;
			xPoints[3] = x1+gra/2+5;
		}else{
			xPoints[0] = (int)((float)x1+fgra/2);
			xPoints[1] = (int)((float)x1+fgra/2-5);
			xPoints[2] = (int)((float)x1+fgra/2);
			xPoints[3] = (int)((float)x1+fgra/2+5);
		}
		yPoints[0] = y*20+40-margY;
		yPoints[1] = y*20+45-margY;
		yPoints[2] = y*20+50-margY;
		yPoints[3] = y*20+45-margY;

		g.setColor(Color.black);
		g.fillPolygon(xPoints, yPoints, 4);
		g.setColor(new Color((float)0.388,(float)0.396,(float)0.388));
		g.drawPolygon(xPoints, yPoints, 4);

		if(printRendering)
		{
			g.setColor(Color.black);
			g.drawString(taskName, x1+16,y*20+50-margY);
		}
	}

	/** Foncion qui affiche les dependances entre les differentes taches */
	public void paintDepend(Graphics g)
	{
		//On recupere toute les taches
		ArrayList listOfTask = tree.getAllTasks();

		//des longueurs de traits
		int l1=6;
		int l2=5;

		//Pour generer des polygones en forme de triangles pour les pointes de fleches
		int xPoints [] = new int[3];
		int yPoints [] = new int[3];

		//on parcours toutes les taches
		for(int i=0;i<listOfTask.size();i++)
		{
			//On recupere la tache
			GanttTask task =(GanttTask) ((DefaultMutableTreeNode)listOfTask.get(i)).getUserObject();

			//Seulement si la tache est visible (visible au sens de l'empilement dans le JTree)
			if(isVisible(task))
			{
				//On recupere le tableau de dependance de la tache
				ArrayList depend = task.getDepend();

				//on parcours toutes les dependances
				for(int j=0;j<depend.size();j++)
				{
					GanttTask task2 = tree.getTask((String)depend.get(j));
					if(this.isVisible(task2))
					{

						//on recupere les index des deux taches a connecter
						int index1 = this.indexOf(listOfParam, task.toString());
						int index2 = this.indexOf(listOfParam, task2.toString());

						if(index1!=-1 && index2!=-1)
						{
							//on trace la fleche correcpondante

							//on recupere les coordonnees en x......
							int xa = ((GanttPaintParam)listOfParam.get(index1)).x2;
							int xb = ((GanttPaintParam)listOfParam.get(index2)).x1;
							//..... et celle en y
							int ya = ((GanttPaintParam)listOfParam.get(index1)).y;
							int yb = ((GanttPaintParam)listOfParam.get(index2)).y;

							//pour savoir si on monte ou on descend
							int e=(yb-ya)>0?10:-10;

							//horizontal  de milieu
							g.drawLine(xa+l1,ya*20+45+e-margY,xb-l1,ya*20+45+e-margY);
							if(xa!=32)
							{
								//horizontal et vertical de debut
								g.drawLine(xa,ya*20+45-margY,xa+l1,ya*20+45-margY);
								g.drawLine(xa+l1,ya*20+45-margY,xa+l1,ya*20+45+e-margY);


							} else {
								g.setColor(Color.white);
								g.drawLine(xa+l1,ya*20+45+e-margY,xa+l1-3,ya*20+45+e-margY);
								g.drawLine(xa+l1-8,ya*20+45+e-margY,xa+l1-11,ya*20+45+e-margY);
								g.drawLine(xa+l1-16,ya*20+45+e-margY,xa+l1-19,ya*20+45+e-margY);
								g.setColor(Color.black);
							}

							g.drawLine(xb-l1,ya*20+45+e-margY,xb-l1,yb*20+45-margY);

							//petite queue de fin
							g.drawLine(xb-l1,yb*20+45-margY,xb,yb*20+45-margY);
							//on trace la pointe de la fleche
							xPoints[0] = xb;
							xPoints[1] = xb-l2;
							xPoints[2] = xb-l2;
							yPoints[0] = yb*20+45-margY;
							yPoints[1] = yb*20+45+3-margY;
							yPoints[2] = yb*20+45-3-margY;
							g.fillPolygon(xPoints, yPoints, 3);
						}
					}
				}
			}
		}
	}



	/** Dessine l'etat d'avancement de la tache */
	public void paintAdvancement (Graphics g, int x1, int x2, int y, int percent, boolean justText)
	{
		//on affiche le pourcentage au bout de la tache
		g.setColor(Color.black);
		g.drawString("["+percent+"%]",x2+8,y*20+50-margY);

		//On dessine aussi la barre d'avancement
		if(!justText)
		{
			float fp = (float)(x2-x1) * (float)percent / 100;
			g.setColor(new Color((float)0.196,(float)0.196,(float)0.196));
			g.fillRect(x1, y*20+44-margY, (int)fp-1,4 );
		}
	}


	/** Cette class permet de stocker les differents parametres d'affichage de chaque tache
	  * pour pouvoir les reutiliser dans l'affichage des differentes dependances */
	public class GanttPaintParam
	{
		/** Les coordonnes en pixel de debut et de fin de la tache*/
		public int x1,x2;

		/** Le nom de la tache associee */
		public String name;

		/** La valeur en y */
		public int y;

		/**Constructeur */
		public GanttPaintParam(String name, int x1, int x2, int y)
		{
			this.name = name;
			this.x1 = x1;
			this.x2 = x2;
			this.y = y;
		}

		public String toString() { return name; }
	}

	/** Methode qui imprime la page */
	public int print(Graphics graphics, PageFormat pageFormat, int pageIndex)
	{

		if (pageIndex >= 1) {
        	return Printable.NO_SUCH_PAGE;
    	}
		printRendering=true;
		upperLeft=new Point((int)pageFormat.getImageableX(), (int)pageFormat.getImageableY());
		paintComponent(graphics);
		printRendering=false;
		return Printable.PAGE_EXISTS;
	}

	/** Cette fonction imprime le projet */
	public void printProject()
	{
		PrinterJob printJob = PrinterJob.getPrinterJob();
        printJob.setPrintable(this);
        if (printJob.printDialog()) {
            try {
                printJob.print();
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }

	}
}

