
/**
  * @author THOMAS Alexandre
  * @author AUDRU Cedric
  */


import java.lang.String;
import java.lang.Object;
import java.util.ArrayList;
import java.util.Calendar;
import javax.swing.tree.*;
import java.io.*;

/**
  * Classe qui gere une tache.
  */
public class GanttTask implements Serializable
{

	/** Notes concernant la tache */
	private String notes;

	/** Date de debut de la tache */
	private GanttCalendar start;

	/*Duree de la tache (nombre de jours). */
	private int length;

	/** Nom de la tache */
	private String name;

	/** Liste des dependances pour la tache */
	private ArrayList depend = new ArrayList();

	/** Etat d'avancement de la tache (entre 0 et 100%)*/
	private int percent;
	
	/**Est -ce que cette tache est un point de bilan.(par defaut  false) */
	private boolean bilan;

/////////////////////////////////////////////////////////////////////////////////


	/** Constructeur */
	public GanttTask(String name, GanttCalendar start, int length)
	{
		this.name = name;
		this.start = start;
		this.length = length;
		this.percent = 0;
		this.bilan = false;
	}


	/** Retourne le nom de la tache. */
	public String toString () { return name; }

	/** Retourne la duree de la tache. */
	public int getLength() { return length; }

	/** Retourne la date de debut de la tache. */
	public GanttCalendar getStart() { return start; }
	
	/** Retourne la date de fin de la tache. */
	public GanttCalendar getEnd() 
	{ 
		GanttCalendar end = start.Clone();
		end.add(length);
		return end; 
	}

	/** Retourne les notes concernant la tache */
	public String getNotes () { return notes; }

	/** Retourne l'etat d'avancement de la tache. */
	public int getPercent () { return percent; }

	/** Retourne la liste des dependances de la tache. */
	public ArrayList getDepend () { return depend; }

	/** Retourne la valeur de bilan. */
	public boolean getBilan() { return bilan; }


	/** Modification du nom de la tache. */
	public void setName (String name) { this.name = name; }

	/** Modification de la duree de la tache. */
	public void setLength (int l) { length = l; }

	/** Modification de la date de debut de la tache. */
	public void setStart (GanttCalendar s) { start = s; }

	/** Modification des notes de la taches */
	public void setNotes (String note) { notes = note; }

	/** Modification de l'etat d'avancement de la tache */
	public void setPercent (int percent) { this.percent = percent; }

	/** Modification de la valeur de bilan */
	public void setBilan (boolean b) { bilan = b; }
	
	/** Ajout d'une nouvelle tache. */
	public void setDepend ( String t ) { depend.add(t); }

	/** Efface la liste des taches. */
	public void clearDepend() { depend.clear(); };
	
	/** Cette fonction regarde dans le tableau des dependance si le nom de la tache "t" existe.
	  * Dans ce cas on le remplace par le nouveau nom "n" */
	 public void checkDepend (String t, String n)
	 {
	 		for(int i=0;i<depend.size();i++)
				if(depend.get(i).toString() == t)
				{
					depend.remove(i);
					depend.add(n);
				}
	 }

	/**
	  * Cette fonction permet de recalculer la date de debut
	  * d'une tache et la duree, en fonction de ses taches filles
	  */
	public void refreshDateAndAdvancement (GanttTree tree)
	{
		if(tree.getNode(name).isRoot()) return ;
		ArrayList l = tree.getAllChildTask(name);
		if(l.size()==0) return;

		GanttTask taskMin = (GanttTask) ((DefaultMutableTreeNode)l.get(0)).getUserObject();
		GanttTask taskMax = (GanttTask) ((DefaultMutableTreeNode)l.get(0)).getUserObject();


		int sumDay=taskMin.length;;
		float sum=(float)taskMin.length*(float)taskMin.percent/100;;

		for(int i=1;i<l.size();i++)
		{
			GanttTask task = (GanttTask) ((DefaultMutableTreeNode)l.get(i)).getUserObject();

			//Recherche de la tache qui commence la premiere
			if(taskMin.start.compareTo(task.start)==1)
				taskMin = task;

			//Recherche de la tache qui se fini la derniere
			if(taskMax.start.newAdd(taskMax.length).compareTo(task.start.newAdd(task.length))==-1)
				taskMax = task;

			//Calcul pour l'etat d'avancement en fonction des taches filles
			if(!task.bilan)
			{
				sumDay+=task.length;
				sum+=(float)task.length*(float)task.percent/100;
			}

		}

		//Modification de l'etat d'avancement de la tache pere
		this.percent = (int)(sum*100/(float)sumDay);

		this.setStart(taskMin.start.Clone());
		this.setLength(taskMin.start.diff(taskMax.start.newAdd(taskMax.length)));
	}
}

