
/**
  * @author THOMAS Alexandre
  * @author AUDRU Cedric
  */

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.tree.*;
import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.*;


/**
  * Classe qui gere le panel avec le JTree.
  */
public class GanttTree extends JPanel
{
	/** Le noeud root du JTree. */
	private DefaultMutableTreeNode rootNode;

	/** Le model du JTree. */
    private DefaultTreeModel treeModel;

	/** Le JTree. */
    private JTree tree;

	/** Pointeur sur la zone graphique. */
	private GanttGraphicArea area=null;

	/** Pointeur sur la fenetre principale. */
	private GanttProject appli;

	/** Un tacleau avec le nom des tache (sert pour l'empilement et le depilement). */
	private final ArrayList expand = new ArrayList();

	/** La scrollbar verticale pour le JTree */
	JScrollBar vbar;
	
	/** Le language utilise */
	GanttLanguage language;

	/** Constructeur. */
	public GanttTree (GanttProject app, GanttLanguage language)
	{
		super();

		this.appli = app;
		this.language=language;

		//creation du neoud root
		rootNode = new DefaultMutableTreeNode(new GanttTask ("None",	new GanttCalendar(2003,0,1), 1));
        treeModel = new DefaultTreeModel(rootNode);
        treeModel.addTreeModelListener(new GanttTreeModelListener());

		//Construcion du JTree
		tree = new JTree(treeModel);
        tree.setEditable(false);
        tree.getSelectionModel().setSelectionMode (TreeSelectionModel.SINGLE_TREE_SELECTION);
        tree.setShowsRootHandles(true);
		tree.setRowHeight(20);
		tree.setRootVisible(false);
		tree.addTreeExpansionListener(new GanttTreeExpansionListener());

		//pour changer les icones dans l'arbre
		DefaultTreeCellRenderer rendu ;
		rendu = (DefaultTreeCellRenderer) tree.getCellRenderer();
		rendu.setOpenIcon(new ImageIcon("icons/taskg.gif"));
		rendu.setLeafIcon(new ImageIcon("icons/taskb.gif"));
		rendu.setClosedIcon(new ImageIcon("icons/taskg.gif"));

		
		JScrollPane scrollpane = new JScrollPane(tree);
		setLayout( new BorderLayout() );
		add( scrollpane , BorderLayout.CENTER);
		vbar = scrollpane.getVerticalScrollBar();
		vbar.addAdjustmentListener(new GanttAdjustmentListener());
		
	
		//Un listener quand on clique sur le bouton droit de la souris et sur un tache
		MouseListener ml = new MouseAdapter() {
		     public void mousePressed(MouseEvent e) {
		         int selRow = tree.getRowForLocation(e.getX(), e.getY());
		         TreePath selPath = tree.getPathForLocation(e.getX(), e.getY());
		         if(selRow != -1) {
					 if(e.getClickCount() == 1 && e.getButton() == MouseEvent.BUTTON3 ) {
						tree.setSelectionPath(selPath);
						createPopupMenu(e.getX(), e.getY()) ;

		             }
		         }
		     }
		 };
		 tree.addMouseListener(ml);
	}
	
	

	/** Creation d'un menu popup dans la fenetre pour la tache courante. */
	public void createPopupMenu(int x, int y)
	{
		
		//Creation d'un menu popup
		JPopupMenu menu = new JPopupMenu();
		menu.add(appli.createNewItem(language.createTask(), "icons/insert.gif"));
		menu.add(appli.createNewItem(language.deleteTask(), "icons/delete.gif"));
		menu.add(appli.createNewItem(language.propertiesTask(), "icons/properties.gif"));
		menu.add(appli.createNewItem(language.notesTask(), "icons/notes.gif"));
		menu.add(appli.createNewItem(language.upTask(), "icons/up.gif"));
		menu.add(appli.createNewItem(language.downTask(), "icons/down.gif"));
		menu.show(this,x,y-area.getScrollBar());
	}

	/** Taille preferencielle du panel. */
	//public Dimension getPreferredSize() { return new Dimension(300, 540); }

	/** On affecte le pointeur sur la partie graphique. */
	public void setGraphicArea (GanttGraphicArea area) { this.area = area; }
	
	/** On reaffecte la valeur du language */
	public void setLanguage (GanttLanguage language) { this.language = language; }

	/** Retourne le tableau dAdjustmentListenere depilement. */
	public ArrayList getExpand() { return expand; }

	/** Ajoute un fils au noeud pere. */
	public DefaultMutableTreeNode addObject(Object child, DefaultMutableTreeNode parent) {
        DefaultMutableTreeNode childNode =
                new DefaultMutableTreeNode(child);

        if (parent == null) parent = rootNode;

		treeModel.insertNodeInto(childNode, parent, parent.getChildCount());

		tree.scrollPathToVisible(new TreePath(childNode.getPath()));
        return childNode;
    }

	/** Retourne la tache selectionnee ou null si aucune ne l'est. */
	public GanttTask getSelectedTask()
	{
		TreePath currentSelection = tree.getSelectionPath();
		if (currentSelection == null) return null;
		String selected = currentSelection.getLastPathComponent().toString();
		if(getNode(selected).isRoot()) return null;
		return getTask (selected);
	}

	/** Retourne la GanttTask en fonction de son nom */
	public GanttTask getTask ( String name )
	{
		DefaultMutableTreeNode base;
		base = (DefaultMutableTreeNode)tree.getModel().getRoot();
		Enumeration e = base.preorderEnumeration();
		while (e.hasMoreElements())
		{
			base = (DefaultMutableTreeNode)e.nextElement();
			if(base.toString() == name)
				return (GanttTask)(base.getUserObject());
		}
		return null;
	}

	/** Retourne le DefaultMutableTreeNodequi porte le nom de name. */
	public DefaultMutableTreeNode getNode (String name)
	{
		DefaultMutableTreeNode res, base;
		base = (DefaultMutableTreeNode)tree.getModel().getRoot();
		Enumeration e = base.preorderEnumeration();
		while (e.hasMoreElements())
		{
			res =((DefaultMutableTreeNode)e.nextElement());
			if(res.toString() == name)
				return res;
		}
		return null;
	}


	/** Retourne un tableau contenant toutes les taches. */
	public ArrayList getAllTasks ()
	{
		ArrayList res = new ArrayList();
		DefaultMutableTreeNode base;
		Enumeration e = (rootNode).preorderEnumeration();
		while (e.hasMoreElements())
			res.add(e.nextElement());
    	return res;
	}

	/** Retourne les taches filles de la tache passee en parametre. */
	public ArrayList getAllChildTask (String task)
	{
		DefaultMutableTreeNode base;
		base = (DefaultMutableTreeNode)getNode(task);
		Enumeration e = base.children();
		ArrayList res = new ArrayList();
		while (e.hasMoreElements())
			res.add(e.nextElement());
		return res;
	}


	/** Retourne toutes les taches dans un tableau. */
	public Object [] getAllTaskArray ()
	{
		ArrayList al = getAllTasks();
		return al.toArray();
	}

	/** Retourne un tableau contenant toutes les taches exceptee celle passe en parametre. */
	public String [] getAllTaskString(String except)
	{
		ArrayList l = getAllTasks();
		String [] res;
		if(except==null) res = new String [ l.size() ];
		else res = new String [ l.size() - 1];

		int i=0,j=0;
		for( ;i<l.size();i++)
		{
			if(except==null || (l.get(i).toString() != except) )
			{
				Array.set(res,j,l.get(i).toString());
				j++;
			}
		}
		return res;
	}

	/** Retourne une arraylist avec toutes les String correspondant aux taches */
	public ArrayList getArryListTaskString(String except)
	{
		ArrayList l = getAllTasks();
		ArrayList res = new ArrayList();
		for(int i=0;i<l.size();i++)
		{
			if((except!=null && l.get(i).toString()!=except) || except==null)
				res.add(l.get(i).toString());
		}
		return res;
	}


	/** Efface le noeud courant du JTree. */
	public void removeCurrentNode() {
        TreePath currentSelection = tree.getSelectionPath();
        if (currentSelection != null) {
            DefaultMutableTreeNode currentNode = (DefaultMutableTreeNode)
                         (currentSelection.getLastPathComponent());
            MutableTreeNode parent = (MutableTreeNode)(currentNode.getParent());
            if (parent != null) {
                treeModel.removeNodeFromParent(currentNode);
                return;
            }
        }
    }

	/** Methode qui efface entierement le JTree. */
	public void clearTree()
	{
		rootNode.removeAllChildren();
		treeModel.reload();
	}

	/** Retourne le pere(DefaultMutableTreeNode) de la tache passee.*/
	public DefaultMutableTreeNode getFatherNode(GanttTask node)
	{
		if(node==null){return null;}
		if(getNode(node.toString()).isRoot()){return null;}
		DefaultMutableTreeNode tmp = (DefaultMutableTreeNode)getNode(node.toString());
		if(tmp==null) {return null;}

		return (DefaultMutableTreeNode)tmp.getParent();
		//return (DefaultMutableTreeNode)getNode(node.toString()).getParent();
	}

	/** Accesseur sur le JTree. */
	public JTree getJTree() { return tree; }

	/** Modification du JTree. */
	public void  setJTree(JTree t) { tree = t; }

	public DefaultTreeModel getJTreeModel() { return treeModel; }

	public void setJTreeModel (DefaultTreeModel model) { treeModel = model; tree.setModel (model); }


	public DefaultMutableTreeNode getRoot() { return rootNode; }

	public void setRoot (DefaultMutableTreeNode dmtn)
	{
		rootNode = dmtn;
		treeModel = new DefaultTreeModel(rootNode);
        tree.setModel(treeModel);
		
		//remise  jour de la liste d'expantion des noeuds
		expand.clear();
		ArrayList l = getAllTasks();
		for(int i=0;i<l.size();i++)
		{
			//On depile l'arbre en entier
			String nameOfTask=l.get(i).toString();
			DefaultMutableTreeNode node=this.getNode(nameOfTask);
			tree.scrollPathToVisible(new TreePath(node.getPath()));

			//on rempli le tableau des elements depils avec le nomde chaque tache mere			
			DefaultMutableTreeNode theFather = getFatherNode(getTask(nameOfTask));			
			if(theFather!=null && !expand.contains(theFather.toString()) )
				expand.add(theFather.toString());
		}
	}


	/** Fonction qui permet de faire remonter la tache courante dans le hierarchie de son pere */
	public void upCurrentNode()
	{
		GanttTask current = getSelectedTask();
		if( current==null) return;
		//on ne peut modifier la position que des feuilles
		if(getAllChildTask(current.toString()).size()!=0) return;
		DefaultMutableTreeNode cdmtn = getNode(current.toString());
		DefaultMutableTreeNode cdmtn2 = (DefaultMutableTreeNode)cdmtn.clone();
		DefaultMutableTreeNode father = this.getFatherNode(current);
		int index = father.getIndex((TreeNode)cdmtn);

		treeModel.removeNodeFromParent(cdmtn);

		//nouvelle position du noeud
		index = (index==0)?0:index-1;

		treeModel.insertNodeInto((MutableTreeNode) cdmtn2, (MutableTreeNode)father, index);
		
		//on reselecte le noeud sur lequel on etait
		TreeNode[] treepath=cdmtn2.getPath();
		TreePath path = new TreePath( treepath );
		tree.setSelectionPath(path);
		area.repaint();
	}

	/** Fonction qui permet de faire descentre la tache courante dans le hierarchie de son pere */
	public void downCurrentNode()
	{
		GanttTask current = getSelectedTask();
		if( current==null) return;
		//on ne peut modifier la position que des feuilles
		if(getAllChildTask(current.toString()).size()!=0) return;
		DefaultMutableTreeNode cdmtn = getNode(current.toString());
		DefaultMutableTreeNode cdmtn2 = (DefaultMutableTreeNode)cdmtn.clone();
		DefaultMutableTreeNode father = this.getFatherNode(current);
		int index = father.getIndex((TreeNode)cdmtn);

		treeModel.removeNodeFromParent(cdmtn);

		//nouvelle position du noeud
		index = (index==father.getChildCount())?father.getChildCount():index+1;

		treeModel.insertNodeInto((MutableTreeNode) cdmtn2, (MutableTreeNode)father, index);
		
		//on reselecte le noeud sur lequel on etait
		TreeNode[] treepath=cdmtn2.getPath();
		TreePath path = new TreePath( treepath );
		tree.setSelectionPath(path);
		area.repaint();
	}
//////////////////////////////////////////////////////////////////////////////////////////
	/**
	  * Class utilisee pour le scrollbar qui bouge. 
	  */
	public class GanttAdjustmentListener implements AdjustmentListener {	
		/** Fonction listener quand l'ascenseur est boug */
		public void adjustmentValueChanged(AdjustmentEvent e) 
		{
			if(area!=null)
			{
				area.setScrollBar(e.getValue());
				area.repaint();
			}
		}
	}


//////////////////////////////////////////////////////////////////////////////////////////
	/**
	  * Classe utilisee quand on empile ou que l'on depile un branche de l'arbre.
	  */
	public class GanttTreeExpansionListener  implements TreeExpansionListener {
		/** Depilement d'une branche. */
		public void treeExpanded(TreeExpansionEvent e) {
			if(area!=null) area.repaint();
			if(!expand.contains(e.getPath().getLastPathComponent().toString()))
				expand.add(e.getPath().getLastPathComponent().toString());
		}

		/** Empilement d'une branche. */
		public void treeCollapsed(TreeExpansionEvent e) {
			if(area!=null) area.repaint();
			expand.remove(expand.indexOf(e.getPath().getLastPathComponent().toString()));			
		}
	}

//////////////////////////////////////////////////////////////////////////////////////////

	/**
	  * Classe pour gerer les evenement sur le model de l'arbre
	  */
	public class GanttTreeModelListener implements TreeModelListener {
		/** Quand on modifie un noeud */
        public void treeNodesChanged(TreeModelEvent e) {
			if(area!=null) area.repaint();
		}
		/** Quand on insere un noeud. */
        public void treeNodesInserted(TreeModelEvent e) {
			
			if(!expand.contains(e.getTreePath().getLastPathComponent().toString()))
				expand.add(e.getTreePath().getLastPathComponent().toString());
			if(area!=null) area.repaint();
        }
		/** Quand on supprime un noeud. */
        public void treeNodesRemoved(TreeModelEvent e) {
			//expand.remove(expand.indexOf(e.getTreePath().getLastPathComponent().toString()));
			if(area!=null) area.repaint();
        }
		/** Quand on modifie la structure de l'arbre. */
        public void treeStructureChanged(TreeModelEvent e) {
			if(area!=null) area.repaint();
        }
    }


}
