/***************************************************************************
                           GanttProject.java  -  description
                             -------------------
    begin                : mar 2003
    copyright            : (C) 2003 by Thomas Alexandre
    email                : alexthomas@free.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

package net.sourceforge.ganttproject;

import java.awt.*;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.security.AccessControlException;
import java.util.Iterator;
import java.util.Locale;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import net.sourceforge.ganttproject.document.Document;
import net.sourceforge.ganttproject.document.DocumentCreator;
import net.sourceforge.ganttproject.document.DocumentsMRU;
import net.sourceforge.ganttproject.gui.GanttLookAndFeelInfo;
import net.sourceforge.ganttproject.gui.GanttLookAndFeels;
import net.sourceforge.ganttproject.gui.UIConfiguration;
import net.sourceforge.ganttproject.language.GanttLanguage;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * The class aible to load and save options on the file
 */
public class GanttOptions {

  public GanttLanguage language = GanttLanguage.getInstance();

  public Color color;

  public int x = 0, y = 0, width = 800, height = 600;

  protected String styleClass = "", styleName = "";
  public GanttLookAndFeelInfo lookAndFeel;

  public boolean isloaded = false;

  public boolean automatic=false;
	
  public boolean dragTime=true;
  
  public boolean openTips=true;
	
  public boolean redline=true;
  
  public int lockDAVMinutes=240;
  
  public String xslDir = GanttOptions.class.getResource("/xslt").toString();
  
  public String xslFo = GanttOptions.class.getResource("/xslfo/ganttproject.xsl").toString();
  
  public String workingDir = "";                      	

  public DocumentsMRU documentsMRU;
    private UIConfiguration myUIConfig;
    private Font myChartMainFont;

    public GanttOptions(Color c, int x, int y, int width, int height,
                      GanttLookAndFeelInfo lookAndFeel,
                      boolean automatic, boolean dragTime,
                      String xslDir, String xslFo,String workingDir,
                      boolean tips, boolean redline, int lockDAVMinutes,
                      DocumentsMRU documentsMRU, UIConfiguration uiConfiguration) {
    color = c;
    this.x = x;
    this.y = y;
    this.width = width;
    this.height = height;
    this.lookAndFeel = lookAndFeel;
	this.automatic=automatic;
	this.dragTime=dragTime;
	this.openTips=tips;
	this.redline=redline;
	this.lockDAVMinutes=lockDAVMinutes;
        myUIConfig = uiConfiguration;

	if(xslFo!=null)
		this.xslFo=xslFo;
	
	if (xslDir != null)
        this.xslDir = xslDir;
	
	try {
		this.workingDir = System.getProperty("user.home");                     	
	} catch (AccessControlException e) {
		// This can happen when running in a sandbox (Java WebStart)
		System.err.println (e + ": " + e.getMessage());
	}
	if (workingDir != null && new File(workingDir).exists())
        this.workingDir = workingDir;

	this.documentsMRU = documentsMRU;	
  }

  /** Save the options file */
  public void save() {
    try {

      File file = new File(System.getProperty("user.home") +
                           System.getProperty("file.separator") +
                           ".ganttproject");
      //DataOutputStream fout = new DataOutputStream(new FileOutputStream(file));
      OutputStreamWriter fout = new OutputStreamWriter(new FileOutputStream(
          file), "UTF-8");

      fout.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<!DOCTYPE ganttproject.sourceforge.net>\n");
      fout.write("<ganttproject-options>\n");

      //write the language
      fout.write("    <language selection=\"" + language.getText("shortLanguage") + "\"/>\n");

      //write the task Color
      fout.write("    <task-color red=\"" + color.getRed() + "\" green=\"" +
                 color.getGreen() + "\" blue=\"" + color.getBlue() + "\"/>\n");

      //Geometry of the window
      fout.write("    <geometry x=\"" + x + "\" y=\"" + y + "\" width=\"" +
                 width + "\" height=\"" + height + "\"/>\n");

      //look'n'feel
      fout.write("    <looknfeel name=\"" + lookAndFeel.getName() + "\" class=\"" +
                 lookAndFeel.getClassName() + "\"/>\n");
	  
	  //automatic popup launch
      fout.write("    <automatic-launch value=\"" + (new Boolean(automatic)).toString() + "\"/>\n");
			
			//automaticdrag time on the chart
      fout.write("    <dragTime value=\"" + (new Boolean(dragTime)).toString() + "\"/>\n");
			
	  //automatic tips of the day launch
      fout.write("    <tips-on-startup value=\"" + (new Boolean(openTips)).toString() + "\"/>\n");
		
      //Show red line for today
      fout.write("    <redline value=\"" + Boolean.valueOf(myUIConfig.isRedlineOn()).toString() + "\"/>\n");

      //Should WebDAV resources be locked, when opening them?
      fout.write("    <lockdavminutes value=\"" + lockDAVMinutes + "\"/>\n");
	  
	  //write the xsl directory
      fout.write("    <xsl-dir dir=\"" + xslDir + "\"/>\n");
	  
	  //write the xslfo directory
      fout.write("    <xsl-fo file=\"" + xslFo + "\"/>\n");
	  
	  //write the working directory directory
      fout.write("    <working-dir dir=\"" + workingDir + "\"/>\n");

      //The last opened files
      fout.write("    <files>\n");
      Iterator iterator = documentsMRU.iterator();
      while (iterator.hasNext()) {
      	Document document = (Document) iterator.next();
		fout.write("        <file path=\"" + correct(document.getPath()) +
				   "\"/>\n");
      }
      fout.write("    </files>\n");
        fout.write("<font category=\"menu\" spec=\""+getFontSpec(getUIConfiguration().getMenuFont())+"\"/>\n");
        fout.write("<font category=\"chart-main\" spec=\""+getFontSpec(getUIConfiguration().getChartMainFont())+"\"/>\n");
      //End of options
      fout.write("</ganttproject-options>");
      fout.close();

    }
    catch (Exception e) {
      System.out.println(e);
    }
  }

    private String getFontSpec(Font font) {
        return font.getFamily()+"-"+getFontStyle(font)+"-"+font.getSize();
    }

    private String getFontStyle(Font font) {
        String result;
        final int BOLDITALIC = Font.BOLD+Font.ITALIC;
        switch (font.getStyle()) {
            case Font.PLAIN: {
                result = "plain";
                break;
            }
            case Font.BOLD: {
                result = "bold";
                break;
            }
            case Font.ITALIC: {
                result="italic";
                break;
            }
            case BOLDITALIC: {
                result = "bolditalic";
                break;
            }
            default : {
                throw new RuntimeException("Illegal value of font style. style="+font.getStyle()+" font="+font);
            }
        }
        return result;
    }

    public String correct(String s) {
    String res;
    res = s.replaceAll("&", "&#38;");
    res = res.replaceAll("<", "&#60;");
    res = res.replaceAll(">", "&#62;");
    return res;
  }

  /** Load the options file */
  public boolean load() {
    // Use an instance of ourselves as the SAX event handler
    DefaultHandler handler = new GanttXMLOptionsParser();

    // Use the default (non-validating) parser
    SAXParserFactory factory = SAXParserFactory.newInstance();
    try {
      File file = new File(System.getProperty("user.home") +
                           System.getProperty("file.separator") +
                           ".ganttproject");
      if (!file.exists()) {
        return false;
      }
      
      documentsMRU.clear();

      // Parse the input
      SAXParser saxParser = factory.newSAXParser();
      saxParser.parse(file, handler);
      
      GanttLookAndFeelInfo info = GanttLookAndFeels.getGanttLookAndFeels().getInfoByClass(styleClass);
      if (null == info)    info = GanttLookAndFeels.getGanttLookAndFeels().getInfoByName(styleName);
	  if (null != info)    lookAndFeel = info;
	  
	  if (null == lookAndFeel) lookAndFeel = GanttLookAndFeels.getGanttLookAndFeels().getDefaultInfo();

    }
    catch (Exception e) {
      e.printStackTrace();
      return false;
    }

    isloaded = true;
    return true;
  }

    public UIConfiguration getUIConfiguration() {
        if (myUIConfig==null) {
            myUIConfig =  new UIConfiguration(myMenuFont, myChartMainFont, color, redline);
        }
        return myUIConfig;
    }

    private Font myMenuFont;

    /** Class to parse the xml option file */
  class GanttXMLOptionsParser
      extends DefaultHandler {


        public void startElement(String namespaceURI,
                             String sName, // simple name
                             String qName, // qualified name
                             Attributes attrs) throws SAXException {

      int r = 0, g = 0, b = 0;

      if (attrs != null) {
        for (int i = 0; i < attrs.getLength(); i++) {
          String aName = attrs.getQName(i); // Attr name
          if (qName.equals("language")) {
            if (aName == "selection") {
              if (attrs.getValue(i).equals("English") ||
                  attrs.getValue(i).equals("en")) {
                language.setLocale(Locale.US);

              }
              else if (attrs.getValue(i).equals("Fran\u00e7ais") ||
                       attrs.getValue(i).equals("fr")) {
                language.setLocale(Locale.FRANCE);

              }
              else if (attrs.getValue(i).equals("Espa\u00f1ol") ||
                       attrs.getValue(i).equals("es")) {
                language.setLocale(new Locale("es", "ES"));

              }
              else if (attrs.getValue(i).equals("Portugues") ||
                       attrs.getValue(i).equals("pt")) {
                language.setLocale(new Locale("pt", "PT"));

              }
			  else if (attrs.getValue(i).equals("Portugu\u00eas do Brasil") ||
                       attrs.getValue(i).equals("pt_BR")) {
                language.setLocale(new Locale("pt", "BR"));

              }
              else if (attrs.getValue(i).equals("Deutsch") ||
                       attrs.getValue(i).equals("de")) {
                language.setLocale(Locale.GERMANY);

              }
              else if (attrs.getValue(i).equals("Norsk") ||
                       attrs.getValue(i).equals("no")) {
                language.setLocale(new Locale("no", "NO"));

              }
              else if (attrs.getValue(i).equals("Italiano") ||
                       attrs.getValue(i).equals("it")) {
                language.setLocale(Locale.ITALY);

              }
	      else if (attrs.getValue(i).equals("Japanese") ||
                       attrs.getValue(i).equals("jpn")) {
                language.setLocale(new Locale("ja", "JP"));

              }
              else if (attrs.getValue(i).equals("T\u00FCrk\u00E7e") ||
                       attrs.getValue(i).equals("tr")) {
                language.setLocale(new Locale("tr", "TR"));

              }
              else if (attrs.getValue(i).equals("ChineseGB") ||
                       attrs.getValue(i).equals("zhgb")) {
                language.setLocale(new Locale("zhgb", "CN"));

              }
              else if (attrs.getValue(i).equals("ChineseBig5") ||
                       attrs.getValue(i).equals("zh_TW")) {
                language.setLocale(new Locale("zh", "TW"));

              }
              else if (attrs.getValue(i).equals("Polski") ||
                       attrs.getValue(i).equals("pl")) {
                language.setLocale(new Locale("pl", "PL"));

              }
              else if (attrs.getValue(i).equals(
                  "\u0420\u0443\u0441\u0441\u043a\u0438\u0439") ||
                       attrs.getValue(i).equals("ru")) {
                language.setLocale(new Locale("ru", "RU"));

              }
              else if (attrs.getValue(i).equals("Estonian") ||
                       attrs.getValue(i).equals("et")) {
                language.setLocale(new Locale("et", "ET"));

              }
              else if (attrs.getValue(i).equals("Hungarian") ||
                       attrs.getValue(i).equals("hu")) {
                language.setLocale(new Locale("hu", "HU"));

              }
							else if (attrs.getValue(i).equals("\u05d0\u05e0\u05d2\u05dc\u05d9\u05ea") ||
                       attrs.getValue(i).equals("iw")) {
                language.setLocale(new Locale("iw", "IW"));

              }
			  else if (attrs.getValue(i).equals("Svenska") ||
                       attrs.getValue(i).equals("sv")) {
                language.setLocale(new Locale("sv", "SV"));

              }else if (attrs.getValue(i).equals("Nederlands") ||
                       attrs.getValue(i).equals("nl")) {
                language.setLocale(new Locale("nl", "NL"));

              }else if (attrs.getValue(i).equals("\u010cesky") ||
                       attrs.getValue(i).equals("cz")) {
                language.setLocale(new Locale("cz", "CZ"));

              }else if (attrs.getValue(i).equals("Dansk") ||
              		attrs.getValue(i).equals("da")) {
              	language.setLocale(new Locale("da", "DK"));

              }
            }
          }
          else if (qName.equals("task-color")) {
            if (aName.equals("red")) {
              r = new Integer(attrs.getValue(i)).hashCode();
            }
            else if (aName.equals("green")) {
              g = new Integer(attrs.getValue(i)).hashCode();
            }
            else if (aName.equals("blue")) {
              b = new Integer(attrs.getValue(i)).hashCode();

            }
          }
          else if (qName.equals("geometry")) {
            if (aName.equals("x")) {
              x = new Integer(attrs.getValue(i)).hashCode();
            }
            if (aName.equals("y")) {
              y = new Integer(attrs.getValue(i)).hashCode();
            }
            if (aName.equals("width")) {
              width = new Integer(attrs.getValue(i)).hashCode();
            }
            if (aName.equals("height")) {
              height = new Integer(attrs.getValue(i)).hashCode();

            }
          }
          else if (qName.equals("looknfeel")) {
            if (aName.equals("name")) {
              styleName = attrs.getValue(i);
            }
            if (aName.equals("class")) {
              styleClass = attrs.getValue(i);
            }
          }
          else if (qName.equals("file")) {
            if (aName.equals("path")) {
              documentsMRU.append(DocumentCreator.createDocument(attrs.getValue(i)));
            }
          }else if (qName.equals("automatic-launch")) {
            if (aName.equals("value")) {
              automatic = (new Boolean(attrs.getValue(i))).booleanValue() ;
            }
					}
					else if (qName.equals("dragTime")) {
            if (aName.equals("value")) {
              dragTime = (new Boolean(attrs.getValue(i))).booleanValue() ;
            }
          }else if (qName.equals("tips-on-startup")) {
            if (aName.equals("value")) {
              openTips = (new Boolean(attrs.getValue(i))).booleanValue() ;
            }
          }else if (qName.equals("redline")) {
            if (aName.equals("value")) {
              redline = (new Boolean(attrs.getValue(i))).booleanValue() ;
            }
          }else if (qName.equals("lockdavminutes")) {
            if (aName.equals("value")) {
                 lockDAVMinutes = (new Integer(attrs.getValue(i))).intValue();
            }
          }else if (qName.equals("xsl-dir")) {
            if (aName.equals("dir")) {
		      if(new File(attrs.getValue(i)).exists())
				  xslDir = attrs.getValue(i);
            }
		  }else if (qName.equals("xsl-fo")) {
            if (aName.equals("file")) {
		      if(new File(attrs.getValue(i)).exists())
				  xslFo = attrs.getValue(i);
            }
		  }else if (qName.equals("working-dir")) {
            if (aName.equals("dir")) {
		      if(new File(attrs.getValue(i)).exists())
				  workingDir = attrs.getValue(i);
            }
		  }
        }

      }

      if (qName.equals("task-color")) {
        color = new Color(r, g, b);

      }

      if (qName.equals("font")) {
          String category = attrs.getValue("category");
          if ("menu".equals(category)) {
              myMenuFont = Font.decode(attrs.getValue("spec"));
          }
          else if ("chart-main".equals(category)) {
              myChartMainFont = Font.decode(attrs.getValue("spec"));
          }

      }

      if(styleName.toUpperCase().equals("GTK")) {
        styleClass = "com.incors.plaf.kunststoff.KunststoffLookAndFeel";
        styleName = "Kunststoff";
      }
			
    }

  }

}
