
package net.sourceforge.ganttproject.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;

import java.security.AccessControlException;
import java.util.Enumeration;
import java.util.Vector;
import java.util.*;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextPane;
import javax.swing.SwingConstants;
import javax.swing.WindowConstants;
import javax.swing.table.AbstractTableModel;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.Document;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;

import net.sourceforge.ganttproject.GanttProject;
import net.sourceforge.ganttproject.language.GanttLanguage;

/**
  * The About dialog box of ganttproject
  */
public class AboutGanttProject extends JDialog
{
  
  private GanttLanguage language = GanttLanguage.getInstance();
  
  /** Components **/
  // Labels
  JLabel jLabelImage = new JLabel(new ImageIcon(getClass().getResource("/icons/splash.gif"))) {
	public void paint (Graphics g) {
    		super.paint(g);
    		Graphics2D g2 = (Graphics2D) g;
	        g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
	        Font font = new Font("Arial", Font.BOLD + Font.ITALIC, 22);
	        g2.setFont(font);
	        g2.drawString(GanttProject.version, (int) getSize().getWidth() - 170, 65);
	}	
  };
  JLabel jLabelAknowlegement = new JLabel (new ImageIcon(getClass().getResource("/icons/aknowlegement.gif")));

  // Buttons
  JButton jButtonOK = new JButton(GanttLanguage.getInstance().getText("close"));
  
  // Tables
  JTable jTableProperties = new JTable();
  JTable jTableAuthors = new JTable();
	//JTextArea jTextAreaLicense = new JTextArea();
	JTextPane jTextAreaLicense = new JTextPane();
  
  AboutFieldTableModel modelproperties = new AboutFieldTableModel();
  AuthorsFieldTableModel modelauthors = new AuthorsFieldTableModel();
  
  // Panels
  JTabbedPane jTabbedPaneAboutGP = new JTabbedPane(SwingConstants.TOP);
  JPanel jPanelAbout = new JPanel(new BorderLayout());
  JPanel jPanelAuthors= new JPanel(new BorderLayout());
  JPanel jPanelInfos = new JPanel(new BorderLayout());
  JPanel jPanelLicense = new JPanel(new BorderLayout());
  
	
  JScrollPane jScrollPaneProperties = new JScrollPane(jTableProperties);
  JScrollPane jScrollPaneAuthors = new JScrollPane(jTableAuthors);
	JScrollPane jScrollPaneLicense = new JScrollPane(jTextAreaLicense);

  public AboutGanttProject(JFrame parent)
  {
    /** Initialization **/
    super(parent, GanttProject.correctLabel(GanttLanguage.getInstance().getText("about"))+" - Ganttproject");
    // Define the size of the window
    this.setSize(new Dimension(570, 443));
    // Center the window
    this.setLocationRelativeTo(null);
    // Specify that the window is modal
    this.setModal(true);
    // Specify that the window is not resizable
    this.setResizable(true);
    // Set 'jButtonOK' as the default button
    this.getRootPane().setDefaultButton(jButtonOK);
    // Define what to do when closing this window using
    this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

    /** Components properties **/
    // Panels
    this.getContentPane().setLayout(new GridBagLayout());
    // Tables
    jTableProperties.setModel(modelproperties);
    try {
        Enumeration enum = System.getProperties().propertyNames();
        SortedSet s = new TreeSet();
        while(enum.hasMoreElements()) {
            s.add((String)enum.nextElement());
        }
        Iterator i = s.iterator();
        while (i.hasNext()) {
            String prop = (String)i.next();
            modelproperties.addField(new SysInfo(prop, System.getProperty(prop)));
        }
    } catch (AccessControlException e) {
	    // This can happen when running in a sandbox (Java WebStart)
	    System.err.println (e + ": " + e.getMessage());
    }
    jTableProperties.getColumnModel().getColumn(0).setPreferredWidth(170);
    jTableProperties.getColumnModel().getColumn(1).setPreferredWidth(380);
		
		jTableAuthors.setModel(modelauthors);
    modelauthors.addField(new SysInfo("THOMAS Alexandre","alexthomas@ganttproject.org"));
    modelauthors.addField(new SysInfo("BARMEIER Matthias","matthias.barmeier@sourcepark.de"));
    modelauthors.addField(new SysInfo("LU Cui","cuix1@yahoo.com"));
    modelauthors.addField(new SysInfo("BARASHEV Dmitry","dmitry.barashev@borland.com"));
    modelauthors.addField(new SysInfo("SCHULZ Maik","post@maikschulz.de"));
    modelauthors.addField(new SysInfo("HAEUSLER Michael","michael@akatose.de"));
    modelauthors.addField(new SysInfo("LIPINSKI Pawel","pawel.lipinski@javart.com.pl"));
		
    modelauthors.addField(new SysInfo("",""));
    modelauthors.addField(new SysInfo("ANDRESEN Roger","Roger@Andresen.com"));
    modelauthors.addField(new SysInfo("AUDRU Cdric","cedricaudru@yahoo.fr"));
    modelauthors.addField(new SysInfo("BALAZS Major","BMajor@Graphisoft.hu"));
    modelauthors.addField(new SysInfo("BREZINA Marek","marek.brezina@seznam.cz"));
    modelauthors.addField(new SysInfo("BROKER Rick","rick_broker@yahoo.com"));
    modelauthors.addField(new SysInfo("CROUNSE Brian","brian@crounse.name"));
    modelauthors.addField(new SysInfo("FERRAZ Nelson","nferraz@phperl.com"));
		modelauthors.addField(new SysInfo("GLOEGL Michael","gloegl@fmi.uni-passau.de"));
    modelauthors.addField(new SysInfo("HERRMANN G.","herr@nike.eonet.ne.jp"));
    modelauthors.addField(new SysInfo("HURSEY John","joshh@cs.earlham.edu"));
    modelauthors.addField(new SysInfo("KARLGREN Jussi","jussi@sics.se"));
    modelauthors.addField(new SysInfo("KITSIK Ahti","ahti@smarthoster.com"));
    modelauthors.addField(new SysInfo("LIN Kirin","kirinlin@users.sourceforge.net"));
    modelauthors.addField(new SysInfo("L'KENFACK Etienne","etiennelk@hotmail.com"));
    modelauthors.addField(new SysInfo("MARCH Stephen","steve@enerds.ca"));
    modelauthors.addField(new SysInfo("MIYATA Yasuhiro","yasuhiro.miyata@ulsystems.co.jp"));
    modelauthors.addField(new SysInfo("NATHANAEL Uwe","Uwe.Nathanael@t-online.de"));
    modelauthors.addField(new SysInfo("OGNESS John","john@ogness.net"));
    modelauthors.addField(new SysInfo("PAOLETTI Tomaso","tom@ipaoletti.net"));
    modelauthors.addField(new SysInfo("PLUSCHKE Andreas","homepage.plueschke@gmx.de"));
    modelauthors.addField(new SysInfo("RACINOWSKI Przemyslaw","p.racinowski@wp.pl"));
    modelauthors.addField(new SysInfo("REY Juan","juanrey@inicia.es"));
    modelauthors.addField(new SysInfo("SAHIN Cengiz","cengiz@sahinc.de"));
    modelauthors.addField(new SysInfo("SENIGAGLIESI Paolo","senigagliesi_inf@hotmail.com"));
    modelauthors.addField(new SysInfo("SHABTAI Yoav","yoavs@pmp-medical.com"));
    modelauthors.addField(new SysInfo("STAVRIDES Paul","pstav@adelie.net"));
    modelauthors.addField(new SysInfo("VAN DER WIEL Andre","a.vd.wiel@chello.nl"));
    modelauthors.addField(new SysInfo("VOCI Elio","elio.voci@gawab.com"));
    modelauthors.addField(new SysInfo("ZAVOLZHSKY Alexandr","zavolzhsky@mail.ru"));

    /** Listeners **/
    // Action listeners
    jButtonOK.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        dispose();
      }
    });

    /** Assembling components **/
    // jPanelAbout
    jPanelAbout.add(jLabelImage,BorderLayout.CENTER);
    jPanelAbout.add(jLabelAknowlegement,BorderLayout.SOUTH);
    // jPanelAuthors
    jPanelAuthors.add(jScrollPaneAuthors);
    // jPanelInfos
    jPanelInfos.add(jScrollPaneProperties);
		// jPanelInfos
    jPanelLicense.add(jScrollPaneLicense);
   // jTabbedPaneAboutGP
    jTabbedPaneAboutGP.add(jPanelAbout, GanttProject.correctLabel(language.getText("about")));
    jTabbedPaneAboutGP.add(jPanelAuthors, language.getText("authors"));
    jTabbedPaneAboutGP.add(jPanelInfos, language.getText("infos"));
    jTabbedPaneAboutGP.add(jPanelLicense, language.getText("license")	);
		jTabbedPaneAboutGP.setTabLayoutPolicy(JTabbedPane.SCROLL_TAB_LAYOUT);
    // Content Panel
    this.getContentPane().add(jTabbedPaneAboutGP,
                              new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.BOTH,
        new Insets(5, 5, 0, 5), 0, 0));
    this.getContentPane().add(jButtonOK,
                              new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
        GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5),
        0, 0));
	
	
	 //License text
			StringBuffer text=new StringBuffer();
		text.append("This program is free software; you can redistribute it and/or modify it under the terms of the GNU General ");
		text.append("Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.\n\n");
		text.append("This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied ");
		text.append("warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.\n\n");
		text.append("You should have received a copy of the GNU General Public License along with this program; if not, write to the Free ");
		text.append("Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n");
		text.append("Browse the entire GPL license at http://www.gnu.org/copyleft/gpl.html\n");
		
		jTextAreaLicense.setEditable(false);
		Style def = StyleContext.getDefaultStyleContext().getStyle(StyleContext.DEFAULT_STYLE);	 
	 //Default style for text
	 Style regular = jTextAreaLicense.addStyle("regular", def);
	 StyleConstants.setFontSize(regular, 14);
   StyleConstants.setFontFamily(def, "SansSerif");	 
	 //Bold style for "Title"
	 Style s = jTextAreaLicense.addStyle("bold", regular);
	 StyleConstants.setFontSize(s, 18);
   StyleConstants.setItalic(s, true);
	 String[] initString =
                { "GanttProject is under the GPL License.\n\n",		
                  text.toString()					
                 };

   String[] initStyles =  { "bold", "regular"};
	 Document doc = new DefaultStyledDocument() ;
	 //Insert text on the panel
	 try {
            for (int i=0; i < initString.length; i++) {
                doc.insertString(doc.getLength(), initString[i],
                                 jTextAreaLicense.getStyle(initStyles[i]));
            }
						jTextAreaLicense.setDocument(doc);
        } catch (BadLocationException ble) {
            System.err.println("Couldn't insert initial text.");
        }
		
	
		
		//Set Position
   Point point = parent.getLocationOnScreen();
   int x = (int) point.getX() + parent.getWidth() / 2;
   int y = (int) point.getY() + parent.getHeight() / 2;
   setLocation(x - getWidth() / 2, y - getHeight() / 2);	
	
	
  }
  
  
										  
}

/**
 * <p>Title: SysInfo</p>
 */
class SysInfo
{
  private String name;
  private String value;

  public SysInfo(String name, String value)
  {
    this.name = name;
    this.value = value;
  }

  public String getName()
  {
    return name;
  }

  public String getValue()
  {
    return value;
  }
}


class AboutFieldTableModel extends AbstractTableModel
{
  private GanttLanguage language = GanttLanguage.getInstance();
  
  final String[] columnNames =
                               {language.getText("name"), language.getText("value")};
  final Class[] columnClasses =
                                {String.class, String.class};
  Vector data = new Vector();

  public void addField(SysInfo w)
  {
    data.addElement(w);
    fireTableRowsInserted(data.size() - 1, data.size() - 1);
  }

  public int getColumnCount()
  {
    return columnNames.length;
  }

  public int getRowCount()
  {
    return data.size();
  }

  public String getColumnName(int col)
  {
    return columnNames[col];
  }

  public Class getColumnClass(int c)
  {
    return columnClasses[c];
  }

  public Object getValueAt(int row, int col)
  {
    SysInfo info = (SysInfo)data.elementAt(row);
    if(col == 0)
    {
      return info.getName();
    }
    else if(col == 1)
    {
      return info.getValue();
    }
    else
    {
      return null;
    }
  }

  public boolean isCellEditable(int row, int col)
  {
    return false;
  }
} 
 

class AuthorsFieldTableModel extends AbstractTableModel
{
 private GanttLanguage language = GanttLanguage.getInstance();
  
  final String[] columnNames =
                               {language.getText("name"), language.getText("colMail")};
  final Class[] columnClasses =
                                {String.class, String.class};
  Vector data = new Vector();

  public void addField(SysInfo w)
  {
    data.addElement(w);
    fireTableRowsInserted(data.size() - 1, data.size() - 1);
  }

  public int getColumnCount()
  {
    return columnNames.length;
  }

  public int getRowCount()
  {
    return data.size();
  }

  public String getColumnName(int col)
  {
    return columnNames[col];
  }

  public Class getColumnClass(int c)
  {
    return columnClasses[c];
  }

  public Object getValueAt(int row, int col)
  {
    SysInfo info = (SysInfo)data.elementAt(row);
    if(col == 0)
    {
      return info.getName();
    }
    else if(col == 1)
    {
      return info.getValue();
    }
    else
    {
      return null;
    }
  }

  public boolean isCellEditable(int row, int col)
  {
    return false;
  }
 }
  
