package net.sourceforge.ganttproject.time;

import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Calendar;
import java.util.HashMap;

/**
 * Created by IntelliJ IDEA.
 * @author bard
 * Date: 01.02.2004
 */
public class GregorianTimeUnitStack {
    private static TimeUnitGraph ourGraph = new TimeUnitGraph();
    public static final TimeUnit SECOND = ourGraph.createAtomTimeUnit("second");
    public static final TimeUnit MINUTE = ourGraph.createTimeUnit("minute", SECOND, 60);
    public static final TimeUnit HOUR = ourGraph.createTimeUnit("hour", MINUTE, 60);
    public static final TimeUnit DAY = ourGraph.createTimeUnit("day", HOUR, 24);
    private static final TimeUnit ATOM_UNIT = SECOND;
    private static final HashMap ourUnit2field = new HashMap();
    static {
        ourUnit2field.put(DAY, new Integer(Calendar.DAY_OF_MONTH));
        ourUnit2field.put(HOUR, new Integer(Calendar.HOUR_OF_DAY));
        ourUnit2field.put(MINUTE, new Integer(Calendar.MINUTE));
        ourUnit2field.put(SECOND, new Integer(Calendar.SECOND));
    }
    public GregorianTimeUnitStack () {

    }

    public TimeFrame createTimeFrame(Date startDate, TimeUnit topUnit, TimeUnit bottomUnit) {
        return new TimeFrameImpl(startDate, topUnit, bottomUnit);
    }


    private static class TimeFrameImpl implements TimeFrame {
        private Date myStartDate;
        private TimeUnit myTopUnit;
        private TimeUnit myBottomUnit;
        private Date myNextFrameStartDate;
        private GregorianCalendar myCalendar;

        TimeFrameImpl(Date startDate, TimeUnit topUnit, TimeUnit bottomUnit) {
            if (!topUnit.isConstructedFrom(ATOM_UNIT)) {
                throw new RuntimeException("Unknown time unit: "+topUnit);
            }
            if (!bottomUnit.isConstructedFrom(ATOM_UNIT)) {
                throw new RuntimeException("Unknown time unit: "+bottomUnit);
            }
            if (!topUnit.isConstructedFrom(bottomUnit)) {
                throw new RuntimeException("Top unit="+topUnit+" is not constructed from bottom unit="+bottomUnit);
            }
            myStartDate = startDate;
            myTopUnit = topUnit;
            myBottomUnit = bottomUnit;
            myCalendar = (GregorianCalendar) GregorianCalendar.getInstance().clone();
            myCalendar.setTime(myStartDate);
            myNextFrameStartDate = shiftDate(myStartDate, myTopUnit, 1);
        }

        private Date shiftDate(Date currentDate, TimeUnit timeUnit, int unitCount) {
            Calendar c = (Calendar) myCalendar.clone();
            c.setTime(currentDate);
            int calendarField = getCalendarField(timeUnit);
            int currentValue = c.get(calendarField);
            clearFields(c, timeUnit);
            c.add(calendarField, unitCount);
            return c.getTime();
        }

        private void clearFields(Calendar c, TimeUnit topUnit) {
            for (TimeUnit currentUnit = topUnit; currentUnit!=null; currentUnit = currentUnit.getDirectAtomUnit()) {
                int calendarField = getCalendarField(currentUnit);
                c.clear(calendarField);
                c.getTime();
            }
        }

        private int getCalendarField(TimeUnit timeUnit) {
            Integer field = (Integer) ourUnit2field.get(timeUnit);
            return field.intValue();
        }

        public Date getStartDate() {
            return myStartDate;
        }

        public TimeUnit getTopUnit() {
            return myTopUnit;
        }

        public TimeUnit getBottomUnit() {
            return myBottomUnit;
        }

        public int getUnitCount(TimeUnit unitLine) {
            int counter = 0;
            for (Date nextUnitStart = shiftDate(myStartDate, unitLine, counter);
                 nextUnitStart.before(myNextFrameStartDate);
                 nextUnitStart = shiftDate(myStartDate, unitLine, ++counter)) {
                //System.err.println("myStart="+myStartDate+" nextFrame="+myNextFrameStartDate+" nextUnitStart="+nextUnitStart);
            }
            return counter;
        }

        public String getUnitText(TimeUnit unitLine, int position) {
            return null;  //To change body of implemented methods use Options | File Templates.
        }

        public Date getUnitStart(TimeUnit unitLine, int position) {
            return null;  //To change body of implemented methods use Options | File Templates.
        }

    }
}
