; The name of the installer
Name "GanttProject"

!define VERSION "2.0-pre1"
!define VM_ARGS "-Xmx512m -Xms128m"

!ifdef VERSION
OutFile installer-ganttproject-${VERSION}.exe
!else
OutFile installer-ganttproject.exe
!endif


; The default installation directory
InstallDir $PROGRAMFILES\GanttProject


LicenseText "GanttProject License"
LicenseData "doc\COPYING"

Page license
Page components
Page directory
Page instfiles

UninstPage uninstConfirm
UninstPage instfiles


Section "GanttProject"

  SectionIn RO ; read-only
  
  ; Set output path to the installation directory.
  SetOutPath $INSTDIR


  SearchPath $2 javaw.exe
  DetailPrint "javaw is located there: $2"

  
  ; Put file there
  File ganttproject_16.ico
  File ganttproject_32_2.ico
  File eclipsito.jar
  File "ganttproject-eclipsito-config.xml"
  File /r doc

  StrCpy $OUTDIR "$INSTDIR\plugins"
  File /r plugins\net.sourceforge.ganttproject_2.0.0
  SetOutPath $INSTDIR

  ; Write the installation path into the registry
  WriteRegStr HKLM SOFTWARE\GanttProject "Install_Dir" "$INSTDIR"
  
  ; Write the uninstall keys for Windows
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\GanttProject" "DisplayName" "GanttProject"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\GanttProject" "UninstallString" '"$INSTDIR\uninstall.exe"'
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\GanttProject" "NoModify" 1
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\GanttProject" "NoRepair" 1
  WriteUninstaller "uninstall.exe"
  
SectionEnd

Section "MS-Project Import/Export plug-in"

  SetOutPath $INSTDIR
  StrCpy $OUTDIR "$INSTDIR\plugins"
  File /r plugins\org.ganttproject.impex.msproject_2.0.0
  SetOutPath $INSTDIR
SectionEnd

Section "PERT network view plug-in"

  SetOutPath $INSTDIR
  StrCpy $OUTDIR "$INSTDIR\plugins"
  File /r plugins\org.ganttproject.chart.pert_1.0.0
  SetOutPath $INSTDIR

SectionEnd

; to uncheck a section by default use Section /o "HTML export plug-in"
Section "HTML export plug-in"

  SetOutPath $INSTDIR
  StrCpy $OUTDIR "$INSTDIR\plugins"
  File /r plugins\org.ganttproject.impex.htmlpdf_2.0.0
  SetOutPath $INSTDIR

SectionEnd

; Optional section (can be disabled by the user)
Section "Start Menu Shortcuts"

  CreateDirectory "$SMPROGRAMS\GanttProject"
  CreateShortCut "$SMPROGRAMS\GanttProject\Uninstall.lnk" "$INSTDIR\uninstall.exe" "" "$INSTDIR\uninstall.exe" 0
  CreateShortCut "$SMPROGRAMS\GanttProject\GanttProject.lnk" "$2" "-classpath eclipsito.jar org.bardsoftware.eclipsito.Boot ganttproject-eclipsito-config.xml" "$INSTDIR\ganttproject_16.ico"
  CreateShortCut "$DESKTOP\GanttProject.lnk" "$2" "${VM_ARGS} -classpath eclipsito.jar org.bardsoftware.eclipsito.Boot ganttproject-eclipsito-config.xml" "$INSTDIR\ganttproject_32_2.ico"


SectionEnd

;--------------------------------

; Uninstaller

Section "Uninstall"

  ; Remove registry keys
  DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\GanttProject"
  DeleteRegKey HKLM SOFTWARE\GanttProject

  ; Remove files and uninstaller
  Delete $INSTDIR\uninstall.exe

  Delete $INSTDIR\eclipsito.jar
  Delete $INSTDIR\ganttproject-eclipsito-config.xml
  Delete $INSTDIR\ganttproject_16.ico
  Delete $INSTDIR\ganttproject_32_2.ico
  RMDir /r "$INSTDIR\plugins"  
  RMDir /r "$INSTDIR\doc"


  ; Remove shortcuts, if any
  Delete "$SMPROGRAMS\GanttProject\*.*"
  Delete "$DESKTOP\GanttProject.lnk"

  ; Remove directories used
  RMDir "$SMPROGRAMS\GanttProject"
  RMDir "$INSTDIR"

SectionEnd