/***************************************************************************
 GanttProject.java  -  description
 -------------------
 begin                : dec 2002
 copyright            : (C) 2002 by Thomas Alexandre
 email                : alexthomas(at)ganttproject.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

package net.sourceforge.ganttproject;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.ComponentOrientation;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowListener;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.security.AccessControlException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.AbstractButton;
import javax.swing.Action;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.ToolTipManager;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import net.sourceforge.ganttproject.GanttGraphicArea.MouseSupport;
import net.sourceforge.ganttproject.action.CalculateCriticalPathAction;
import net.sourceforge.ganttproject.action.CopyAction;
import net.sourceforge.ganttproject.action.CutAction;
import net.sourceforge.ganttproject.action.DeleteAssignmentAction;
import net.sourceforge.ganttproject.action.DeleteHumanAction;
import net.sourceforge.ganttproject.action.GPAction;
import net.sourceforge.ganttproject.action.ImportResources;
import net.sourceforge.ganttproject.action.NewArtefactAction;
import net.sourceforge.ganttproject.action.NewHumanAction;
import net.sourceforge.ganttproject.action.NewTaskAction;
import net.sourceforge.ganttproject.action.PasteAction;
import net.sourceforge.ganttproject.action.RedoAction;
import net.sourceforge.ganttproject.action.RefreshViewAction;
import net.sourceforge.ganttproject.action.ResourceActionSet;
import net.sourceforge.ganttproject.action.RolloverAction;
import net.sourceforge.ganttproject.action.ScrollGanttChartLeftAction;
import net.sourceforge.ganttproject.action.ScrollGanttChartRightAction;
import net.sourceforge.ganttproject.action.SwitchViewAction;
import net.sourceforge.ganttproject.action.UndoAction;
import net.sourceforge.ganttproject.action.ZoomInAction;
import net.sourceforge.ganttproject.action.ZoomOutAction;
import net.sourceforge.ganttproject.calendar.GPCalendar;
import net.sourceforge.ganttproject.calendar.WeekendCalendarImpl;
import net.sourceforge.ganttproject.chart.Chart;
import net.sourceforge.ganttproject.chart.ChartViewState;
import net.sourceforge.ganttproject.chart.ToggleChartAction;
import net.sourceforge.ganttproject.delay.DelayManager;
import net.sourceforge.ganttproject.document.AbstractURLDocument;
import net.sourceforge.ganttproject.document.Document;
import net.sourceforge.ganttproject.document.DocumentCreator;
import net.sourceforge.ganttproject.document.DocumentManager;
import net.sourceforge.ganttproject.document.DocumentsMRU;
import net.sourceforge.ganttproject.document.FileDocument;
import net.sourceforge.ganttproject.document.HttpDocument;
import net.sourceforge.ganttproject.document.OpenDocumentAction;
import net.sourceforge.ganttproject.export.ExportFileAction;
import net.sourceforge.ganttproject.export.ExportFileWizardImpl;
import net.sourceforge.ganttproject.filter.GanttCSVFilter;
import net.sourceforge.ganttproject.filter.GanttHTMLFileFilter;
import net.sourceforge.ganttproject.filter.GanttJPGFileFilter;
import net.sourceforge.ganttproject.filter.GanttMSProjectLFileFilter;
import net.sourceforge.ganttproject.filter.GanttPDFFileFilter;
import net.sourceforge.ganttproject.filter.GanttPNGFileFilter;
import net.sourceforge.ganttproject.filter.GanttXFIGFileFilter;
import net.sourceforge.ganttproject.filter.GanttXMLFileFilter;
import net.sourceforge.ganttproject.gui.GanttDialogCalendar;
import net.sourceforge.ganttproject.gui.GanttDialogInfo;
import net.sourceforge.ganttproject.gui.GanttDialogPerson;
import net.sourceforge.ganttproject.gui.GanttDialogProperties;
import net.sourceforge.ganttproject.gui.GanttLookAndFeelInfo;
import net.sourceforge.ganttproject.gui.GanttLookAndFeels;
import net.sourceforge.ganttproject.gui.GanttStatusBar;
import net.sourceforge.ganttproject.gui.GanttURLChooser;
import net.sourceforge.ganttproject.gui.TestGanttRolloverButton;
import net.sourceforge.ganttproject.gui.TipsDialog;
import net.sourceforge.ganttproject.gui.UIConfiguration;
import net.sourceforge.ganttproject.gui.UIFacade;
import net.sourceforge.ganttproject.gui.about.AboutDialog;
import net.sourceforge.ganttproject.gui.options.SettingsDialog;
import net.sourceforge.ganttproject.gui.previousState.GanttDialogCompareToPreviousState;
import net.sourceforge.ganttproject.gui.previousState.GanttDialogSaveAsPreviousState;
import net.sourceforge.ganttproject.gui.projectwizard.NewProjectWizard;
import net.sourceforge.ganttproject.gui.scrolling.ScrollingManager;
import net.sourceforge.ganttproject.gui.scrolling.ScrollingManagerImpl;
import net.sourceforge.ganttproject.gui.zoom.ZoomManager;
import net.sourceforge.ganttproject.importer.ImportFileAction;
import net.sourceforge.ganttproject.importer.ImportFileWizardImpl;
import net.sourceforge.ganttproject.io.GPSaver;
import net.sourceforge.ganttproject.io.GanttXMLOpen;
import net.sourceforge.ganttproject.io.GanttXMLSaver;
import net.sourceforge.ganttproject.language.GanttLanguage;
import net.sourceforge.ganttproject.parser.AllocationTagHandler;
import net.sourceforge.ganttproject.parser.BlankLineTagHandler;
import net.sourceforge.ganttproject.parser.CustomPropertiesTagHandler;
import net.sourceforge.ganttproject.parser.DefaultWeekTagHandler;
import net.sourceforge.ganttproject.parser.DependencyTagHandler;
import net.sourceforge.ganttproject.parser.GPParser;
import net.sourceforge.ganttproject.parser.HolidayTagHandler;
import net.sourceforge.ganttproject.parser.ParserFactory;
import net.sourceforge.ganttproject.parser.PreviousStateTasksTagHandler;
import net.sourceforge.ganttproject.parser.ResourceTagHandler;
import net.sourceforge.ganttproject.parser.RoleTagHandler;
import net.sourceforge.ganttproject.parser.TaskDisplayColumnsTagHandler;
import net.sourceforge.ganttproject.parser.TaskPropertiesTagHandler;
import net.sourceforge.ganttproject.parser.TaskTagHandler;
import net.sourceforge.ganttproject.parser.VacationTagHandler;
import net.sourceforge.ganttproject.parser.ViewTagHandler;
import net.sourceforge.ganttproject.print.PrintManager;
import net.sourceforge.ganttproject.print.PrintPreview;
import net.sourceforge.ganttproject.resource.AssignmentContext;
import net.sourceforge.ganttproject.resource.HumanResource;
import net.sourceforge.ganttproject.resource.HumanResourceManager;
import net.sourceforge.ganttproject.resource.ProjectResource;
import net.sourceforge.ganttproject.resource.ResourceContext;
import net.sourceforge.ganttproject.resource.ResourceEvent;
import net.sourceforge.ganttproject.resource.ResourceManager;
import net.sourceforge.ganttproject.resource.ResourceView;
import net.sourceforge.ganttproject.roles.RoleManager;
import net.sourceforge.ganttproject.task.BlankLineNode;
import net.sourceforge.ganttproject.task.CustomColumnsStorage;
import net.sourceforge.ganttproject.task.Task;
import net.sourceforge.ganttproject.task.TaskContainmentHierarchyFacade;
import net.sourceforge.ganttproject.task.TaskManager;
import net.sourceforge.ganttproject.task.TaskManagerConfig;
import net.sourceforge.ganttproject.task.TaskNode;
import net.sourceforge.ganttproject.task.TaskSelectionManager;
import net.sourceforge.ganttproject.task.algorithm.AdjustTaskBoundsAlgorithm;
import net.sourceforge.ganttproject.task.algorithm.RecalculateTaskCompletionPercentageAlgorithm;
import net.sourceforge.ganttproject.task.dependency.TaskDependencyException;
import net.sourceforge.ganttproject.task.dependency.constraint.FinishStartConstraintImpl;
import net.sourceforge.ganttproject.time.TimeUnitStack;
import net.sourceforge.ganttproject.time.gregorian.GPTimeUnitStack;
import net.sourceforge.ganttproject.undo.GPUndoManager;
import net.sourceforge.ganttproject.undo.UndoManagerImpl;
import net.sourceforge.ganttproject.util.BrowserControl;

/**
 * Main frame of the project
 */
public class GanttProject extends GanttProjectBase implements ActionListener,
        IGanttProject, ResourceView, KeyListener, UIFacade {

    /** The current version of ganttproject */
    public static final String version = "2.0-pre1";

    /* List of calendar for the project */
    private ArrayList listOfCalendar = new ArrayList();

    /** Command line creation or gui creation */
    public static boolean byCommandLine = false;

    /** The language use */
    private GanttLanguage language = GanttLanguage.getInstance();

    /** The JTree part. */
    private GanttTree2 tree;

    /** GanttGraphicArea for the calendar with Gantt */
    private GanttGraphicArea area;

    /** GanttPeoplePanel to edit person that work on the project */
    private GanttResourcePanel resp;

    /** The differents menus */
    public JMenu mProject, mMRU, mEdit, mTask, mHuman, mHelp, mServer,
            mCalendar;

    // public JMenu mView;

    /** The differetns menuitem */
    public JMenuItem miNew, miOpen, miOpenURL, miSave, miSaveAs, miSaveAsURL,
            miPublishToWeb, /* miOpenDB,miSaveAsDB, */
            miPrint, miPreview, miQuit, miCut, miCopy, miPaste, miOptions,
            miDeleteTask, miPropertiesTask, miUp, miDown, miDelHuman,
            miPropHuman, miSendMailHuman, miEditCalendar, miPrjCal, miWebPage,
            miTips, miAbout, miManual, miRefresh, miChartOptions;

    public JMenuItem miUndo, miRedo;

    private static final int maxSizeMRU = 5;

    private DocumentsMRU documentsMRU = new DocumentsMRU(maxSizeMRU);

    /** The differents button of toolbar */
    private TestGanttRolloverButton bExit, bNew, bOpen, bSave, bSaveAs,
            bExport, bImport, bPrint, bPreviewPrint, bCopy, bCut, bPaste,
            bNewTask, bDelete, bProperties, bUnlink, bLink, bInd, bUnind, bUp,
            bDown, bPrev, bScrollCenter, bNext, bZoomFit, bAbout;

    private TestGanttRolloverButton bShowHiddens;

    private JPopupMenu menu = new JPopupMenu();;

    TestGanttRolloverButton bZoomIn, bZoomOut;

    private TestGanttRolloverButton bUndo, bRedo;

    private TestGanttRolloverButton bCritical;

    private TestGanttRolloverButton bSaveCurrent, bComparePrev;

    private TestGanttRolloverButton bRefresh;

    /** The project filename */
    public Document projectDocument = null;

    /** Informations for the current project. */
    public PrjInfos prjInfos = new PrjInfos();

    /** Boolean to know if the file has been modify */
    public boolean askForSave = false;

    /** The info for the look'n'feel */
    public GanttLookAndFeelInfo lookAndFeel;

    /** Is the application only for viewer. */
    public boolean isOnlyViewer;

    /** The list of all managers installed in this project */
    private Hashtable managerHash = new Hashtable();

    private ResourceActionSet myResourceActions;

    private boolean isApplet;

    /** Frame for the help Viewer */
    private JFrame helpFrame = null;

    private final TaskManager myTaskManager;

    private FacadeInvalidator myFacadeInvalidator;

    private UIConfiguration myUIConfiguration;

    private static class TaskContainmentHierarchyFacadeImpl implements
            TaskContainmentHierarchyFacade {
        private Map myTask2treeNode = new HashMap();

        private Task myRootTask;

        private List myPathBuffer = new ArrayList();

        private GanttTree2 myTree;

        public TaskContainmentHierarchyFacadeImpl(GanttTree2 tree) {
            ArrayList allTasks = ((GanttTree2) tree).getAllTasks();
            // comboBox.addItem("no set");
            // for (int i = 0; i < allTasks.size(); i++) {
            // DefaultMutableTreeNode treeNode = (DefaultMutableTreeNode)
            // allTasks.get(i);
            for (Iterator it = allTasks.iterator(); it.hasNext();) {
                TaskNode treeNode = (TaskNode) it.next();
                Task task = (Task) treeNode.getUserObject();
                if (treeNode.isRoot()) {
                    myRootTask = task;
                }
                myTask2treeNode.put(task, treeNode);
            }
            myTree = tree;
        }

        public Task[] getNestedTasks(Task container) {
            Task[] result = null;
            DefaultMutableTreeNode treeNode = (DefaultMutableTreeNode) myTask2treeNode
                    .get(container);
            if (treeNode != null) {
                ArrayList list = new ArrayList();
                for (Enumeration children = treeNode.children(); children
                        .hasMoreElements();) {
                    DefaultMutableTreeNode next = (DefaultMutableTreeNode) children
                            .nextElement();
                    if (next instanceof TaskNode)
                        list.add(next.getUserObject());
                }
                result = (Task[]) list.toArray(new Task[0]);
            }
            return result == null ? new Task[0] : result;
        }

        /**
         * Purpose: Returns true if the container Task has any nested tasks.
         * This should be a quicker check than using getNestedTasks().
         * 
         * @param container
         *            The Task on which to check for children.
         */
        public boolean hasNestedTasks(Task container) {
            DefaultMutableTreeNode treeNode = (DefaultMutableTreeNode) myTask2treeNode
                    .get(container);
            if (treeNode != null) {
                if (treeNode.children().hasMoreElements()) {
                    return true;
                }
            }
            return false;
        }

        public Task getRootTask() {
            return myRootTask;
        }

        public Task getContainer(Task nestedTask) {
            DefaultMutableTreeNode treeNode = (DefaultMutableTreeNode) myTask2treeNode
                    .get(nestedTask);
            if (treeNode == null) {
                return null;
            }
            DefaultMutableTreeNode containerNode = (DefaultMutableTreeNode) treeNode
                    .getParent();
            return containerNode == null ? null : (Task) containerNode
                    .getUserObject();
        }

        public boolean areUnrelated(Task first, Task second) {
            myPathBuffer.clear();
            for (Task container = getContainer(first); container != null; container = getContainer(container)) {
                myPathBuffer.add(container);
            }
            if (myPathBuffer.contains(second)) {
                return false;
            }
            myPathBuffer.clear();
            for (Task container = getContainer(second); container != null; container = getContainer(container)) {
                myPathBuffer.add(container);
            }
            if (myPathBuffer.contains(first)) {
                return false;
            }
            return true;
        }

        public void move(Task whatMove, Task whereMove) {
            DefaultMutableTreeNode targetNode = (DefaultMutableTreeNode) myTask2treeNode
                    .get(whereMove);
            DefaultMutableTreeNode movedNode = (DefaultMutableTreeNode) myTask2treeNode
                    .get(whatMove);
            if (movedNode != null) {
                TreePath movedPath = new TreePath(movedNode.getPath());
                boolean wasSelected = (myTree.getJTree().getSelectionModel()
                        .isPathSelected(movedPath));
                if (wasSelected) {
                    myTree.getJTree().getSelectionModel().removeSelectionPath(
                            movedPath);
                }
                myTree.getModel().removeNodeFromParent(movedNode);
                myTree.getModel().insertNodeInto(movedNode, targetNode,
                        targetNode.getChildCount());
                if (wasSelected) {
                    movedPath = new TreePath(movedNode.getPath());
                    myTree.getJTree().getSelectionModel().addSelectionPath(
                            movedPath);
                }

            } else {
                myTree.addObjectWithExpand(whatMove, targetNode);
            }
        }

        public int getDepth(Task task) {
            DefaultMutableTreeNode treeNode = (DefaultMutableTreeNode) myTask2treeNode
                    .get(task);
            return treeNode.getLevel();
        }
    }

    private static class FacadeInvalidator implements TreeModelListener {
        private boolean isValid;

        public FacadeInvalidator(TreeModel treeModel) {
            isValid = false;
            treeModel.addTreeModelListener(this);
        }

        boolean isValid() {
            return isValid;
        }

        void reset() {
            isValid = true;
        }

        public void treeNodesChanged(TreeModelEvent e) {
            isValid = false;
        }

        public void treeNodesInserted(TreeModelEvent e) {
            isValid = false;
        }

        public void treeNodesRemoved(TreeModelEvent e) {
            isValid = false;
        }

        public void treeStructureChanged(TreeModelEvent e) {
            isValid = false;
        }
    }

    private final GanttOptions options;

    private JMenuBar bar;

    // ! Toolbar of ui
    private JToolBar toolBar;

    private DefaultListModel iconList = new DefaultListModel();

    private JList list = new JList(iconList);

    private DefaultListModel deletedIconList = new DefaultListModel();

    private JList deletedList = new JList(deletedIconList);

    // list.setName("list");

    // ! a status bar on the main frame
    private GanttStatusBar statusBar;

    private NewTaskAction myNewTaskAction;

    private NewHumanAction myNewHumanAction;

    private NewArtefactAction myNewArtefactAction;

    private CopyAction myCopyAction;

    private PasteAction myPasteAction;

    private CutAction myCutAction;

    private RefreshViewAction myRefreshAction;

    private DeleteHumanAction myDeleteHumanAction;

    private DeleteAssignmentAction myDeleteAssignmentAction;

    private TaskContainmentHierarchyFacadeImpl myCachedFacade;

    private List myRolloverActions = new ArrayList();

    private ArrayList myPreviousStates = new ArrayList();

    private MouseListener myStopEditingMouseListener = null;

    private DelayManager myDelayManager;

    // private boolean bQuickSave;//to know if gantt has to quicksave the
    // project
    // private int currentQuickSave;
    // private ArrayList aQuick;//List of all the quicksaves
    // private int lastQuickSave;
    // private int firstQuickSave;
    // private int undoNumber;

    private JSplitPane mySplitPane;

    private Chart[] myCharts = null;

    public TaskContainmentHierarchyFacade getTaskContainment() {
        if (!myFacadeInvalidator.isValid() || myCachedFacade == null) {
            myCachedFacade = new TaskContainmentHierarchyFacadeImpl(tree);
            myFacadeInvalidator.reset();
        }
        return myCachedFacade;
    }

    /** Constructor */
    public GanttProject(boolean isOnlyViewer, String filename, boolean isApplet) {
        ToolTipManager.sharedInstance().setInitialDelay(200);
        ToolTipManager.sharedInstance().setDismissDelay(60000);
        
        Mediator.registerRoleManager(getRoleManager());

        CustomColumnsStorage customColumnsStorage = new CustomColumnsStorage();
        Mediator.registerCustomColumnsStorage(customColumnsStorage);

        TaskSelectionManager taskSelectionManager = new TaskSelectionManager();
        Mediator.registerTaskSelectionManager(taskSelectionManager);

        this.isOnlyViewer = isOnlyViewer;
        if (!isOnlyViewer)
            setTitle(language.getText("appliTitle"));
        else
            setTitle("GanttViewer");
        setFocusable(true);
        lookAndFeel = GanttLookAndFeels.getGanttLookAndFeels().getDefaultInfo();
        options = new GanttOptions(getRoleManager(), isOnlyViewer);
        // Color color = GanttGraphicArea.taskDefaultColor;
        options.setUIConfiguration(myUIConfiguration);
        options.setDocumentsMRU(documentsMRU);
        options.setLookAndFeel(lookAndFeel);
        if (options.load()) {
            language = options.getLanguage();
            GanttGraphicArea.taskDefaultColor = options.getDefaultColor();

            lookAndFeel = options.getLnfInfos();
            HttpDocument.setLockDAVMinutes(options.getLockDAVMinutes());
        }

        myUIConfiguration = options.getUIConfiguration();
        class TaskManagerConfigImpl implements TaskManagerConfig {
            public Color getDefaultColor() {
                return myUIConfiguration.getTaskColor();
            }

            public GPCalendar getCalendar() {
                return GanttProject.this.getActiveCalendar();
            }

            public TimeUnitStack getTimeUnitStack() {
                return GanttProject.this.getTimeUnitStack();
            }

            public ResourceManager getResourceManager() {
                return GanttProject.this.getHumanResourceManager();
            }

        }
        TaskManagerConfig taskConfig = new TaskManagerConfigImpl();
        myTimeUnitStack = new GPTimeUnitStack(getLanguage());
        myTaskManager = TaskManager.Access.newInstance(
                new TaskContainmentHierarchyFacade.Factory() {
                    public TaskContainmentHierarchyFacade createFacede() {
                        return GanttProject.this.getTaskContainment();
                    }
                }, taskConfig);
        Mediator.registerTaskManager(myTaskManager);
        this.isApplet = isApplet;
        myZoomManager = new ZoomManager(this);
        ImageIcon icon = new ImageIcon(getClass().getResource(
                "/icons/ganttproject.png"));
        setIconImage(icon.getImage());

        // Create each objects
        tree = new GanttTree2(this, myTaskManager, getUIFacade());
        myFacadeInvalidator = new FacadeInvalidator(tree.getJTree().getModel());
        area = new GanttGraphicArea(this, tree, getTaskManager(),
                getZoomManager(), getUndoManager(), myUIConfiguration);
        area.getMyChartModel().addOptionChangeListener(
                new RowHeightAligner(tree, area.getMyChartModel()));
        GanttImagePanel but = new GanttImagePanel("big.png", "tasks.png", 300,
                42);
        tree.setGraphicArea(area);
        //

        miChartOptions = new JMenuItem(area.getOptionsDialogAction());

        getZoomManager().addZoomListener(area.getZoomListener());

        myCopyAction = new CopyAction((GanttTree2) getTree(), options
                .getIconSize());
        myPasteAction = new PasteAction((GanttTree2) getTree(), options
                .getIconSize());
        myCutAction = new CutAction((GanttTree2) getTree(), options
                .getIconSize());
        myRefreshAction = new RefreshViewAction(getUIFacade(), options);

        myRolloverActions.add(myCopyAction);
        myRolloverActions.add(myPasteAction);
        myRolloverActions.add(myCutAction);
        myRolloverActions.add(myRefreshAction);
        tree.setActions();

        // Create the menus

        bar = new JMenuBar();
        if (!isOnlyViewer)
            setJMenuBar(bar);
        // Allocation of the menus
        mProject = new JMenu();
        mMRU = new JMenu();
        mMRU.setIcon(new ImageIcon(getClass().getResource(
                "/icons/recent_16.gif")));
        mEdit = new JMenu();
        // mView = new JMenu ();
        mTask = new JMenu();
        mHuman = new JMenu();
        mHelp = new JMenu();
        mCalendar = new JMenu();
        mServer = new JMenu();
        mServer.setIcon(new ImageIcon(getClass().getResource(
                "/icons/server_16.gif")));
        miUndo = new JMenuItem(new UndoAction(getUndoManager(), "16"));
        mEdit.add(miUndo);
        // miUndo.setEnabled(false);
        miRedo = new JMenuItem(new RedoAction(getUndoManager(), "16"));
        mEdit.add(miRedo);
        // miRedo.setEnabled(false);
        mEdit.addSeparator();

        createProjectMenu();

        miRefresh = new JMenuItem(myRefreshAction);
        // miRefresh.setAccelerator((KeyStroke)myRefreshAction.getValue(Action.ACCELERATOR_KEY));
        mEdit.add(miRefresh);
        mEdit.addSeparator();

        miCut = new JMenuItem(myCutAction);
        mEdit.add(miCut);
        miCopy = new JMenuItem(myCopyAction);
        mEdit.add(miCopy);
        miPaste = new JMenuItem(myPasteAction);
        mEdit.add(miPaste);
        mEdit.addSeparator();
        miOptions = createNewItem("/icons/settings_16.gif");
        mEdit.add(miOptions);
        myNewTaskAction = new NewTaskAction((IGanttProject) this);
        mTask.add(myNewTaskAction);
        miDeleteTask = createNewItem("/icons/delete_16.gif");
        mTask.add(miDeleteTask);
        miPropertiesTask = createNewItem("/icons/properties_16.gif");
        mTask.add(miPropertiesTask);
        //
        // 
        myNewHumanAction = new NewHumanAction(getHumanResourceManager(),
                getRoleManager(), this, this) {
            public void actionPerformed(ActionEvent event) {
                super.actionPerformed(event);
                getTabs().setSelectedIndex(UIFacade.RESOURCES_INDEX);
            }
        };

        myDeleteAssignmentAction = new DeleteAssignmentAction(
                getHumanResourceManager(),
                (AssignmentContext) getResourcePanel(), this);

        mHuman.add(myNewHumanAction);
        myDeleteHumanAction = new DeleteHumanAction(getHumanResourceManager(),
                (ResourceContext) getResourcePanel(), this);
        miDelHuman = new JMenuItem(myDeleteHumanAction);
        mHuman.add(miDelHuman);
        miPropHuman = createNewItem("/icons/properties_16.gif");
        mHuman.add(miPropHuman);
        miSendMailHuman = createNewItem("/icons/send_mail_16.gif");
        mHuman.add(miSendMailHuman);

        mHuman.add(new ImportResources(getHumanResourceManager(),
                getTaskManager(), getRoleManager(), this));

        miEditCalendar = createNewItem("/icons/clock_16.gif");
        mCalendar.add(miEditCalendar);
        miPrjCal = createNewItem("/icons/default_calendar_16.gif");
        mCalendar.add(miPrjCal);
        miWebPage = createNewItem("/icons/home_16.gif");
        mHelp.add(miWebPage);
        miManual = createNewItem("/icons/help_16.gif");
        try { // See if helpgui library is on the classpath
            Class.forName("net.sourceforge.helpgui.HelpGui");
            mHelp.add(miManual);
            miManual.setAccelerator(KeyStroke.getKeyStroke("F1"));
        } catch (Exception ex) {
            // Not add the help button on the ui
        }
        miTips = createNewItem("/icons/about_16.gif");
        mHelp.add(miTips);
        miAbout = createNewItem("/icons/manual_16.gif");
        mHelp.add(miAbout);
        if (!isApplet) {
            bar.add(mProject); // for a applet veiwer, Project menu is not
            // neccessary By CL
        }
        bar.add(mEdit);
        JMenu viewMenu = createViewMenu();
        if (viewMenu != null)
            bar.add(viewMenu);
        // bar.add (mView);
        bar.add(mTask);
        bar.add(mHuman);
        // bar.add(mCalendar);
        bar.add(mHelp);
        setMemonic();
        // to create a default project
        // createDefaultTree(tree);
        JPanel left = new JPanel(new BorderLayout());
        left.add(but, BorderLayout.NORTH);
        left.add(tree, BorderLayout.CENTER);
        left.setPreferredSize(new Dimension(315, 600));
        left.setBackground(Color.white);

        // A splitpane is use
        mySplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        if (language.getComponentOrientation() == ComponentOrientation.LEFT_TO_RIGHT) {
            mySplitPane.setLeftComponent(left);
            mySplitPane.setRightComponent(area);
            mySplitPane
                    .applyComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
        } else {
            mySplitPane.setRightComponent(left);
            mySplitPane.setLeftComponent(area);
            mySplitPane.setDividerLocation((int) (Toolkit.getDefaultToolkit()
                    .getScreenSize().getWidth() - left.getPreferredSize()
                    .getWidth()));
            mySplitPane
                    .applyComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
        }
        mySplitPane.setOneTouchExpandable(true);
        mySplitPane.setPreferredSize(new Dimension(800, 500));

        // Add Gantt Panel, Human resource pannel and RESOURCE panel
        getTabs().addTab(language.getText("gantt"),
                new ImageIcon(getClass().getResource("/icons/tasks_16.gif")),
                mySplitPane, area);
        getTabs().addTab(language.getText("human"),
                new ImageIcon(getClass().getResource("/icons/res_16.gif")),
                getResourcePanel(), getResourcePanel().area);
        // pert area
        getTabs().setTabLayoutPolicy(JTabbedPane.SCROLL_TAB_LAYOUT);
        getTabs().addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                bUnlink
                        .setEnabled(getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX);
                bLink
                        .setEnabled(getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX);
                bInd
                        .setEnabled(getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX);
                bUnind
                        .setEnabled(getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX);

                bCut
                        .setEnabled(getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX);
                bCopy
                        .setEnabled(getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX);
                bPaste
                        .setEnabled(getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX);
                bCritical
                        .setEnabled(getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX);
                bScrollCenter
                        .setEnabled(getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX);
                bComparePrev
                        .setEnabled(getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX);
                bSaveCurrent
                        .setEnabled(getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX);

                bNewTask
                        .setEnabled(getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX
                                || getTabs().getSelectedIndex() == UIFacade.RESOURCES_INDEX);

                bDelete
                        .setEnabled(getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX
                                || getTabs().getSelectedIndex() == UIFacade.RESOURCES_INDEX);

                bProperties
                        .setEnabled(getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX
                                || getTabs().getSelectedIndex() == UIFacade.RESOURCES_INDEX);

                bDown
                        .setEnabled(getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX
                                || getTabs().getSelectedIndex() == UIFacade.RESOURCES_INDEX);
                bUp
                        .setEnabled(getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX
                                || getTabs().getSelectedIndex() == UIFacade.RESOURCES_INDEX);
                bZoomIn
                        .setEnabled(getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX
                                || getTabs().getSelectedIndex() == UIFacade.RESOURCES_INDEX);

                bZoomOut
                        .setEnabled(getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX
                                || getTabs().getSelectedIndex() == UIFacade.RESOURCES_INDEX);
                bPrev
                        .setEnabled(getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX
                                || getTabs().getSelectedIndex() == UIFacade.RESOURCES_INDEX);
                bNext
                        .setEnabled(getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX
                                || getTabs().getSelectedIndex() == UIFacade.RESOURCES_INDEX);

                if (getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX) { // Gantt
                    // Chart
                    bNewTask.setToolTipText(getToolTip(correctLabel(language
                            .getText("createTask"))));
                    bDelete.setToolTipText(getToolTip(correctLabel(language
                            .getText("deleteTask"))));
                    bProperties.setToolTipText(getToolTip(correctLabel(language
                            .getText("propertiesTask"))));

                    if (options.getButtonShow() != GanttOptions.ICONS) {
                        bNewTask.setText(correctLabel(language
                                .getText("createTask")));
                        bDelete.setText(correctLabel(language
                                .getText("deleteTask")));
                        bProperties.setText(correctLabel(language
                                .getText("propertiesTask")));
                    }

                } else if (getTabs().getSelectedIndex() == UIFacade.RESOURCES_INDEX) { // Resources
                    // Chart
                    bNewTask.setToolTipText(getToolTip(correctLabel(language
                            .getText("newHuman"))));
                    bDelete.setToolTipText(getToolTip(correctLabel(language
                            .getText("deleteHuman"))));
                    bProperties.setToolTipText(getToolTip(correctLabel(language
                            .getText("propertiesHuman"))));

                    if (options.getButtonShow() != GanttOptions.ICONS) {
                        bNewTask.setText(correctLabel(language
                                .getText("newHuman")));
                        bDelete.setText(correctLabel(language
                                .getText("deleteHuman")));
                        bProperties.setText(correctLabel(language
                                .getText("propertiesHuman")));
                    }
                }
            }
        });
        // Add tabpan on the content pane
        getContentPane().add(getTabs(), BorderLayout.CENTER);
        // Add toolbar
        toolBar = new JToolBar("GanttProject", options.getToolBarPosition());
        toolBar.setBorderPainted(true);
        toolBar.setRollover(true);
        toolBar.setFloatable(true);
        toolBar.addComponentListener(new ComponentListener() {

            public void componentResized(ComponentEvent arg0) {
                setHiddens();
                refresh();
            }

            public void componentMoved(ComponentEvent arg0) {
            }

            public void componentShown(ComponentEvent arg0) {
            }

            public void componentHidden(ComponentEvent arg0) {
            }
        });
        this.addButtons(toolBar);
        getContentPane()
                .add(
                        toolBar,
                        (toolBar.getOrientation() == JToolBar.HORIZONTAL) ? BorderLayout.NORTH
                                : BorderLayout.WEST);

        // add the status bar
        statusBar = new GanttStatusBar();
        if (!isOnlyViewer)
            getContentPane().add(statusBar, BorderLayout.SOUTH);
        statusBar.setVisible(options.getShowStatusBar());

        // add a keyboard listener
        addKeyListener(this);

        SwitchViewAction switchAction = new SwitchViewAction(this);
        JMenuItem invisibleItem = new JMenuItem(switchAction);
        invisibleItem.setVisible(false);
        bar.add(invisibleItem);

        // Open the the project passed in argument
        if (filename != null) {
            Document document = DocumentCreator.createDocument(filename);
            try {
                openDocument(document);
            } catch (IOException ex) {
                System.err.println(language.getText("msg2") + "\n" + filename);
                System.err.println(ex.getMessage());
            }
        }
        // update 18-03-2003
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });
        // update 18-03-2003
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension windowSize = getPreferredSize();
        // Put the frame at the middle of the screen
        setLocation(screenSize.width / 2 - (windowSize.width / 2),
                screenSize.height / 2 - (windowSize.height / 2));
        this.pack();
        changeLanguage();

        /*
         * [bbaranne] I add a Mediator object so that we can get the GanttProject
         * singleton whereever we are in the source code. Perhaps some of you
         * don't like this, but I believe that it is practical...
         */
        Mediator.registerGanttProject(this);

        // changeUndoNumber ();
        changeLookAndFeel(lookAndFeel);
        changeLookAndFeel(lookAndFeel); // Twice call for update font on menu
        if (options.isLoaded()) {
            setBounds(options.getX(), options.getY(), options.getWidth(),
                    options.getHeight());
        }
        if (options.getOpenTips() && !byCommandLine) {
            TipsDialog tips = new TipsDialog(this, options.getOpenTips());
            tips.show();
            tips.toFront(); // somehow assure, that the TipsDialog is the top
            // window an MacOS
        }
        applyComponentOrientation(GanttLanguage.getInstance()
                .getComponentOrientation());
        myTaskManager.addTaskListener(new TaskModelModificationListener(this));
        if (ourWindowListener != null) {
            addWindowListener(ourWindowListener);
        }
        addMouseListenerToAllContainer(this.getComponents());
        myDelayManager = new DelayManager(myTaskManager, tree);
        Mediator.registerDelayManager(myDelayManager);
        myDelayManager.addObserver(tree);
        myTaskManager.addTaskListener(myDelayManager);
    }

    private void addMouseListenerToAllContainer(Component[] cont) {
        for (int i = 0; i < cont.length; i++) {
            cont[i].addMouseListener(getStopEditingMouseListener());
            if (cont[i] instanceof Container)
                addMouseListenerToAllContainer(((Container) cont[i])
                        .getComponents());
        }
    }

    /**
     * Returns a mouseListener that stop the edition in the ganttTreeTable.
     * 
     * @return A mouseListener that stop the edition in the ganttTreeTable.
     */
    private MouseListener getStopEditingMouseListener() {
        if (myStopEditingMouseListener == null)
            myStopEditingMouseListener = new MouseAdapter() {
                public void mouseClicked(MouseEvent e) {
                    if (e.getSource() != bNewTask && e.getClickCount() == 1)
                        tree.stopEditing();
                    if (e.getButton() == MouseEvent.BUTTON1
                            && !(e.getSource() instanceof JTable)
                            && !(e.getSource() instanceof AbstractButton)) {
                        Task taskUnderPointer = area.new MouseSupport()
                                .findTaskUnderMousePointer(e.getX(), e.getY());
                        if (taskUnderPointer == null) {
                            tree.selectTreeRow(-1);
                        }
                    }
                }
            };
        return myStopEditingMouseListener;
    }

    private void createProjectMenu() {
        miNew = createNewItem("/icons/new_16.gif");
        mProject.add(miNew);
        miOpen = createNewItem("/icons/open_16.gif");
        mProject.add(miOpen);
        mProject.add(mMRU);
        updateMenuMRU();
        mProject.addSeparator();
        miSave = createNewItem("/icons/save_16.gif");
        miSave.setEnabled(false);
        mProject.add(miSave);
        miSaveAs = createNewItem("/icons/saveas_16.gif");
        mProject.add(miSaveAs);
        mProject.addSeparator();
        //
        ImportFileAction importFile = new ImportFileAction(getUIFacade(), this);
        mProject.add(importFile);
        ExportFileAction exportFile = new ExportFileAction(getUIFacade(), this,
                options);
        mProject.add(exportFile);
        // miExport = createNewItem("/icons/export_16.gif");
        // mProject.add(miExport);
        mProject.addSeparator();

        miOpenURL = createNewItem("");
        miSaveAsURL = createNewItem("");
        miPublishToWeb = createNewItem("/icons/publish.gif");
        mServer.add(miOpenURL);
        mServer.add(miSaveAsURL);
        mServer.addSeparator();
        mServer.add(miPublishToWeb);
        mProject.add(mServer);
        mProject.addSeparator();
        miPrint = createNewItem("/icons/print_16.gif");
        mProject.add(miPrint);
        miPreview = createNewItem("/icons/preview_16.gif");
        mProject.add(miPreview);
        mProject.addSeparator();
        miQuit = createNewItem(""/* "/icons/exit_16.gif" */);
        mProject.add(miQuit);
    }

    private JMenu createViewMenu() {
        JMenu result = changeMenuLabel(new JMenu(), language.getText("view"));
        result.add(miChartOptions);
        Chart[] charts = Mediator.getPluginManager().getCharts();

        if (charts.length > 0)
            result.addSeparator();

        for (int i = 0; i < charts.length; i++) {
            result.add(new JCheckBoxMenuItem(new ToggleChartAction(charts[i],
                    getViewManager())));
        }
        return result;
    }

    public GanttProject(boolean isOnlyViewer, String filename) {
        this(isOnlyViewer, filename, false);
    }

    /**
     * Updates the last open file menu items.
     */
    private void updateMenuMRU() {
        mMRU.removeAll();
        int index = 0;
        Iterator iterator = documentsMRU.iterator();
        while (iterator.hasNext()) {
            index++;
            Document document = (Document) iterator.next();
            JMenuItem mi = new JMenuItem(new OpenDocumentAction(index,
                    document, this));
            mMRU.add(mi);
        }
    }

    public String getXslDir() {
        return options.getXslDir();
    }

    /** @return the options of ganttproject. */
    public GanttOptions getOptions() {
        return options;
    }

    public void restoreOptions() {
        options.initByDefault(); // options by default
        iconList = initIconList();
        deletedIconList = initDeletedIconList();
        addButtons();
        myUIConfiguration = options.getUIConfiguration();
        GanttGraphicArea.taskDefaultColor = new Color(140, 182, 206);
        area.repaint();
    }

    /** @return the status Bar of the main frame. */
    public GanttStatusBar getStatusBar() {
        return statusBar;
    }

    public String getXslFo() {
        return options.getXslFo();
    }

    /** Create memonic for keyboard */
    public void setMemonic() {
        int MENU_MASK = GPAction.MENU_MASK;

        // --NEW----------------------------------
        // miNew.setMnemonic(KeyEvent.VK_N);
        miNew.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, MENU_MASK));
        // --OPEN----------------------------------
        miOpen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, MENU_MASK));
        // Open from the web
        // miOpenURL.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I,
        // MENU_MASK));
        // --SAVE----------------------------------
        miSave.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, MENU_MASK));
        // --PRINT----------------------------------
        miPrint
                .setAccelerator(KeyStroke
                        .getKeyStroke(KeyEvent.VK_P, MENU_MASK));

        // --UNDO----------------------------------
        miUndo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, MENU_MASK));
        // --REDO----------------------------------
        miRedo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Y, MENU_MASK));
        if (!isOnlyViewer) {
            // --QUIT----------------------------------
            miQuit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q,
                    MENU_MASK));
            // //--CUT----------------------------------
            // miCut.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X,
            // MENU_MASK));
            // //--COPY----------------------------------
            // miCopy.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C,
            // MENU_MASK));
            // //--PASTE----------------------------------
            // miPaste.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V,
            // MENU_MASK));
            // --OPTIONS----------------------------------
            miOptions.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_G,
                    MENU_MASK));
            // --NEW TASK----------------------------------
            // miNewTask.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T,
            // MENU_MASK));
            // --PROPERTIES TASK----------------------------------
            miPropertiesTask.setAccelerator(KeyStroke.getKeyStroke(
                    KeyEvent.VK_R, MENU_MASK));
            // --DELETE TASK----------------------------------
            miDeleteTask.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D,
                    MENU_MASK));
            // --NEW HUMAN----------------------------------
            // miNewHuman.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_H,
            // MENU_MASK));
            // miDelHuman.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_J,
            // MENU_MASK));
            miPropHuman.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L,
                    MENU_MASK));
        }

    }

    /** Create an item with a label */
    public JMenuItem createNewItemText(String label) {
        JMenuItem item = new JMenuItem(label);
        item.addActionListener(this);
        return item;
    }

    /** Create an item with an icon */
    public JMenuItem createNewItem(String icon) {
        URL url = getClass().getResource(icon);
        JMenuItem item = url == null ? new JMenuItem() : new JMenuItem(
                new ImageIcon(url));
        item.addActionListener(this);
        return item;
    }

    /** Create an item with a label and an icon */
    public JMenuItem createNewItem(String label, String icon) {
        JMenuItem item = new JMenuItem(label, new ImageIcon(getClass()
                .getResource(icon)));
        item.addActionListener(this);
        return item;
    }

    /** Function to change language of the project */
    public void changeLanguage() {

        if (helpFrame != null)
            helpFrame.dispose();
        helpFrame = null;

        applyComponentOrientation(language.getComponentOrientation());
        changeLanguageOfMenu();
        area.repaint();
        getResourcePanel().area.repaint();
        getResourcePanel().refresh(language);

        this.tree.changeLanguage(language);
        CustomColumnsStorage.changeLanguage(language);

        applyComponentOrientation(language.getComponentOrientation());

        // change the value of the separators in iconList
        for (int i = 0; i < iconList.size(); i++)
            if (iconList.getElementAt(i).getClass() != TestGanttRolloverButton.class) {
                iconList.removeElementAt(i);
                iconList.add(i, language.getText("separator").toUpperCase());
            }
    }

    /** Change the style of the application */
    public void changeLookAndFeel(GanttLookAndFeelInfo lookAndFeel) {
        try {
            UIManager.setLookAndFeel(lookAndFeel.getClassName());
            SwingUtilities.updateComponentTreeUI(this);
            this.lookAndFeel = lookAndFeel;
        } catch (Exception e) {
            GanttLookAndFeelInfo info = GanttLookAndFeels
                    .getGanttLookAndFeels().getDefaultInfo();
            System.out.println("Can't find the LookAndFeel\n"
                    + lookAndFeel.getClassName() + "\n" + lookAndFeel.getName()
                    + "\nSetting the default Look'n'Feel" + info.getName());
            try {
                UIManager.setLookAndFeel(info.getClassName());
                SwingUtilities.updateComponentTreeUI(this);
                this.lookAndFeel = info;
            } catch (Exception ex) {
            }
        }
        // MetalLookAndFeel.setCurrentTheme(new GanttMetalTheme());
        // must force to do that instead of the task on tree are not in
        // continuity of the calendar
        if (tree.getTable().getRowHeight() <= 22)
            tree.getTable().setRowHeight(20);
    }

    // Correct the label of menu without '$' character
    public static String correctLabel(String label) {
        int index = label.indexOf('$');
        if (index != -1 && label.length() - index > 1)
            label = label.substring(0, index).concat(label.substring(++index));
        return label;
    }

    /** Change the label for menu, in fact check in the label contains a mnemonic */
    public JMenu changeMenuLabel(JMenu menu, String label) {
        int index = label.indexOf('$');
        if (index != -1 && label.length() - index > 1) {
            menu.setText(label.substring(0, index).concat(
                    label.substring(++index)));
            menu.setMnemonic(Character.toLowerCase(label.charAt(index)));
        } else {
            menu.setText(label);
            // menu.setMnemonic('');
        }
        return menu;
    }

    /**
     * Change the label for menuItem, in fact check in the label contains a
     * mnemonic
     */
    public JMenuItem changeMenuLabel(JMenuItem menu, String label) {
        int index = label.indexOf('$');
        if (index != -1 && label.length() - index > 1) {
            menu.setText(label.substring(0, index).concat(
                    label.substring(++index)));
            menu.setMnemonic(Character.toLowerCase(label.charAt(index)));
        } else {
            menu.setText(label);
            // menu.setMnemonic('');
        }
        return menu;
    }

    /**
     * Change the label for JCheckBoxmenuItem, in fact check in the label
     * contains a mnemonic
     */
    public JCheckBoxMenuItem changeMenuLabel(JCheckBoxMenuItem menu,
            String label) {
        int index = label.indexOf('$');
        if (index != -1 && label.length() - index > 1) {
            menu.setText(label.substring(0, index).concat(
                    label.substring(++index)));
            menu.setMnemonic(Character.toLowerCase(label.charAt(index)));
        } else {
            menu.setText(label);
            // menu.setMnemonic('');
        }
        return menu;
    }

    /** Set the menus language after the user select a different language */
    private void changeLanguageOfMenu() {
        mProject = changeMenuLabel(mProject, language.getText("project"));
        mEdit = changeMenuLabel(mEdit, language.getText("edit"));
        // mView = changeMenuLabel(mView, language.getText("view"));
        mTask = changeMenuLabel(mTask, language.getText("task"));
        mHuman = changeMenuLabel(mHuman, language.getText("human"));
        mHelp = changeMenuLabel(mHelp, language.getText("help"));
        mCalendar = changeMenuLabel(mCalendar, language.getText("calendars"));
        miNew = changeMenuLabel(miNew, language.getText("newProject"));
        miOpen = changeMenuLabel(miOpen, language.getText("openProject"));
        mMRU = changeMenuLabel(mMRU, language.getText("lastOpen"));
        miSave = changeMenuLabel(miSave, language.getText("saveProject"));
        miSaveAs = changeMenuLabel(miSaveAs, language.getText("saveAsProject"));

        mServer = changeMenuLabel(mServer, language.getText("webServer"));
        miOpenURL = changeMenuLabel(miOpenURL, language
                .getText("openFromServer"));
        miSaveAsURL = changeMenuLabel(miSaveAsURL, language
                .getText("saveToServer"));
        miPublishToWeb = changeMenuLabel(miPublishToWeb, language
                .getText("publishToWeb"));

        miPrint = changeMenuLabel(miPrint, language.getText("printProject"));
        miPreview = changeMenuLabel(miPreview, language.getText("preview"));
        miQuit = changeMenuLabel(miQuit, language.getText("quit"));
        miUndo = changeMenuLabel(miUndo, language.getText("undo"));
        miRedo = changeMenuLabel(miRedo, language.getText("redo"));
        miCut = changeMenuLabel(miCut, language.getText("cut"));
        miCopy = changeMenuLabel(miCopy, language.getText("copy"));
        miPaste = changeMenuLabel(miPaste, language.getText("paste"));
        miOptions = changeMenuLabel(miOptions, language.getText("settings"));
        // miNewTask = changeMenuLabel(miNewTask,
        // language.getText("createTask"));
        miDeleteTask = changeMenuLabel(miDeleteTask, language
                .getText("deleteTask"));
        miPropertiesTask = changeMenuLabel(miPropertiesTask, language
                .getText("propertiesTask"));

        mHuman.insert(changeMenuLabel(mHuman.getItem(0), language
                .getText("newHuman")), 0);
        miDelHuman = changeMenuLabel(miDelHuman, language
                .getText("deleteHuman"));
        miPropHuman = changeMenuLabel(miPropHuman, language
                .getText("propertiesHuman"));
        mHuman.insert(changeMenuLabel(mHuman.getItem(4), language
                .getText("importResources")), 4);
        miSendMailHuman = changeMenuLabel(miSendMailHuman, language
                .getText("sendMail"));

        miEditCalendar = changeMenuLabel(miEditCalendar, language
                .getText("editCalendars"));
        miPrjCal = changeMenuLabel(miPrjCal, language
                .getText("projectCalendar"));

        miWebPage = changeMenuLabel(miWebPage, language.getText("webPage"));
        miAbout = changeMenuLabel(miAbout, language.getText("about"));
        miTips = changeMenuLabel(miTips, language.getText("tipsOfTheDay"));
        miManual = changeMenuLabel(miManual, language.getText("manual"));
        miChartOptions = changeMenuLabel(miChartOptions, language
                .getText("chartOptions"));
        miRefresh = changeMenuLabel(miRefresh, language.getText("refresh"));
        // //////////////////////////////////////////

        bExit
                .setToolTipText(getToolTip(correctLabel(language
                        .getText("quit"))));
        bNew.setToolTipText(getToolTip(correctLabel(language
                .getText("newProject"))));
        bOpen.setToolTipText(getToolTip(correctLabel(language
                .getText("openProject"))));
        bSave.setToolTipText(getToolTip(correctLabel(language
                .getText("saveProject"))));
        bSaveAs.setToolTipText(getToolTip(correctLabel(language
                .getText("saveAsProject"))));
        bPrint.setToolTipText(getToolTip(correctLabel(language
                .getText("printProject"))));
        bPreviewPrint.setToolTipText(getToolTip(correctLabel(language
                .getText("preview"))));
        bExport.setToolTipText(getToolTip(correctLabel(language
                .getText("export"))));
        bImport.setToolTipText(getToolTip(correctLabel(language
                .getText("import"))));
        bNewTask.setToolTipText(getToolTip(correctLabel(language
                .getText("createTask"))));
        bCut.setToolTipText(getToolTip(correctLabel(language.getText("cut"))));
        bCopy
                .setToolTipText(getToolTip(correctLabel(language
                        .getText("copy"))));
        bPaste
                .setToolTipText(getToolTip(correctLabel(language
                        .getText("paste"))));
        bDelete.setToolTipText(getToolTip(correctLabel(language
                .getText("deleteTask"))));
        bProperties.setToolTipText(getToolTip(correctLabel(language
                .getText("propertiesTask"))));
        bUnlink.setToolTipText(getToolTip(correctLabel(language
                .getText("unlink"))));
        bLink
                .setToolTipText(getToolTip(correctLabel(language
                        .getText("link"))));
        bInd.setToolTipText(getToolTip(correctLabel(language
                .getText("indentTask"))));
        bUnind.setToolTipText(getToolTip(correctLabel(language
                .getText("dedentTask"))));
        bUp
                .setToolTipText(getToolTip(correctLabel(language
                        .getText("upTask"))));
        bDown.setToolTipText(getToolTip(correctLabel(language
                .getText("downTask"))));
        bPrev.setToolTipText(getToolTip(correctLabel(language
                .getText("backDate"))));
        bScrollCenter.setToolTipText(getToolTip(correctLabel(language
                .getText("centerOnSelectedTasks"))));
        bNext.setToolTipText(getToolTip(correctLabel(language
                .getText("forwardDate"))));
        bZoomIn.setToolTipText(getToolTip(correctLabel(language
                .getText("zoomIn"))));
        bZoomOut.setToolTipText(getToolTip(correctLabel(language
                .getText("zoomOut"))));
        bAbout
                .setToolTipText(getToolTip(correctLabel(language
                        .getText("about"))));
        bUndo
                .setToolTipText(getToolTip(correctLabel(language
                        .getText("undo"))));
        bRedo
                .setToolTipText(getToolTip(correctLabel(language
                        .getText("redo"))));
        // bZoomFit.setToolTipText(getToolTip(language.zoomFit()));

        bCritical.setToolTipText(getToolTip(language.getText("criticalPath")));
        bComparePrev
                .setToolTipText(getToolTip(language.getText("comparePrev")));
        bSaveCurrent
                .setToolTipText(getToolTip(language.getText("saveCurrent")));
        bRefresh.setToolTipText(getToolTip(language.getText("refresh")));
        bShowHiddens
                .setToolTipText(getToolTip(language.getText("showHiddens")));
        getTabs().setTitleAt(1, correctLabel(language.getText("human")));
        setButtonText();
    }

    /** Invoked when a key has been pressed. */
    public void keyPressed(KeyEvent e) {
        final int MENU_MASK = Toolkit.getDefaultToolkit()
                .getMenuShortcutKeyMask();

        // System.out.println(e.getKeyCode());
        /*
         * Consume the event to prevent it to go farther.
         */
        int code = e.getKeyCode();
        int modifiers = e.getModifiersEx();

        if (code == KeyEvent.KEY_LOCATION_UNKNOWN)
            e.consume();

        switch (code) {
        case KeyEvent.VK_DELETE:
            e.consume();
            if (!isOnlyViewer) {
                if (getViewIndex() == UIFacade.GANTT_INDEX)
                    deleteTasks(true);
                else if (getViewIndex() == UIFacade.RESOURCES_INDEX) {
                    deleteResources();
                    deleteAssignements();
                }
            }
            break;
        // case KeyEvent.VK_INSERT :
        // // if(!isOnlyViewer);newTask();break;

        case KeyEvent.VK_ENTER:
            if (!isOnlyViewer) {
                e.consume();
                if ((modifiers & InputEvent.ALT_DOWN_MASK) == InputEvent.ALT_DOWN_MASK) {
                    if (getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX) {// Gantt
                        // Chart
                        propertiesTask();
                    } else if (getTabs().getSelectedIndex() == UIFacade.RESOURCES_INDEX) { // Resource
                        // chart
                        getResourcePanel().propertiesHuman(GanttProject.this);
                    }
                }
            }
            break;
        case KeyEvent.VK_TAB: {
            e.consume();
            if ((modifiers & InputEvent.SHIFT_DOWN_MASK) == InputEvent.SHIFT_DOWN_MASK) {
                tree.dedentCurrentNodes();
            } else
                tree.indentCurrentNodes();
            break;
        }
        case KeyEvent.VK_F5: {
            e.consume();
            repaint2();
            break;
        }

        // case KeyEvent.VK_F7:
        // {
        // if((modifiers & InputEvent.CTRL_DOWN_MASK) ==
        // InputEvent.CTRL_DOWN_MASK)
        // {
        // tabpane.setSelectedIndex((tabpane.getSelectedIndex()+1)%
        // tabpane.getTabCount());
        // }
        // break;
        // }
        // case KeyEvent.VK_HOME : tree.selectTreeRow(0); break; //go at the
        // first task
        // case KeyEvent.VK_END : tree.selectTreeRow(tree.nbTasks-1); break;
        // //go at the last task
        }
    }

    /** Invoked when a key has been released. */
    public void keyReleased(KeyEvent e) {
    }

    /** Invoked when a key has been typed. */
    public void keyTyped(KeyEvent e) {
    }

    /** Return the tooltip in html (with yello bgcolor */
    public static String getToolTip(String msg) {
        return "<html><body bgcolor=#EAEAEA>" + msg + "</body></html>";
    }

    /** Set the text on the buttons. */
    public void setButtonText() {
        if (options.getButtonShow() == GanttOptions.ICONS) {
            bExit.setText("");
            bNew.setText("");
            bOpen.setText("");
            bSave.setText("");
            bCut.setText("");
            bCopy.setText("");
            bPaste.setText("");
            bSaveAs.setText("");
            bNewTask.setText("");
            bImport.setText("");
            bExport.setText("");
            bPrint.setText("");
            bPreviewPrint.setText("");
            bDelete.setText("");
            bProperties.setText("");
            bUnlink.setText("");
            bLink.setText("");
            bInd.setText("");
            bUnind.setText("");
            bUp.setText("");
            bDown.setText("");
            bPrev.setText("");
            bNext.setText("");
            bScrollCenter.setText("");
            bZoomOut.setText("");
            bZoomIn.setText("");
            bAbout.setText("");
            bUndo.setText("");
            bRedo.setText("");
            bCritical.setText("");
            bComparePrev.setText("");
            bSaveCurrent.setText("");
            bRefresh.setText("");
            bShowHiddens.setText("");
        } else {
            bExit.setText(correctLabel(language.getText("quit")));
            bNew.setText(correctLabel(language.getText("newProject")));
            bOpen.setText(correctLabel(language.getText("openProject")));
            bSave.setText(correctLabel(language.getText("saveProject")));
            bSaveAs.setText(correctLabel(language.getText("saveAsProject")));
            bImport.setText(correctLabel(language.getText("import")));
            bExport.setText(correctLabel(language.getText("export")));
            bPrint.setText(correctLabel(language.getText("printProject")));
            bPreviewPrint.setText(correctLabel(language.getText("preview")));
            bCut.setText(correctLabel(language.getText("cut")));
            bCopy.setText(correctLabel(language.getText("copy")));
            bPaste.setText(correctLabel(language.getText("paste")));

            bNewTask.setText(correctLabel(language.getText(getTabs()
                    .getSelectedIndex() == UIFacade.GANTT_INDEX ? "createTask"
                    : "newHuman")));
            bDelete.setText(correctLabel(language.getText(getTabs()
                    .getSelectedIndex() == UIFacade.GANTT_INDEX ? "deleteTask"
                    : "deleteHuman")));
            bProperties
                    .setText(correctLabel(language
                            .getText(getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX ? "propertiesTask"
                                    : "propertiesHuman")));

            bUnlink.setText(correctLabel(language.getText("unlink")));
            bLink.setText(correctLabel(language.getText("link")));
            bInd.setText(correctLabel(language.getText("indentTask")));
            bUnind.setText(correctLabel(language.getText("dedentTask")));
            bUp.setText(correctLabel(language.getText("upTask")));
            bDown.setText(correctLabel(language.getText("downTask")));
            bPrev.setText(correctLabel(language.getText("backDate")));
            bScrollCenter.setText(correctLabel(language
                    .getText("centerOnSelectedTasks")));
            bNext.setText(correctLabel(language.getText("forwardDate")));
            bZoomOut.setText(correctLabel(language.getText("zoomOut")));
            bZoomIn.setText(correctLabel(language.getText("zoomIn")));
            bAbout.setText(correctLabel(language.getText("about")));
            bUndo.setText(correctLabel(language.getText("undo")));
            bRedo.setText(correctLabel(language.getText("redo")));
            bCritical.setText(language.getText("criticalPath"));
            bComparePrev.setText(correctLabel(language.getText("comparePrev")));
            bSaveCurrent.setText(correctLabel(language.getText("saveCurrent")));
            bRefresh.setText(correctLabel(language.getText("refresh")));
            // bShowHiddens.setText
            // (correctLabel(language.getText("showHiddens")));
        }
    }

    /** Apply Buttons options. */
    public void applyButtonOptions() {
        setButtonText();
        if (options.getButtonShow() == GanttOptions.TEXT) {
            // remove the icons
            bExit.setDefaultIcon(null);
            bNew.setDefaultIcon(null);
            bOpen.setDefaultIcon(null);
            bSave.setDefaultIcon(null);
            bSaveAs.setDefaultIcon(null);
            bImport.setDefaultIcon(null);
            bExport.setDefaultIcon(null);
            bPrint.setDefaultIcon(null);
            bPreviewPrint.setDefaultIcon(null);
            bNewTask.setDefaultIcon(null);
            bDelete.setDefaultIcon(null);
            bCut.setDefaultIcon(null);
            bCopy.setDefaultIcon(null);
            bPaste.setDefaultIcon(null);
            bProperties.setDefaultIcon(null);
            bUnlink.setDefaultIcon(null);
            bLink.setDefaultIcon(null);
            bInd.setDefaultIcon(null);
            bUnind.setDefaultIcon(null);
            bUp.setDefaultIcon(null);
            bDown.setDefaultIcon(null);
            bPrint.setDefaultIcon(null);
            bPrev.setDefaultIcon(null);
            bScrollCenter.setDefaultIcon(null);
            bZoomOut.setDefaultIcon(null);
            bZoomIn.setDefaultIcon(null);
            bAbout.setDefaultIcon(null);
            bUndo.setDefaultIcon(null);
            bRedo.setDefaultIcon(null);
            bCritical.setDefaultIcon(null);
            bComparePrev.setDefaultIcon(null);
            bSaveCurrent.setDefaultIcon(null);
            bRefresh.setDefaultIcon(null);
            // bShowHiddens.setDefaultIcon(null);
            for (int i = 0; i < myRolloverActions.size(); i++) {
                RolloverAction next = (RolloverAction) myRolloverActions.get(i);
                next.isIconVisible(false);
                next.setIconSize(options.getIconSize());
            }
        } else {
            // set the approrpiate icons
            bExit.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/exit_" + options.getIconSize() + ".gif")));
            bNew.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/new_" + options.getIconSize() + ".gif")));
            bOpen.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/open_" + options.getIconSize() + ".gif")));
            bSave.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/save_" + options.getIconSize() + ".gif")));
            bSaveAs.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/saveas_" + options.getIconSize() + ".gif")));
            bImport.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/import_" + options.getIconSize() + ".gif")));
            bExport.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/export_" + options.getIconSize() + ".gif")));
            bPrint.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/print_" + options.getIconSize() + ".gif")));
            bPreviewPrint.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/preview_" + options.getIconSize() + ".gif")));
            bNewTask.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/insert_" + options.getIconSize() + ".gif")));
            bDelete.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/delete_" + options.getIconSize() + ".gif")));
            bProperties.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/properties_" + options.getIconSize() + ".gif")));
            bCut.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/cut_" + options.getIconSize() + ".gif")));
            bCopy.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/copy_" + options.getIconSize() + ".gif")));
            bPaste.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/paste_" + options.getIconSize() + ".gif")));
            bUnlink.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/unlink_" + options.getIconSize() + ".gif")));
            bLink.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/link_" + options.getIconSize() + ".gif")));
            bInd.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/indent_" + options.getIconSize() + ".gif")));
            bUnind.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/unindent_" + options.getIconSize() + ".gif")));
            bUp.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/up_" + options.getIconSize() + ".gif")));
            bDown.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/down_" + options.getIconSize() + ".gif")));
            bPrev.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/prev_" + options.getIconSize() + ".gif")));
            bNext.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/next_" + options.getIconSize() + ".gif")));
            bScrollCenter.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/scrollcenter_" + options.getIconSize() + ".gif")));
            bZoomOut.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/zoomm_" + options.getIconSize() + ".gif")));
            bZoomIn.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/zoomp_" + options.getIconSize() + ".gif")));
            bAbout.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/manual_" + options.getIconSize() + ".gif")));
            if (!myUIConfiguration.isCriticalPathOn())
                bCritical.setDefaultIcon(new ImageIcon(getClass().getResource(
                        "/icons/criticalPathOff_" + options.getIconSize()
                                + ".gif")));
            else
                bCritical.setDefaultIcon(new ImageIcon(getClass().getResource(
                        "/icons/criticalPathOn_" + options.getIconSize()
                                + ".gif")));
            bComparePrev.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/comparePrev_" + options.getIconSize() + ".gif")));
            bSaveCurrent.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/saveCurrentAsPrev_" + options.getIconSize()
                            + ".gif")));
            bRefresh.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/" + myRefreshAction.getIconFilePrefix()
                            + options.getIconSize() + ".gif")));
            bShowHiddens.setDefaultIcon(new ImageIcon(getClass().getResource(
                    "/icons/showHiddens.gif")));
            for (int i = 0; i < myRolloverActions.size(); i++) {
                RolloverAction next = (RolloverAction) myRolloverActions.get(i);
                next.isIconVisible(true);
                next.setIconSize(options.getIconSize());

            }
        }
    }

    /** Create the button on toolbar */
    public void addButtons(JToolBar toolBar) {
        // toolBar.addSeparator(new Dimension(20,0));
        // /////////////////////////////////////////////////////////////////////////////////////////////////////////

        bExit = new TestGanttRolloverButton(new ImageIcon(getClass()
                .getResource("/icons/exit_" + options.getIconSize() + ".gif")));
        bExit.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                quitApplication();
            }
        });
        // if(isOnlyViewer)
        // {
        // toolBar.add(bExit);
        // toolBar.addSeparator(new Dimension(20,0));
        // }
        // /////////////////////////////////////////////////////////////////////////////////////////////////////////
        bNew = new TestGanttRolloverButton(new ImageIcon(getClass()
                .getResource("/icons/new_" + options.getIconSize() + ".gif")));
        bNew.addActionListener(this);
        // if(!isOnlyViewer) toolBar.add(bNew);
        // /////////////////////////////////////////////////////////////////////////////////////////////////////////
        bOpen = new TestGanttRolloverButton(new ImageIcon(getClass()
                .getResource("/icons/open_" + options.getIconSize() + ".gif")));
        bOpen.addActionListener(this);
        // toolBar.add(bOpen);
        // /////////////////////////////////////////////////////////////////////////////////////////////////////////
        bSave = new TestGanttRolloverButton(new ImageIcon(getClass()
                .getResource("/icons/save_" + options.getIconSize() + ".gif")));
        bSave.setEnabled(false);
        bSave.addActionListener(this);
        // toolBar.add(bSave);
        // /////////////////////////////////////////////////////////////////////////////////////////////////////////
        bSaveAs = new TestGanttRolloverButton(
                new ImageIcon(getClass().getResource(
                        "/icons/saveas_" + options.getIconSize() + ".gif")));
        bSaveAs.addActionListener(this);
        // toolBar.add(bSaveAs);
        // /////////////////////////////////////////////////////////////////////////////////////////////////////////
        bImport = new TestGanttRolloverButton(
                new ImageIcon(getClass().getResource(
                        "/icons/import_" + options.getIconSize() + ".gif")));
        bImport.addActionListener(this);
        // toolBar.add(bImport);
        // /////////////////////////////////////////////////////////////////////////////////////////////////////////
        bExport = new TestGanttRolloverButton(
                new ImageIcon(getClass().getResource(
                        "/icons/export_" + options.getIconSize() + ".gif")));
        bExport.addActionListener(this);
        // toolBar.add(bExport);
        // /////////////////////////////////////////////////////////////////////////////////////////////////////////
        bPrint = new TestGanttRolloverButton(new ImageIcon(getClass()
                .getResource("/icons/print_" + options.getIconSize() + ".gif")));
        bPrint.addActionListener(this);
        bPreviewPrint = new TestGanttRolloverButton(new ImageIcon(
                getClass().getResource(
                        "/icons/preview_" + options.getIconSize() + ".gif")));
        bPreviewPrint.addActionListener(this);
        // toolBar.add(bPrint);
        // /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // toolBar.addSeparator(new Dimension(20,0));
        // /////////////////////////////////////////////////////////////////////////////////////////////////////////
        bComparePrev = new TestGanttRolloverButton(
                new ImageIcon(getClass().getResource(
                        "/icons/comparePrev_" + options.getIconSize() + ".gif")));
        bComparePrev.setEnabled(false);
        bComparePrev.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                compareToPreviousState();
                if (myPreviousStates.size() == 0)
                    bComparePrev.setEnabled(false);
            }
        });
        // /////////////////////////////////////////////////////////////////////////////////////////////////////////
        bSaveCurrent = new TestGanttRolloverButton(new ImageIcon(getClass()
                .getResource(
                        "/icons/saveCurrentAsPrev_" + options.getIconSize()
                                + ".gif")));
        bSaveCurrent.setEnabled(false);
        bSaveCurrent.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                saveAsPreviousState();

            }
        });
        // /////////////////////////////////////////////////////////////////////////////////////////////////////////

        bCut = new TestGanttRolloverButton(new ImageIcon(getClass()
                .getResource("/icons/cut_" + options.getIconSize() + ".gif")));
        bCut.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                tree.cutSelectedNode();
                setAskForSave(true);
            }
        });
        // if(!isOnlyViewer)toolBar.add(bCut);
        bCopy = new TestGanttRolloverButton(new ImageIcon(getClass()
                .getResource("/icons/copy_" + options.getIconSize() + ".gif")));
        bCopy.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                tree.copySelectedNode();
            }
        });
        // if(!isOnlyViewer)toolBar.add(bCopy);
        bPaste = new TestGanttRolloverButton(new ImageIcon(getClass()
                .getResource("/icons/paste_" + options.getIconSize() + ".gif")));
        bPaste.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                tree.pasteNode();
                setAskForSave(true);
            }
        });

        // bCut = new TestGanttRolloverButton(new
        // ImageIcon(getClass().getResource("/icons/cut_"+options.getIconSize()+".gif")));
        // bCut.addActionListener(new ActionListener() {
        // public void actionPerformed(ActionEvent e) {
        // tree.cutSelectedNode();
        // setAskForSave(true);
        // }
        // });
        // bCut = new TestGanttRolloverButton(myCutAction);
        // if(!isOnlyViewer)toolBar.add(bCut);
        // bCopy = new TestGanttRolloverButton(new
        // ImageIcon(getClass().getResource("/icons/copy_"+options.getIconSize()+".gif")));
        // bCopy.addActionListener(new ActionListener() {
        // public void actionPerformed(ActionEvent e) {
        // tree.copySelectedNode();
        // }
        // });
        // bCopy = new TestGanttRolloverButton(myCopyAction);
        // if(!isOnlyViewer)toolBar.add(bCopy);
        // bPaste = new TestGanttRolloverButton(new
        // ImageIcon(getClass().getResource("/icons/paste_"+options.getIconSize()+".gif")));
        // bPaste.addActionListener(new ActionListener() {
        // public void actionPerformed(ActionEvent e) {
        // tree.pasteNode();
        // setAskForSave(true);
        // }
        // });
        // /bPaste = new TestGanttRolloverButton(myPasteAction);
        // if(!isOnlyViewer)
        {
            // toolBar.add(bPaste);
            // toolBar.addSeparator(new Dimension(20,0));
        }

        // /////////////////////////////////////////////////////////////////////////////////////////////////////////
        // bNewTask = new TestGanttRolloverButton(new
        // ImageIcon(getClass().getResource("/icons/insert_"+options.getIconSize()+".gif")));
        myNewArtefactAction = new NewArtefactAction(
                new NewArtefactAction.ActiveActionProvider() {
                    public AbstractAction getActiveAction() {
                        return getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX ? (AbstractAction) myNewTaskAction
                                : (AbstractAction) myNewHumanAction;

                    }
                }, options.getIconSize());
        bNewTask = new TestGanttRolloverButton(myNewArtefactAction);
        myRolloverActions.add(myNewArtefactAction);
        // bNewTask.addActionListener(new ActionListener() {
        // public void actionPerformed(ActionEvent e) {
        // if (tabpane.getSelectedIndex() == 0) {//Gantt Chart
        // newTask();
        // } else if (tabpane.getSelectedIndex() == 1) { //Resource chart
        // HumanResource people = new HumanResource();
        // GanttDialogPerson dp = new GanttDialogPerson(
        // GanttProject.this, getLanguage(), people);
        // dp.show();
        // if (dp.result()) {
        // getHumanResourceManager().add(people);
        // setAskForSave(true);
        // }
        // }
        // }
        // });
        // if(!isOnlyViewer) toolBar.add(bNewTask);
        // /////////////////////////////////////////////////////////////////////////////////////////////////////////
        bDelete = new TestGanttRolloverButton(
                new ImageIcon(getClass().getResource(
                        "/icons/delete_" + options.getIconSize() + ".gif")));
        bDelete.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX) {// Gantt
                    // Chart
                    // deleteTask();
                    deleteTasks(true);
                } else if (getTabs().getSelectedIndex() == UIFacade.RESOURCES_INDEX) { // Resource
                    // chart
                    final ProjectResource[] context = getResourcePanel()
                            .getContext().getResources();
                    if (context.length > 0) {
                        GanttDialogInfo gdi = new GanttDialogInfo(
                                GanttProject.this, GanttDialogInfo.QUESTION,
                                GanttDialogInfo.YES_NO_OPTION, getLanguage()
                                        .getText("msg6")
                                        + getDisplayName(context) + "??",
                                getLanguage().getText("question"));
                        gdi.show();
                        if (gdi.res == GanttDialogInfo.YES) {
                            for (int i = 0; i < context.length; i++) {
                                getHumanResourceManager().remove(context[i],
                                        getUndoManager());
                                refreshProjectInfos();
                                // setQuickSave(true);
                            }
                        }
                    }
                    deleteAssignements();
                }
            }
        });
        // if(!isOnlyViewer) toolBar.add(bDelete);
        // /////////////////////////////////////////////////////////////////////////////////////////////////////////
        bProperties = new TestGanttRolloverButton(new ImageIcon(getClass()
                .getResource(
                        "/icons/properties_" + options.getIconSize() + ".gif")));
        bProperties.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX) {// Gantt
                    // Chart
                    propertiesTask();
                } else if (getTabs().getSelectedIndex() == UIFacade.RESOURCES_INDEX) { // Resource
                    // chart
                    getResourcePanel().propertiesHuman(GanttProject.this);
                }
            }
        });
        // if(!isOnlyViewer) toolBar.add(bProperties);
        // /////////////////////////////////////////////////////////////////////////////////////////////////////////
        bUnlink = new TestGanttRolloverButton(
                new ImageIcon(getClass().getResource(
                        "/icons/unlink_" + options.getIconSize() + ".gif")));
        bUnlink.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX) {// Gantt
                    // Chart
                    unlinkRelationships();
                }
            }
        });
        // if(!isOnlyViewer) toolBar.add(bUnlink);
        // /////////////////////////////////////////////////////////////////////////////////////////////////////////
        bLink = new TestGanttRolloverButton(new ImageIcon(getClass()
                .getResource("/icons/link_" + options.getIconSize() + ".gif")));
        bLink.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX) {// Gantt
                    // Chart
                    linkRelationships();
                }
            }
        });
        // if(!isOnlyViewer) toolBar.add(bLink);
        // /////////////////////////////////////////////////////////////////////////////////////////////////////////
        bInd = new TestGanttRolloverButton(
                new ImageIcon(getClass().getResource(
                        "/icons/indent_" + options.getIconSize() + ".gif")));
        bInd.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX) {// Gantt
                    // Chart
                    // tree.indentCurrentNode();
                    tree.indentCurrentNodes();
                    setAskForSave(true);
                }
            }
        });
        // if(!isOnlyViewer) toolBar.add(bInd);
        // /////////////////////////////////////////////////////////////////////////////////////////////////////////
        bUnind = new TestGanttRolloverButton(new ImageIcon(getClass()
                .getResource(
                        "/icons/unindent_" + options.getIconSize() + ".gif")));
        bUnind.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX) {// Gantt
                    // Chart
                    // tree.dedentCurrentNode();
                    tree.dedentCurrentNodes();
                    setAskForSave(true);
                }
            }
        });
        // if(!isOnlyViewer) toolBar.add(bUnind);
        // /////////////////////////////////////////////////////////////////////////////////////////////////////////
        bUp = new TestGanttRolloverButton(new ImageIcon(getClass().getResource(
                "/icons/up_" + options.getIconSize() + ".gif")));
        bUp.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                upDatas();
            }
        });
        // if(!isOnlyViewer) toolBar.add(bUp);
        // /////////////////////////////////////////////////////////////////////////////////////////////////////////

        bDown = new TestGanttRolloverButton(new ImageIcon(getClass()
                .getResource("/icons/down_" + options.getIconSize() + ".gif")));
        bDown.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                downDatas();
            }
        });
        if (!isOnlyViewer) {
            // toolBar.add(bDown);
            // toolBar.addSeparator(new Dimension(20,0));
        }
        // /////////////////////////////////////////////////////////////////////////////////////////////////////////
        ScrollingManager scrollingManager = getScrollingManager();
        scrollingManager.addScrollingListener(area.getViewState());
        scrollingManager.addScrollingListener(getResourcePanel().area
                .getViewState());
        Action scrollLeft = new ScrollGanttChartLeftAction(scrollingManager,
                options.getIconSize());
        myRolloverActions.add(scrollLeft);
        bPrev = new TestGanttRolloverButton(scrollLeft);
        bPrev.setAutoRepeatMousePressedEvent(300);
        // toolBar.add(bPrev);

        Action scrollCenter = area.getScrollCenterAction(scrollingManager,
                Mediator.getTaskSelectionManager(), options.getIconSize());
        myRolloverActions.add(scrollCenter);
        bScrollCenter = new TestGanttRolloverButton(scrollCenter);
        bScrollCenter.setAutoRepeatMousePressedEvent(300);
        // toolBar.add(bScrollCenter);

        Action scrollRight = new ScrollGanttChartRightAction(scrollingManager,
                options.getIconSize());
        myRolloverActions.add(scrollRight);
        bNext = new TestGanttRolloverButton(scrollRight);
        bNext.setAutoRepeatMousePressedEvent(300);
        // toolBar.add(bNext);

        // /////////////////////////////////////////////////////////////////////////////////////////////////////////
        Action zoomOut = new ZoomOutAction(getZoomManager(), options
                .getIconSize());
        myRolloverActions.add(zoomOut);
        bZoomOut = new TestGanttRolloverButton(zoomOut);
        // toolBar.add(bZoomOut);

        // bZoomOut = new TestGanttRolloverButton(new
        // ImageIcon(getClass().getResource("/icons/zoomm_"+options.getIconSize()+".gif")));

        // /////////////////////////////////////////////////////////////////////////////////////////////////////////
        Action zoomIn = new ZoomInAction(getZoomManager(), options
                .getIconSize());
        myRolloverActions.add(zoomIn);
        bZoomIn = new TestGanttRolloverButton(zoomIn);
        // toolBar.add(bZoomIn);
        // /////////////////////////////////////////////////////////////////////////////////////////////////////////
        bAbout = new TestGanttRolloverButton(
                new ImageIcon(getClass().getResource(
                        "/icons/manual_" + options.getIconSize() + ".gif")));
        bAbout.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                aboutDialog();
            }
        });
        if (isOnlyViewer) {
            // toolBar.addSeparator(new Dimension(20,0));
            // toolBar.add(bAbout);
        }
        // /////////////////////////////////////////////////////////////////////////////////////////////////////////
        /*
         * bZoomFit = new JButton (new
         * ImageIcon(getClass().getResource("/icons/zoomf.gif")));
         * bZoomFit.addActionListener(new ActionListener() { public void
         * actionPerformed(ActionEvent e) { area.fitWholeProject(false);
         * area.repaint(); } }); toolBar.add(bZoomFit);
         */
        Action undo = new UndoAction(getUndoManager(), options.getIconSize());
        myRolloverActions.add(undo);
        bUndo = new TestGanttRolloverButton(undo);

        // bUndo = new TestGanttRolloverButton(new
        // ImageIcon(getClass().getResource("/icons/undo_"+options.getIconSize()+".gif")));
        // bUndo.setEnabled(false);
        // bUndo.addActionListener(new ActionListener() {
        // public void actionPerformed(ActionEvent e) {
        // undo ();}});
        Action redo = new RedoAction(getUndoManager(), options.getIconSize());
        myRolloverActions.add(redo);
        bRedo = new TestGanttRolloverButton(redo);
        // bRedo = new TestGanttRolloverButton(new
        // ImageIcon(getClass().getResource("/icons/redo_"+options.getIconSize()+".gif")));
        // bRedo.setEnabled(false);
        // bRedo.addActionListener(new ActionListener() {
        // public void actionPerformed(ActionEvent e) {
        // redo ();}});

        // toolBar.addSeparator(new Dimension(20,0));
        // toolBar.add(bUndo);
        // toolBar.add(bRedo);

        Action critic = new CalculateCriticalPathAction(getTaskManager(), tree,
                options, getUIConfiguration());
        myRolloverActions.add(critic);
        bCritical = new TestGanttRolloverButton(critic);
        // bCritical.setFocusPainted(false);
        // toolBar.addSeparator(new Dimension(20,0));
        // toolBar.add(bCritical);

        // myRolloverActions.add(myRefreshAction);
        // bRefresh = new TestGanttRolloverButton(myRefreshAction);
        bRefresh = new TestGanttRolloverButton(new ImageIcon(
                getClass().getResource(
                        "/icons/refresh_" + options.getIconSize() + ".gif")));
        bRefresh.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                getUIFacade().refresh();
            }
        });

        bShowHiddens = new TestGanttRolloverButton(new ImageIcon(getClass()
                .getResource("/icons/showHiddens.gif")));
        bShowHiddens.addActionListener(this);
        iconList = initIconList();
        deletedIconList = initDeletedIconList();
        addButtons();
        applyButtonOptions();
    }

    protected void saveAsPreviousState() {
        GanttDialogSaveAsPreviousState ps = new GanttDialogSaveAsPreviousState(
                this);
        ps.show();
        if (ps.isSaved()) {
            bSaveCurrent.setEnabled(false);
            bComparePrev.setEnabled(true);
            myPreviousStates.add(ps.getPreviousState());
        }

    }

    public ArrayList getPreviouStates() {
        return myPreviousStates;
    }

    protected void compareToPreviousState() {
        GanttDialogCompareToPreviousState cp = new GanttDialogCompareToPreviousState(
                this);
        cp.show();
    }

    /**
     * Adds a separator.
     */
    /*
     * private static void addToolBarSeparator(JToolBar toolbar, int width, int
     * height) { JPanel sepL = new JPanel(); sepL.setMaximumSize(new
     * Dimension(width/2,height)); toolbar.add(sepL); JPanel sep = new JPanel();
     * sep.setMaximumSize(new Dimension(1,height)); sep.setMinimumSize(new
     * Dimension(1,height)); sep.setPreferredSize(new Dimension(1,height));
     * sep.setBorder(BorderFactory.createLineBorder(Color.GRAY));
     * toolbar.add(sep); JPanel sep2 = new JPanel(); sep2.setMaximumSize(new
     * Dimension(1,height)); sep2.setMinimumSize(new Dimension(1,height));
     * sep2.setPreferredSize(new Dimension(1,height));
     * sep2.setBorder(BorderFactory.createLineBorder(Color.WHITE));
     * toolbar.add(sep2); JPanel sepR = new JPanel(); sepR.setMaximumSize(new
     * Dimension(width/2,height)); toolbar.add(sepR); }
     */

    private void aboutDialog() {
        AboutDialog agp = new AboutDialog(this);
        agp.show();
    }

    private String getDisplayName(Object[] objs) {
        if (objs.length == 1) {
            return objs[0].toString();
        }
        StringBuffer result = new StringBuffer();
        for (int i = 0; i < objs.length; i++) {
            result.append(objs[i].toString());
            if (i < objs.length - 1) {
                result.append(", ");
            }
        }
        return result.toString();
    }

    /** Move up the datas (resources or tasks) */
    private void upDatas() {
        if (getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX) {// Gantt
            // Chart
            tree.upCurrentNodes();
        } else if (getTabs().getSelectedIndex() == UIFacade.RESOURCES_INDEX) { // Resource
            // chart
            getResourcePanel().upResource();
            // getResourcePanel().setPeople(getResourcePanel().getPeople());
            getResourcePanel().area.repaint();
        }
        setAskForSave(true);
    }

    /** Move down datas (resources or tasks) */
    private void downDatas() {
        if (getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX) { // Gantt
            // Chart
            tree.downCurrentNodes();
        } else if (getTabs().getSelectedIndex() == UIFacade.RESOURCES_INDEX) { // Resource
            // chart
            getResourcePanel().downResource();
            // getResourcePanel().setPeople(getResourcePanel().getPeople());
            getResourcePanel().area.repaint();
        }
        setAskForSave(true);
    }

    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {
        quitApplication();
    }

    /**
     * Check if the project has been modified, before creating a new one or open
     * another
     */
    public boolean checkCurrentProject() {
        GanttDialogInfo gdi = new GanttDialogInfo(this,
                GanttDialogInfo.WARNING, GanttDialogInfo.YES_NO_CANCEL_OPTION,
                language.getText("msg1"), language.getText("warning"));
        GanttDialogInfo gdiSaveError = new GanttDialogInfo(this,
                GanttDialogInfo.ERROR, GanttDialogInfo.YES_NO_CANCEL_OPTION,
                language.getText("msg12"), language.getText("error"));
        if (askForSave == true) {
            gdi.show();
            if (gdi.res == GanttDialogInfo.CANCEL)
                return false;
            if (gdi.res == GanttDialogInfo.YES) {
                boolean trySave = true;
                do {
                    try {
                        trySave = false;
                        saveProject();
                    } catch (Exception e) {
                        System.err.println(e);
                        gdiSaveError.show();
                        if (gdiSaveError.res == GanttDialogInfo.CANCEL)
                            return false;
                        trySave = (gdiSaveError.res == GanttDialogInfo.YES);
                    }
                } while (trySave);
            }
        }
        return true;
    }

    /** A menu has been activate */
    public void actionPerformed(ActionEvent evt) {
        if (evt.getSource() instanceof JMenuItem) {
            String arg = evt.getActionCommand();
            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            if (arg.equals(correctLabel(language.getText("newProject")))) {
                newProject();
            }
            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            else if (arg.equals(correctLabel(language.getText("openProject")))) {
                try {
                    if (checkCurrentProject()) {
                        openFile();
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    if (isVisible()) {
                        GanttDialogInfo gdi = new GanttDialogInfo(this,
                                GanttDialogInfo.ERROR,
                                GanttDialogInfo.YES_OPTION, language
                                        .getText("msg8"), language
                                        .getText("error"));
                        gdi.show();
                    } else
                        System.out.println("\n====" + language.getText("error")
                                + "====\n" + language.getText("msg8") + "\n");
                }
            }
            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            else if (arg
                    .equals(correctLabel(language.getText("openFromServer")))) {
                if (checkCurrentProject()) {
                    openURL();
                }
            }
            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            else if (arg.equals(correctLabel(language.getText("saveProject")))) {
                try {
                    saveProject();
                } catch (Exception e) {
                    System.err.println(e);
                }
            }
            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            else if (arg
                    .equals(correctLabel(language.getText("saveAsProject")))) {
                try {
                    saveAsProject();
                } catch (Exception e) {
                    System.err.println(e);
                }
            }
            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            else if (arg.equals(correctLabel(language.getText("saveToServer")))) {
                try {
                    saveAsURLProject();
                } catch (Exception e) {
                    System.err.println(e);
                }
            }
            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            // else if (arg.equals(correctLabel(language.getText("export")))) {
            // export();}
            else if (arg.equals(correctLabel(language.getText("publishToWeb")))) {
                ExportFileInfo info = new ExportFileInfo(null,
                        ExportFileInfo.FORMAT_HTML, options.getExportSettings());
                statusBar.setFirstText(language.getText("htmlexport"), 2000);
                // construct URL from settings
                StringBuffer urlString = new StringBuffer();
                urlString.append("ftp://");
                urlString.append(options.getFTPUser());
                urlString.append(":");
                urlString.append(options.getFTPPwd());
                urlString.append("@");
                urlString.append(options.getFTPUrl());
                urlString.append("/");
                urlString.append(options.getFTPDirectory() == "" ? "" : options
                        .getFTPDirectory()
                        + "/");

                int confirm = JOptionPane.showConfirmDialog(null, language
                        .getText("confirmFTPTransfer_1")
                        + " "
                        + urlString.substring(urlString.indexOf("@") + 1)
                        + "\n" + language.getText("confirmFTPTransfer_2"),
                        language.getText("confirm"), JOptionPane.YES_NO_OPTION);
                /*
                 * if (confirm == 0) { try { GanttHTMLExport.publish(
                 * urlString.toString(), prjInfos, this, tree, area,
                 * getResourcePanel().area,
                 * statusBar.setFirstText(language.getText("successFTPTransfer"),
                 * 2000); info.myStorageOptions);
                 * JOptionPane.showMessageDialog(null,
                 * language.getText("successFTPTransfer"),
                 * language.getText("success") ,
                 * JOptionPane.INFORMATION_MESSAGE); } catch (IOException e) {
                 * e.printStackTrace(); JOptionPane.showMessageDialog(null,
                 * e.getMessage(), language.getText("error"),
                 * JOptionPane.ERROR_MESSAGE); } }
                 */
            }

            else if (arg.equals(language.getText("hideTask"))) {
                tree.hideSelectedNodes();
            } else if (arg.equals(language.getText("displayHiddenTasks")))
                tree.displayHiddenTasks();

            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            else if (arg.equals(correctLabel(language.getText("printProject")))) {
                printProject();
            }
            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            else if (arg.equals(correctLabel(language.getText("preview")))) {
                previewPrint();
            }
            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            else if (arg.equals(correctLabel(language.getText("quit")))) {
                quitApplication();
            }
            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            else if (arg.equals(correctLabel(language.getText("deleteTask")))) {
                deleteTasks(true);
            }
            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            else if (arg
                    .equals(correctLabel(language.getText("propertiesTask")))) {
                propertiesTask();
            }
            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            else if (arg.equals(correctLabel(language.getText("upTask")))) {
                upDatas();
            }
            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            else if (arg.equals(correctLabel(language.getText("downTask")))) {
                downDatas();
            }
            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            else if (arg.equals(correctLabel(language.getText("indentTask")))) {
                tree.indentCurrentNodes();
                setAskForSave(true);
            }
            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            else if (arg.equals(correctLabel(language.getText("dedentTask")))) {
                tree.dedentCurrentNodes();
                setAskForSave(true);
            }
            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            else if (arg.equals(correctLabel(language.getText("unlink")))) {
                unlinkRelationships();
            }
            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            else if (arg.equals(correctLabel(language.getText("link")))) {
                linkRelationships();
            }
            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            else if (arg.equals(correctLabel(language
                    .getText("propertiesHuman")))) {
                getTabs().setSelectedIndex(UIFacade.RESOURCES_INDEX);
                getResourcePanel().propertiesHuman(this);
            }
            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            else if (arg
                    .equals(correctLabel(language.getText("editCalendars")))) {
                GanttDialogCalendar dialogCalendar = new GanttDialogCalendar(
                        this);
                dialogCalendar.show();
            }
            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            else if (arg.equals(correctLabel(language
                    .getText("projectCalendar")))) {
                System.out.println("Project calendar");
            }
            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            else if (arg.equals(correctLabel(language.getText("webPage")))) {
                try {
                    openWebPage();
                } catch (Exception e) {
                    System.err.println(e);
                }
            }
            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            else if (arg.equals(correctLabel(language.getText("tipsOfTheDay")))) {
                TipsDialog tips = new TipsDialog(this, options.getOpenTips());
                tips.show();
            }
            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            else if (arg.equals(correctLabel(language.getText("manual")))) {
                if (helpFrame == null) {
                    try {
                        helpFrame = new net.sourceforge.helpgui.gui.MainFrame(
                                "/docs/help/", "eclipse");
                        // french translation for the user manual
                        if (language.getText("shortLanguage").equals("fr"))
                            helpFrame = new net.sourceforge.helpgui.gui.MainFrame(
                                    "/docs/help_fr/", "eclipse");

                        helpFrame.setTitle(language.getText("ganttManual"));
                        ImageIcon icon = new ImageIcon(getClass().getResource(
                                "/icons/ganttproject.png"));
                        helpFrame.setIconImage(icon.getImage());
                        helpFrame.setSize((int) (Toolkit.getDefaultToolkit()
                                .getScreenSize().getWidth() * 0.75),
                                (int) (Toolkit.getDefaultToolkit()
                                        .getScreenSize().getHeight() * 0.75));
                        helpFrame.setLocationRelativeTo(null);
                    } catch (Exception e) {
                    }
                }
                helpFrame.setVisible(true);
            }
            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            else if (arg.equals(correctLabel(language.getText("about")))) {
                aboutDialog();
            }
            // /////////////////////////////////////////////////////////////////////////////////////////////////////////
            else if (arg.equals(correctLabel(language.getText("sendMail")))) {
                getTabs().setSelectedIndex(1);
                getResourcePanel().sendMail(this);
            }
            // Newly added code /CL
            // //////////////////////////////////////////////////////////////////////////////////////////////////////////
            // else if (arg.equals(correctLabel(language.getText("cut")))) {
            // tree.cutSelectedNode();
            // setAskForSave(true);
            // }
            // ////////////////////////////////////////////////////////////////////////////////////////////////////////////
            // else if (arg.equals(correctLabel(language.getText("copy")))) {
            // tree.copySelectedNode();
            // }
            // ////////////////////////////////////////////////////////////////////////////////////////////////////////////
            // else if (arg.equals(correctLabel(language.getText("paste")))) {
            // tree.pasteNode();
            // setAskForSave(true);
            // }
            // //////////////////////////////////////////////////////////////////////////////////////////////////////////
            else if (arg.equals(correctLabel(language.getText("settings")))) {
                launchOptionsDialog();
            }
            // //////////////////////////////////////////////////////////////////////////////////////////////////////////
            // Test if it's a file name
        } else if (evt.getSource() instanceof Document) {
            if (checkCurrentProject()) {
                final ActionEvent ae = evt;
                getUndoManager().undoableEdit("StartupDocument",
                        new Runnable() {
                            public void run() {
                                openStartupDocument((Document) ae.getSource());
                            }
                        });
            }
        } // //////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Test if it's buttons actions
        else if (evt.getSource() instanceof JButton) {
            if (evt.getSource() == bNew) // new
                newProject();
            else if (evt.getSource() == bOpen) { // open
                try {
                    if (checkCurrentProject())
                        openFile();
                } catch (Exception ex) {
                    System.err.println(ex);
                }
            } else if (evt.getSource() == bSave) { // save
                try {
                    saveProject();
                } catch (Exception ex) {
                    System.err.println(ex);
                }
            } else if (evt.getSource() == bSaveAs) { // saveas
                try {
                    saveAsProject();
                } catch (Exception ex) {
                    System.err.println(ex);
                }
            } else if (evt.getSource() == bExport) { // export
                ExportFileWizardImpl wizard = new ExportFileWizardImpl(this
                        .getUIFacade(), this, this.getOptions());
                wizard.show();
            } else if (evt.getSource() == bImport) { // import
                ImportFileWizardImpl wizard = new ImportFileWizardImpl(this
                        .getUIFacade(), this);
                wizard.show();
            } else if (evt.getSource() == bPrint) { // print
                printProject();
            } else if (evt.getSource() == bPreviewPrint) { // print
                previewPrint();
            } else if (evt.getSource() == bShowHiddens) {
                showHiddenButtonsPaneMenu();
            }
        }
        // repaint();
    }

    /** Launch the options dialog */
    public void launchOptionsDialog() {

        // old options dialog box
        /*
         * GanttDialogOptions dialogOptions = new GanttDialogOptions(this,
         * myUIConfiguration); dialogOptions.show(); if (dialogOptions.change) {
         * setAskForSave(true); }
         */

        // new options dialog box
        statusBar.setFirstText(language.getText("settingsPreferences"), 2000);
        SettingsDialog dialogOptions = new SettingsDialog(this);
        dialogOptions.show();
        getResourcePanel().reloadRoles();
        area.repaint();
    }

    public ProjectResource newHumanResource() {
        final HumanResource people = ((HumanResourceManager) getHumanResourceManager())
                .newHumanResource();
        people.setRole(getRoleManager().getDefaultRole());
        GanttDialogPerson dp = new GanttDialogPerson(this, getLanguage(),
                people);
        dp.setVisible(true);
        if (dp.result()) {

            getUndoManager().undoableEdit("new Resource", new Runnable() {
                public void run() {
                    getHumanResourceManager().add(people);
                }
            });
        }
        return people;
    }

    /** Create a new task */

    public Task newTask() {

        getTabs().setSelectedIndex(UIFacade.GANTT_INDEX);

        int index = -1;
        MutableTreeNode selectedNode = (MutableTreeNode) Mediator
                .getGanttProjectSingleton().getTree().getSelectedNode();
        if (selectedNode != null) {
            DefaultMutableTreeNode parent1 = (DefaultMutableTreeNode) selectedNode
                    .getParent();
            index = parent1.getIndex(selectedNode) + 1;
            tree.getTreeTable().getTree().setSelectionPath(
                    new TreePath(parent1.getPath()));
            tree.getTreeTable().getTreeTable().editingStopped(
                    new ChangeEvent(tree.getTreeTable().getTreeTable()));
        }

        // TODO check if the following code (now commentted) is really
        // necessary...
        // boolean editing = tree.getTreeTable().getTreeTable().isEditing();
        // TaskNode tn = (TaskNode) tree.getSelectedNode();
        // TaskNode parent;
        // if (editing) {
        // System.out.println("Editing");
        // if (tn != null) {
        // if (tn.isRoot())
        // parent = tn;
        // else
        // parent = (TaskNode) tn.getParent();
        // Task t = (Task) parent.getUserObject();
        // tree.selectTask(t, false);
        // }
        // }

        // GanttTask current = tree.getSelectedTask();
        GanttCalendar cal = new GanttCalendar(area.getViewState()
                .getStartDate());

        DefaultMutableTreeNode node = tree.getSelectedNode();
        GanttLanguage lang = GanttLanguage.getInstance();
        String nameOfTask = options.getTaskNamePrefix(); // language.getText("newTask");
        // if (current != null) {
        // current.setMilestone(false);
        // node = (TaskNode) tree.getSelectedNode();
        // cal = current.getStart();
        // if (!node.isRoot())
        // nameOfTask = current.toString();
        // }
        GanttTask task = getTaskManager().createTask();
        task.setStart(cal);
        task.setLength(1);
        getTaskManager().registerTask(task);// create a new task in the tab
        // paneneed to register it
        task.setName(nameOfTask + "_" + task.getTaskID());
        task.setColor(area.getTaskColor());
        // if (current != null) {
        // if (current.colorDefined()) {
        // task.setColor(current.getColor());
        // }
        // if (current.shapeDefined())
        // task.setShape(current.getShape());
        // }
        TaskNode taskNode = tree.addObject(task, node, index);

        /*
         * this will add new custom columns to the newly created task.
         */
        Mediator.getCustomColumnsStorage().processNewTask(task);

        AdjustTaskBoundsAlgorithm alg = getTaskManager()
                .getAlgorithmCollection().getAdjustTaskBoundsAlgorithm();
        alg.run(task);
        RecalculateTaskCompletionPercentageAlgorithm alg2 = getTaskManager()
                .getAlgorithmCollection()
                .getRecalculateTaskCompletionPercentageAlgorithm();
        alg2.run(task);
        // refresh the differents tasks
        // if (current != null) {
        // tree.refreshAllChild(nameOfTask);
        // // DefaultMutableTreeNode father = tree.getSelectedNode();
        // // GanttTask taskFather = null;
        // // For refresh all the parent task
        // // while (tree.getNode(task.getTaskID()).isRoot() == false) {
        // // father = tree.getFatherNode(task);
        // // tree.refreshAllChild(father.toString());
        // // taskFather = (GanttTask) father.getUserObject();
        // // taskFather.refreshDateAndAdvancement(tree);
        // // father.setUserObject(taskFather);
        // // task = taskFather;
        // // }
        // }
        area.repaint();
        setAskForSave(true);
        statusBar.setFirstText(language.getText("createNewTask"), 1000);
        if (options.getAutomatic()) {
            propertiesTask(new DefaultMutableTreeNode[] { taskNode });
        }
        // setQuickSave(true);
        tree.setEditingTask(task);
        repaint2();
        return task;
    }

    public void deleteResources() {
        myDeleteHumanAction.actionPerformed(null);
    }

    public void deleteAssignements() {
        myDeleteAssignmentAction.actionPerformed(null);
    }

    /**
     * Delete the currant task
     * 
     * @param confirmation
     *            TODO
     */
    public void deleteTasks(boolean confirmation) {
        getTabs().setSelectedIndex(UIFacade.GANTT_INDEX);

        final DefaultMutableTreeNode[] cdmtn = tree.getSelectedNodes();
        if (cdmtn == null || cdmtn.length == 0) {
            statusBar.setFirstText(language.getText("msg21"), 2000);
            return;
        }

        GanttDialogInfo gdi = new GanttDialogInfo(this,
                GanttDialogInfo.QUESTION, GanttDialogInfo.YES_NO_OPTION,
                language.getText("msg19"), language.getText("question"));

        if (confirmation)
            gdi.show();
        else
            gdi.res = GanttDialogInfo.YES;

        if (gdi.res == GanttDialogInfo.YES) {
            getUndoManager().undoableEdit("Task removed", new Runnable() {
                public void run() {
                    ArrayList fathers = new ArrayList();
                    for (int i = 0; i < cdmtn.length; i++) {
                        if (cdmtn[i] != null && cdmtn[i] instanceof TaskNode) {
                            Task ttask = (Task) (cdmtn[i].getUserObject());

                            DefaultMutableTreeNode father = tree
                                    .getFatherNode(ttask);
                            tree.removeCurrentNode();
                            if (father != null) {
                                GanttTask taskFather = (GanttTask) father
                                        .getUserObject();
                                AdjustTaskBoundsAlgorithm alg = getTaskManager()
                                        .getAlgorithmCollection()
                                        .getAdjustTaskBoundsAlgorithm();
                                alg.run(taskFather);
                                // taskFather.refreshDateAndAdvancement(tree);
                                father.setUserObject(taskFather);
                                fathers.add(father);
                            }
                            getTaskManager().deleteTask(ttask);
                            ttask.delete();
                        } else if (cdmtn[i] != null
                                && cdmtn[i] instanceof BlankLineNode) {
                            ((GanttTreeTableModel) tree.getTreeTable()
                                    .getTreeTableModel())
                                    .removeNodeFromParent(cdmtn[i]);
                        }

                    }
                    for (int i = 0; i < fathers.size(); i++) {
                        DefaultMutableTreeNode father = (DefaultMutableTreeNode) fathers
                                .get(i);
                        if (father.getChildCount() == 0)
                            ((Task) father.getUserObject())
                                    .setProjectTask(false);
                    }

                }
            });
            refreshProjectInfos();
            area.repaint();
            this.repaint2();
            getResourcePanel().area.repaint();
            setAskForSave(true);
            // setQuickSave (true);
            // quickSave("deleteTasks");
        }
    }

    /** Edit task parameters */
    public void propertiesTask() {
        getTabs().setSelectedIndex(UIFacade.GANTT_INDEX);
        if (tree.getSelectedTaskNodes() != null
                && tree.getSelectedTaskNodes().length > 0)
            propertiesTask(tree.getSelectedTaskNodes());
    }

    /** Edit task parameters */

    public void propertiesTask(DefaultMutableTreeNode[] nodes) {
        final int rowHeight = tree.getTreeTable().getTable().getRowHeight();
        if (nodes == null || nodes[0].isRoot()) {
            statusBar.setFirstText(language.getText("msg21"), 2000);
            return;
        } else {

            statusBar.setFirstText(language.getText("editingParameters"), 2000);
            int nbTasks = nodes.length;
            String tasksNames = "";
            for (int i = 0; i < nodes.length; i++)
                if (nodes[i].isRoot()) {
                    nbTasks = nbTasks - 1;
                    break;
                }

            GanttTask[] tasks = new GanttTask[nbTasks];
            for (int i = 0; i < nodes.length; i++)
                if (!nodes[i].isRoot()) {
                    tasks[i] = (GanttTask) (nodes[i].getUserObject());
                    if (!tasksNames.equals(""))
                        tasksNames = tasksNames + ", ";
                    tasksNames = tasksNames + tasks[i].getName();
                }

            GanttDialogProperties pd = new GanttDialogProperties(this, tree,
                    managerHash, tasks, area, tasksNames);
            pd.show();
            if (pd.change) {
                setAskForSave(true);
                setRowHeight(rowHeight);
                getResourcePanel().getResourceTreeTableModel()
                        .updateResources();
            }
        }
    }

    /** Unlink the relationships of the selected task */
    public void unlinkRelationships() {
        getTabs().setSelectedIndex(UIFacade.GANTT_INDEX);

        final DefaultMutableTreeNode[] cdmtn = tree.getSelectedTaskNodes();
        if (cdmtn == null) {
            statusBar.setFirstText(language.getText("msg21"), 2000);
            return;
        }

        getUndoManager().undoableEdit("Unlink", new Runnable() {
            public void run() {
                for (int i = 0; i < cdmtn.length; i++) {
                    if (cdmtn[i] != null && !cdmtn[i].isRoot()) {
                        GanttTask t = (GanttTask) (cdmtn[i].getUserObject());
                        t.unlink();
                    }
                    // setQuickSave (true);
                }
            }
        });
        area.repaint();
        setAskForSave(true);
        /*
         * JFileChooser fc = new JFileChooser(options.getWorkingDir());
         * FileFilter ganttFilter = new GanttXMLFileFilter();
         * fc.addChoosableFileFilter(ganttFilter); InputStream inputStream =
         * null; int returnVal = fc.showOpenDialog(GanttProject.this); if
         * (returnVal == JFileChooser.APPROVE_OPTION) { final JFileChooser jfc =
         * fc; try { Document document = new FileDocument (new File
         * (jfc.getSelectedFile().getAbsolutePath())); inputStream =
         * document.getInputStream(); //System.out.println (inputStream.read()); }
         * catch (IOException e) { // TODO Auto-generated catch block
         * e.printStackTrace(); } }
         */

    }

    /** Link the selected Tasks */
    public void linkRelationships() {
        getTabs().setSelectedIndex(UIFacade.GANTT_INDEX);

        final DefaultMutableTreeNode[] cdmtn = tree.getSelectedTaskNodes();
        if (cdmtn == null) {
            statusBar.setFirstText(language.getText("msg21"), 2000);
            return;
        }

        if (cdmtn.length < 2) {
            statusBar.setFirstText(language.getText("msg22"), 2000);
            return;
        }

        getUndoManager().undoableEdit("Link", new Runnable() {
            public void run() {
                for (int i = 0; i < cdmtn.length - 1; i++) {
                    if (cdmtn[i] != null && cdmtn[i + 1] != null) {
                        GanttTask firstTask = (GanttTask) (cdmtn[i]
                                .getUserObject());
                        ;
                        GanttTask secondTask = (GanttTask) (cdmtn[i + 1]
                                .getUserObject());
                        try {
                            getTaskManager().getDependencyCollection()
                                    .createDependency(secondTask, firstTask,
                                            new FinishStartConstraintImpl());
                        } catch (TaskDependencyException e1) {
                            // e1.printStackTrace(); //To change body of catch
                            // statement use File | Settings | File Templates.
                        }
                    }
                }
            }
        });

        // setQuickSave (true);
        area.repaint();
        setAskForSave(true);
    }

    /** Export the calendar on a png file */
    public void export() {
        ExportFileInfo info = selectExportFile(null);
        if (!info.equals(ExportFileInfo.EMPTY)) {
            doExport(info);
        }
    }

    private void doExport(ExportFileInfo info) {
        Chart resourcesChart = (Chart) getResourcePanel().area;
        Chart ganttChart = (Chart) getArea();
        Chart visibleChart = (getTabs().getSelectedIndex() == UIFacade.RESOURCES_INDEX) ? resourcesChart
                : ganttChart;
        ExportFileWizardImpl wizard = new ExportFileWizardImpl(info,
                getUIFacade(), (IGanttProject) this, ganttChart,
                resourcesChart, visibleChart, options);
        wizard.doExport();
    }

    /** Execute the export functions. */

    /** Refresh the informations of the project on the status bar. */
    public void refreshProjectInfos() {
        if (tree.nbTasks == 0 && resp.nbPeople() == 0)
            statusBar.setSecondText("");
        else
            statusBar.setSecondText(correctLabel(language.getText("task"))
                    + " : " + tree.nbTasks + "  "
                    + correctLabel(language.getText("resources")) + " : "
                    + resp.nbPeople());
    }

    private ExportFileInfo selectExportFile(FileFilter currentFilter) {
        ExportFileInfo result = ExportFileInfo.EMPTY;
        FileFilter figFilter = new GanttXFIGFileFilter();
        FileFilter pngFilter = new GanttPNGFileFilter();
        FileFilter jpgFilter = new GanttJPGFileFilter();
        FileFilter csvFilter = new GanttCSVFilter();
        FileFilter pdfFilter = new GanttPDFFileFilter();
        FileFilter htmlFilter = new GanttHTMLFileFilter();
        FileFilter msprojectFilter = new GanttMSProjectLFileFilter();
        JFileChooser fc = new JFileChooser(options.getWorkingDir());
        fc.addChoosableFileFilter(figFilter);
        fc.addChoosableFileFilter(msprojectFilter);
        fc.addChoosableFileFilter(pngFilter);
        fc.addChoosableFileFilter(jpgFilter);
        fc.addChoosableFileFilter(csvFilter);
        fc.addChoosableFileFilter(pdfFilter);
        fc.addChoosableFileFilter(htmlFilter);
        if (currentFilter != null)
            fc.setFileFilter(currentFilter);

        // Remove the possibility to use a file filter for all files
        FileFilter[] filefilters = fc.getChoosableFileFilters();
        for (int i = 0; i < filefilters.length; i++) {
            if (filefilters[i] != figFilter && filefilters[i] != pngFilter
                    && filefilters[i] != jpgFilter
                    && filefilters[i] != csvFilter
                    && filefilters[i] != pdfFilter
                    && filefilters[i] != htmlFilter
                    && filefilters[i] != msprojectFilter)
                fc.removeChoosableFileFilter(filefilters[i]);
        }

        int returnVal = fc.showSaveDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            String filename = fc.getSelectedFile().toString();
            FileFilter selectedFilter = fc.getFileFilter();

            // if( the file exists, ask for overwriting
            if (new File(filename).exists()) {
                GanttDialogInfo gdi = new GanttDialogInfo(this,
                        GanttDialogInfo.WARNING, GanttDialogInfo.YES_NO_OPTION,
                        filename + "\n" + language.getText("msg18"), language
                                .getText("warning"));
                gdi.show();
                if (gdi.res == GanttDialogInfo.NO)
                    return selectExportFile(selectedFilter);
            }

            int type = -1;
            if (selectedFilter.equals(htmlFilter)) {
                type = ExportFileInfo.FORMAT_HTML;
                if (!fc.getFileFilter().accept(new File(filename))) {
                    filename += ".html";
                }
            } else if (selectedFilter.equals(pngFilter)) {
                type = ExportFileInfo.FORMAT_PNG;
            } else if (selectedFilter.equals(jpgFilter)) {
                type = ExportFileInfo.FORMAT_JPG;
            } else if (selectedFilter.equals(pdfFilter)) {
                type = ExportFileInfo.FORMAT_PDF;
            } else if (selectedFilter.equals(figFilter)) {
                type = ExportFileInfo.FORMAT_XFIG;
            } else if (selectedFilter.equals(csvFilter)) {
                type = ExportFileInfo.FORMAT_CSV;
            } else if (selectedFilter.equals(msprojectFilter)) {
                type = ExportFileInfo.FORMAT_MSPROJECT;
            } else {
                statusBar.setFirstText(
                        "Unknown file filter has been selected : "
                                + selectedFilter, 2000);
                throw new RuntimeException(
                        "Unknown file filter has been selected: "
                                + selectedFilter);
            }
            changeWorkingDirectory(new File(filename).getParent());
            File file = new File(filename);

            result = new ExportFileInfo(file, type, options.getExportSettings());

        }
        return result;
    }

    /** Print the project */
    public void printProject() {

        Object selectedUserObject = getTabs().getSelectedUserObject();
        if (selectedUserObject instanceof Chart)
            PrintManager.printChart((Chart) selectedUserObject, options
                    .getExportSettings());
        else
            System.err.println("not printable");

        // if (getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX) {// Gantt
        // // Chart
        // GanttExportSettings bool = new GanttExportSettings();
        // // GanttDialogExport gde = new GanttDialogExport(this, bool,
        // // language);
        // // gde.show();
        // // if (bool.ok)
        // // area.printProject(bool.name, bool.percent, bool.depend);
        // GanttExportSettings ges = options.getExportSettings();
        // // ges.setStartDate(area.getChartModel().getStartDate());
        // // ges.setEndDate(area.getChartModel().getEndDate());
        // ges.setStartDate(area.getStartDate());
        // ges.setEndDate(area.getEndDate());
        // area.printProject(ges);
        //
        // // try
        // // {
        // // ((GanttTree2)getTree()).getTreeTable().print();
        // // }
        // // catch (PrinterException e)
        // // {
        // // // TODO Auto-generated catch block
        // // e.printStackTrace();
        // // }
        //
        // } else if (getTabs().getSelectedIndex() == UIFacade.RESOURCES_INDEX)
        // {// Resources
        // // Chart
        // getResourcePanel().area.printProject(options.getExportSettings());
        // } else {
        // Component comp = getTabs().getSelectedComponent();
        //
        // Chart chart = (Chart)comp;
        // PrintManager.printChart(chart, options.getExportSettings());
        // }
    }

    /** Preview the project before print */
    // public void previewPrint() {
    // GanttPreviewPrint preview = null;
    // if (getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX) {// Gantt
    // // Chart
    // preview = new GanttPreviewPrint(this, area, area.getChartModel()
    // .getStartDate(), area.getChartModel().getEndDate());
    // } else if (getTabs().getSelectedIndex() == UIFacade.RESOURCES_INDEX) {//
    // Resources
    // // Chart
    // preview = new GanttPreviewPrint(this, getResourcePanel().area,
    // getResourcePanel().area.getChartModel().getStartDate(),
    // getResourcePanel().area.getChartModel().getEndDate());
    // }
    // if (preview != null)
    // preview.setVisible(true);
    // }
    public void previewPrint() {

        Date startDate, endDate;
        Object selectedUserObject = getTabs().getSelectedUserObject();
        Chart chart = null;

        if (selectedUserObject instanceof Chart)
            chart = (Chart) selectedUserObject;
        else {
            System.err.println("not printable");
            return;
        }

        try {
            startDate = chart.getStartDate();
            endDate = chart.getEndDate();
        } catch (UnsupportedOperationException e) {
            startDate = null;
            endDate = null;
        }

        PrintPreview preview = null;
        if (getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX) {
            startDate = area.getChartModel().getStartDate();
            endDate = area.getChartModel().getEndDate();
        } else if (getTabs().getSelectedIndex() == UIFacade.RESOURCES_INDEX) {
            startDate = getResourcePanel().area.getChartModel().getStartDate();
            endDate = getResourcePanel().area.getChartModel().getEndDate();
        }
        preview = new PrintPreview(this, chart, correctLabel(language
                .getText("preview")), startDate, endDate);
        if (preview != null)
            preview.setVisible(true);
    }

    /** Create a new project */
    public void newProject() {
        getUndoManager().undoableEdit("Init new Project", new Runnable() {
            public void run() {
                if (checkCurrentProject()) {

                    closeProject(true);
                    statusBar.setFirstText(language.getText("newProject2"),
                            1500);
                    refreshProjectInfos();
                }
            }
        });

        prjInfos = new PrjInfos();
        showNewProjectWizard();

    }

    private void showNewProjectWizard() {
        NewProjectWizard wizard = new NewProjectWizard();
        wizard.createNewProject(this, getUIFacade());

    }

    /**
     * Closes a project. Make sure you have already called checkCurrentProject()
     * before.
     * 
     * @see #checkCurrentProject()
     */
    public void closeProject(boolean discardUndoEdits) {
        // Clear the jtree
        // refresh graphic area
        area.repaint();
        area.setProjectLevelTaskColor(null);
        getResourcePanel().area.repaint();
        // reset people
        getResourcePanel().reset();
        // getHumanResourceManager().clear();
        prjInfos = new PrjInfos();
        // GanttTask.resetMaxID();
        RoleManager.Access.getInstance().clear();
        if (null != projectDocument)
            projectDocument.releaseLock();
        projectDocument = null;
        // change title of the frame
        if (!isOnlyViewer)
            this.setTitle(language.getText("appliTitle"));
        else
            this.setTitle("GanttViewer");
        setAskForSave(false);
        getTaskManager().clear();
        tree.clearTree();
        tree.getTreeTable().initTreeTable();
        Mediator.getCustomColumnsStorage().reset();
        // getZoomManager().clear();
        if (discardUndoEdits)
            getUndoManager().die();
        for (int i = 0; i < myPreviousStates.size(); i++)
            ((GanttPreviousState) myPreviousStates.get(i)).remove();
        myPreviousStates = new ArrayList();

        bSaveCurrent.setEnabled(false);
        bComparePrev.setEnabled(false);
        area.setPreviousStateTasks(null);

        Chart[] charts = Mediator.getPluginManager().getCharts();
        for (int i = 0; i < charts.length; i++)
            charts[i].reset();
    }

    /** Open a local project file with dialog box (JFileChooser) */
    public void openFile() throws IOException {
        // Create a filechooser
        JFileChooser fc = new JFileChooser(options.getWorkingDir());
        FileFilter ganttFilter = new GanttXMLFileFilter();
        fc.addChoosableFileFilter(ganttFilter);

        // Remove the possibility to use a file filter for all files
        FileFilter[] filefilters = fc.getChoosableFileFilters();
        for (int i = 0; i < filefilters.length; i++) {
            if (filefilters[i] != ganttFilter)
                fc.removeChoosableFileFilter(filefilters[i]);
        }

        int returnVal = fc.showOpenDialog(GanttProject.this);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            final JFileChooser jfc = fc;

            getUndoManager().undoableEdit("OpenFile", new Runnable() {
                public void run() {
                    try {
                        openDocument(getDocumentManager().getDocument(
                                jfc.getSelectedFile().getAbsolutePath()));
                    } catch (IOException e) {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                    }
                    changeWorkingDirectory(jfc.getSelectedFile().getParent());
                }
            });
            // changeUndoNumber ();
        }
    }

    /** Open a remote project file with dialog box (GanttURLChooser) */
    public void openURL() {
        openURL(projectDocument);
    }

    /** Open a remote project file with dialog box (GanttURLChooser) */

    public void openURL(Document lastDocument) {
        GanttURLChooser uc = new GanttURLChooser(this, true,
                (null != lastDocument) ? lastDocument.getURLPath() : null,
                (null != lastDocument) ? lastDocument.getUsername() : null,
                (null != lastDocument) ? lastDocument.getPassword() : null);
        uc.show();
        if (uc.change) {
            Document openDoc;
            if ((lastDocument instanceof AbstractURLDocument)
                    && uc.fileurl.equals(lastDocument.getURLPath())) {
                lastDocument.setUserInfo(uc.userName, uc.password);
                openDoc = lastDocument;
            } else {
                openDoc = DocumentCreator.createDocument(uc.fileurl,
                        uc.userName, uc.password);
            }
            try {
                openDocument(openDoc);
            } catch (IOException e) {
                if (isVisible()) {
                    GanttDialogInfo gdi = new GanttDialogInfo(this,
                            GanttDialogInfo.ERROR, GanttDialogInfo.YES_OPTION,
                            language.getText("msg11"), language
                                    .getText("error"));
                    gdi.show();
                } else
                    System.out.println("\n====" + language.getText("error")
                            + "====\n" + language.getText("msg11") + "\n");
                openURL(openDoc);
            }
        }
    }

    private void openDocument(Document document) throws IOException {
        if (document.getDescription().endsWith(".xml")
                || document.getDescription().endsWith(".gan")) {
            InputStream inputStream = document.getInputStream();
            closeProject(true);
            boolean locked = document.acquireLock();
            if (!locked) {
                GanttDialogInfo gdi = new GanttDialogInfo(this,
                        GanttDialogInfo.WARNING, GanttDialogInfo.YES_NO_OPTION,
                        language.getText("msg13"), language.getText("warning"));
                gdi.show();
                if (gdi.res == GanttDialogInfo.NO)
                    return;
            }

            GanttXMLOpen opener = new GanttXMLOpen(prjInfos, area,
                    getTaskManager(), getUIFacade());

            ResourceManager hrManager = getHumanResourceManager();
            RoleManager roleManager = getRoleManager();

            TaskManager taskManager = getTaskManager();

            ResourceTagHandler resourceHandler = new ResourceTagHandler(
                    hrManager, roleManager);
            DependencyTagHandler dependencyHandler = new DependencyTagHandler(
                    opener.getContext(), taskManager);
            AllocationTagHandler allocationHandler = new AllocationTagHandler(
                    hrManager, taskManager, roleManager);
            VacationTagHandler vacationHandler = new VacationTagHandler(
                    hrManager);

            RoleTagHandler rolesHandler = new RoleTagHandler(roleManager);
            TaskTagHandler taskHandler = new TaskTagHandler(taskManager, opener
                    .getContext());
            BlankLineTagHandler blankLineHandler = new BlankLineTagHandler(tree);
            TaskPropertiesTagHandler taskPropHandler = new TaskPropertiesTagHandler(
                    this.tree.getTreeTable());
            opener.addTagHandler(taskPropHandler);
            PreviousStateTasksTagHandler previousStateHandler = new PreviousStateTasksTagHandler(
                    this.getPreviouStates());
            DefaultWeekTagHandler weekHandler = new DefaultWeekTagHandler(
                    getActiveCalendar());
            CustomPropertiesTagHandler customPropHandler = new CustomPropertiesTagHandler(
                    opener.getContext(), taskManager);
            opener.addTagHandler(customPropHandler);
            TaskDisplayColumnsTagHandler taskDisplayHandler = new TaskDisplayColumnsTagHandler(
                    this.tree.getTreeTable());
            opener.addTagHandler(taskDisplayHandler);
            ViewTagHandler viewHandler = new ViewTagHandler(getUIFacade());
            HolidayTagHandler holidayHandler = new HolidayTagHandler(this);

            opener.addParsingListener(taskDisplayHandler);
            opener.addParsingListener(customPropHandler);
            opener.addTagHandler(weekHandler);
            opener.addTagHandler(previousStateHandler);
            opener.addTagHandler(viewHandler);
            opener.addTagHandler(holidayHandler);
            opener.addParsingListener(holidayHandler);
            opener.addParsingListener(taskPropHandler);

            opener.addTagHandler(taskHandler);
            opener.addTagHandler(blankLineHandler);

            opener.addTagHandler(opener.getDefaultTagHandler());
            opener.addTagHandler(resourceHandler);
            opener.addTagHandler(dependencyHandler);
            opener.addTagHandler(allocationHandler);
            opener.addParsingListener(allocationHandler);
            opener.addTagHandler(vacationHandler);

            opener.addTagHandler(rolesHandler);

            opener.addParsingListener(dependencyHandler);
            opener.addParsingListener(resourceHandler);

            if (opener.load(document.getInputStream())) {

                // Add this project to the last opened project
                if (documentsMRU.add(document))
                    updateMenuMRU();
                if (locked) {
                    projectDocument = document;
                    if (!isOnlyViewer)
                        this.setTitle(language.getText("appliTitle") + " ["
                                + document.getDescription() + "]");
                    else
                        this.setTitle("GanttViewer ["
                                + document.getDescription() + "]");
                }
                setAskForSave(false);
                statusBar.setFirstText(language.getText("opening") + " "
                        + document.getPath(), 2000);
                if (myPreviousStates.size() != 0)
                    bComparePrev.setEnabled(true);
            }
        } else {
            if (isVisible()) {
                GanttDialogInfo gdi = new GanttDialogInfo(this,
                        GanttDialogInfo.ERROR, GanttDialogInfo.YES_OPTION,
                        language.getText("msg2") + "\n"
                                + document.getDescription(), language
                                .getText("error"));
                gdi.show();
            } else {
                System.out.println("\n====" + language.getText("error")
                        + "====\n" + language.getText("msg2") + "\n");
            }
        }
        Chart[] charts = Mediator.getPluginManager().getCharts();
        for (int i = 0; i < charts.length; i++) {
            charts[i].setTaskManager(myTaskManager);
            charts[i].reset();
        }

        getResourcePanel().reloadRoles();
        // myDelayManager.fireDelayObservation(); // it is done in repaint2
        addMouseListenerToAllContainer(this.getComponents());
        getTaskManager().processCriticalPath((TaskNode) tree.getRoot());
        ArrayList projectTasks = tree.getProjectTasks();
        if (projectTasks.size() != 0)
            for (int i = 0; i < projectTasks.size(); i++)
                getTaskManager().processCriticalPath(
                        (TaskNode) projectTasks.get(i));

        repaint2();
    }

    public void openStartupDocument(String path) {
        if (path != null) {
            Document document = DocumentCreator.createDocument(path);
            openStartupDocument(document);
        }
    }

    public void openStartupDocument(Document document) {
        try {
            openDocument(document);
        } catch (IOException ex) {
            if (document instanceof AbstractURLDocument) {
                // if there are problems opening an AbstractURLDocument
                // (possibly because of bad authentication),
                // give the user a chance, to enter correct credentials
                openURL(document);
            } else {
                GanttDialogInfo gdi = new GanttDialogInfo(this,
                        GanttDialogInfo.ERROR, GanttDialogInfo.YES_OPTION,
                        language.getText("msg2") + "\n"
                                + document.getDescription(), language
                                .getText("error"));
                gdi.show();
            }
        }
    }

    /**
     * Open a XML stream that represent the file //By CL !@#
     * 
     * @param ins
     */
    public void openXMLStream(InputStream ins, String path) {
        try {
            tree.clearTree();
            getResourcePanel().reset();
            // getHumanResourceManager().clear();
            // GanttTask.resetMaxID();
            getTaskManager().clear();
            RoleManager.Access.getInstance().clear();
            GanttXMLOpen opener = new GanttXMLOpen(prjInfos, area,
                    getTaskManager(), getUIFacade());
            ResourceManager hrManager = getHumanResourceManager();
            RoleManager roleManager = getRoleManager();
            ResourceTagHandler resourceHandler = new ResourceTagHandler(
                    hrManager, roleManager);
            DependencyTagHandler dependencyHandler = new DependencyTagHandler(
                    opener.getContext(), getTaskManager());
            AllocationTagHandler allocationHandler = new AllocationTagHandler(
                    hrManager, getTaskManager(), getRoleManager());
            VacationTagHandler vacationHandler = new VacationTagHandler(
                    hrManager);
            TaskTagHandler taskHandler = new TaskTagHandler(getTaskManager(),
                    opener.getContext());
            BlankLineTagHandler blankLineHandler = new BlankLineTagHandler(tree);
            TaskPropertiesTagHandler taskPropHandler = new TaskPropertiesTagHandler(
                    this.tree.getTreeTable());
            opener.addTagHandler(taskPropHandler);
            CustomPropertiesTagHandler customPropHandler = new CustomPropertiesTagHandler(
                    opener.getContext(), getTaskManager());
            opener.addTagHandler(customPropHandler);
            TaskDisplayColumnsTagHandler taskDisplayHandler = new TaskDisplayColumnsTagHandler(
                    this.tree.getTreeTable());
            opener.addTagHandler(taskDisplayHandler);
            HolidayTagHandler holidayHandler = new HolidayTagHandler(this);
            opener.addTagHandler(holidayHandler);
            opener.addParsingListener(holidayHandler);
            opener.addTagHandler(taskHandler);
            opener.addTagHandler(blankLineHandler);
            opener.addParsingListener(taskPropHandler);
            opener.addParsingListener(taskDisplayHandler);
            opener.addParsingListener(customPropHandler);
            opener.addTagHandler(opener.getDefaultTagHandler());
            opener.addTagHandler(resourceHandler);
            opener.addTagHandler(dependencyHandler);
            opener.addTagHandler(allocationHandler);
            opener.addParsingListener(allocationHandler);
            opener.addTagHandler(vacationHandler);
            opener.addParsingListener(dependencyHandler);
            opener.load(ins);
            // addProjectFileToLastOpen(projectfile);
        } catch (Exception ex) {
            GanttDialogInfo gdi = new GanttDialogInfo(this,
                    GanttDialogInfo.ERROR, GanttDialogInfo.YES_OPTION, language
                            .getText("msg2")
                            + "\n" + path, language.getText("error"));
            gdi.show();
        }
        setAskForSave(false);
    }

    /** Save the project as (with a dialog file chooser) */
    public boolean saveAsProject() throws IOException {
        tree.displayHiddenTasks();
        JFileChooser fc = new JFileChooser(options.getWorkingDir());
        FileFilter ganttFilter = new GanttXMLFileFilter();
        fc.addChoosableFileFilter(ganttFilter);

        // Remove the possibility to use a file filter for all files
        FileFilter[] filefilters = fc.getChoosableFileFilters();
        for (int i = 0; i < filefilters.length; i++) {
            if (filefilters[i] != ganttFilter)
                fc.removeChoosableFileFilter(filefilters[i]);
        }

        int returnVal = fc.showSaveDialog(GanttProject.this);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            String projectfile = fc.getSelectedFile().toString();
            if (!fc.getFileFilter().accept(new File(projectfile))) {
                if (fc.getFileFilter().accept(new File(projectfile + ".gan"))) {
                    projectfile += ".gan";
                }
            }

            // if( the file exists, ask for overwriting
            if (new File(projectfile).exists()) {
                GanttDialogInfo gdi = new GanttDialogInfo(this,
                        GanttDialogInfo.WARNING, GanttDialogInfo.YES_NO_OPTION,
                        projectfile + "\n" + language.getText("msg18"),
                        language.getText("warning"));
                gdi.show();
                if (gdi.res == GanttDialogInfo.NO)
                    saveAsProject();
            }

            saveProject(new FileDocument(new File(projectfile)));
            return true;
        }
        return false;
    }

    /** Save the project on a server (with a GanttURLChooser) */
    public boolean saveAsURLProject() throws IOException {
        return saveAsURLProject(projectDocument);
    }

    /** Save the project on a server (with a GanttURLChooser) */
    public boolean saveAsURLProject(Document document) throws IOException {
        GanttURLChooser uc = new GanttURLChooser(this, false,
                (null != document) ? document.getURLPath() : null,
                (null != document) ? document.getUsername() : null,
                (null != document) ? document.getPassword() : null);
        uc.show();
        if (uc.change) {
            Document saveDocument = null;
            if (null != document)
                if (uc.fileurl.equals(document.getURLPath())
                        && uc.userName.equals(document.getUsername())
                        && uc.password.equals(document.getPassword()))
                    saveDocument = document;
            if (null == saveDocument)
                saveDocument = DocumentCreator.createDocument(uc.fileurl,
                        uc.userName, uc.password);
            saveProject(saveDocument);
            return true;
        }
        return false;

        /*
         * ServerDialog sd = new ServerDialog(this); sd.show(); return false;
         */
    }

    /** Save the project on a file */
    public void saveProject() throws IOException {
        tree.displayHiddenTasks();
        saveProject(projectDocument);
    }

    /** Save the project to the given document, if possible */
    public void saveProject(Document document) throws IOException {
        if (null == document) {
            saveAsProject();
            return;
        }
        if (!document.canWrite()) {
            GanttDialogInfo gdi = new GanttDialogInfo(this,
                    GanttDialogInfo.ERROR, GanttDialogInfo.YES_OPTION, language
                            .getText("msg10"), language.getText("error"));
            gdi.show();
            if (document instanceof AbstractURLDocument) {
                saveAsURLProject(document);
            } else {
                saveAsProject();
            }
            return;
        }
        if (!document.acquireLock()) {
            GanttDialogInfo gdi = new GanttDialogInfo(this,
                    GanttDialogInfo.ERROR, GanttDialogInfo.YES_OPTION, language
                            .getText("msg14"), language.getText("error"));
            gdi.show();
            if (document instanceof AbstractURLDocument) {
                saveAsURLProject(document);
            } else {
                saveAsProject();
            }
            return;
        }
        if (document.getDescription().endsWith(".xml")
                || document.getDescription().endsWith(".gan")) {
            GanttXMLSaver saver = new GanttXMLSaver((IGanttProject) this, tree,
                    getResourcePanel(), area, (UIFacade) this);
            saver.save(document.getOutputStream(), version);
            statusBar.setFirstText(GanttLanguage.getInstance()
                    .getText("saving")
                    + " " + document.getPath(), 2000);

            // Add this project to the last opened projects
            if (documentsMRU.add(document))
                updateMenuMRU();
            if (projectDocument != document) {
                if (projectDocument != null)
                    projectDocument.releaseLock();
                projectDocument = document;
            }
        }

        // change title of the window
        if (!isOnlyViewer)
            this.setTitle(language.getText("appliTitle") + " ["
                    + document.getDescription() + "]");
        else
            this.setTitle("GanttViewer [" + document.getDescription() + "]");
        String filepath = document.getFilePath();
        if (null != filepath) {
            changeWorkingDirectory(new File(filepath).getParent());
        }
        setAskForSave(false);
    }

    public void changeWorkingDirectory(String newWorkDir) {
        if (null != newWorkDir)
            options.setWorkingDirectory(newWorkDir);
    }

    /** @return the uiconfiguration. */
    public UIConfiguration getUIConfiguration() {
        return myUIConfiguration;
    }

    /** Function that launch the dialog to edit project properties */
    /*
     * public void editSettings() { GanttDialogSettings ds = new
     * GanttDialogSettings(this, language); ds.show(); if (ds.change) {
     * setAskForSave(ds.change); } }
     */
    /** Quit the application */
    public void quitApplication() {
        options.setWindowPosition(getX(), getY());
        options.setWindowSize(getWidth(), getHeight());
        options.setUIConfiguration(myUIConfiguration);
        options.setDocumentsMRU(documentsMRU);
        options.setLookAndFeel(lookAndFeel);
        options.setToolBarPosition(toolBar.getOrientation());
        options.save();
        if (checkCurrentProject()) {
            closeProject(true);
            setVisible(false);
            dispose();
            System.exit(0);
        } else {
            setVisible(true);
        }
    }

    /** Open the web page */
    public void openWebPage() throws IOException {
        if (!BrowserControl.displayURL("http://ganttproject.org/")) {
            GanttDialogInfo gdi = new GanttDialogInfo(this,
                    GanttDialogInfo.ERROR, GanttDialogInfo.YES_OPTION, language
                            .getText("msg4"), language.getText("error"));
            gdi.show();
            return;
        }
        statusBar.setFirstText(GanttLanguage.getInstance().getText("opening")
                + " www.ganttproject.org", 2000);
    }

    // change by G. Herrmann
    public void setAskForSave(boolean afs) {
        if (isOnlyViewer)
            return;
        String title = getTitle();
        // String last = title.substring(title.length() - 11, title.length());
        bSave.setEnabled(afs);
        miSave.setEnabled(afs);
        if (getTabs().getSelectedIndex() == UIFacade.GANTT_INDEX) {
            if (afs)
                bSaveCurrent.setEnabled(afs);
            if (myPreviousStates.size() != 0)
                bComparePrev.setEnabled(true);
        }
        askForSave = afs;
        try {
            if (System.getProperty("mrj.version") != null) {
                rootPane.putClientProperty("windowModified", Boolean
                        .valueOf(afs));
                // see http://developer.apple.com/qa/qa2001/qa1146.html
            } else {
                if (askForSave) {
                    /*
                     * if (!last.equals(" (modified)")) { setTitle(getTitle() + "
                     * (modified)"); }
                     */

                    if (!title.endsWith(" *")) {
                        setTitle(getTitle() + " *");
                    }
                }

            }
        } catch (AccessControlException e) {
            // This can happen when running in a sandbox (Java WebStart)
            System.err.println(e + ": " + e.getMessage());
        }
    }

    /** Print the help for ganttproject on the system.out */
    private static void usage() {
        System.out.println();
        System.out
                .println("GanttProject usage : java -jar ganttproject-(VERSION).jar <OPTIONS>");
        System.out.println();
        System.out.println("  Here are the possible options:");
        System.out.println("    -h, --help : Print this message");
        System.out
                .println("    [project_file_name] a XML file based on ganttproject format to directly open (project.xml or project.gan)");
        System.out
                .println("    -html [project_file_name] [export_directory_name], export directly a ganttproject file to web pages");
        System.out
                .println("         -xsl-dir [xsl_directory]                        localisation of the xsl directory for html export");
        System.out
                .println("    -pdf  [project_file_name] [pdf_file_name],         export directly a ganttproject file to web pages");
        System.out
                .println("         -xsl-fo [xsl_fo_file]                           localisation of the xsl-fo file for pdf export");
        System.out
                .println("    -csv  [project_file_name] [csv_image_filename],    export directly a ganttproject file to csv document compatible with spreadsheets");
        System.out
                .println("    -png  [project_file_name] [png_image_filename],    export directly a ganttproject file to png image");
        System.out
                .println("    -jpg  [project_file_name] [jpg_image_filename],    export directly a ganttproject file to jpg image");
        System.out
                .println("    -fig/-xfig  [project_file_name] [fig_image_filename],    export directly a ganttproject file to xfig image");
        System.out.println();
        System.out
                .println("    In all these cases the project_file_name can either be a file on local disk or an URL.");
        System.out
                .println("    If the URL is password-protected, you can give credentials this way:");
        System.out
                .println("      http://username:password@example.com/filename");
        System.out.println(" ");
    }

    public GanttResourcePanel getResourcePanel() {
        if (this.resp == null) {
            this.resp = new GanttResourcePanel(this, getTree());
            this.resp.setResourceActions(getResourceActions()); // TODO pass
            // resource
            // actions
            // created with
            // resource
            // panel as
            // context
            getHumanResourceManager().addView(this.resp);
        }
        return this.resp;
    }

    public GanttLanguage getLanguage() {
        return this.language;
    }

    public GanttGraphicArea getArea() {
        return this.area;
    }

    public GanttTree2 getTree() {
        return this.tree;
    }

    public Action getCopyAction() {
        return myCopyAction;
    }

    public Action getCutAction() {
        return myCutAction;
    }

    public Action getPasteAction() {
        return myPasteAction;
    }

    private ResourceActionSet getResourceActions() {
        if (myResourceActions == null) {
            myResourceActions = new ResourceActionSet((IGanttProject) this,
                    (ResourceContext) getResourcePanel(), this);
        }
        return myResourceActions;
    }

    /** The main */
    public static void main(String[] arg) {

        // // ------------------------------------------------------------
        // List result = new ArrayList();
        // String extensionPointID = "net.sourceforge.ganttproject.chart";
        // IConfigurationElement[] configElements = Platform
        // .getExtensionRegistry().getConfigurationElementsFor(
        // extensionPointID);
        // for (int i = 0; i < configElements.length; i++) {
        // try {
        // Object test = configElements[i]
        // .createExecutableExtension("class");
        // result.add(test);
        // } catch (CoreException e) {
        // e.printStackTrace();
        // // throw new RuntimeException("Failed to instantiate chart", e);
        // }
        // }
        //
        // // ------------------------------------------------------------

        if (arg.length > 0 && ("-h".equals(arg[0]) || "--help".equals(arg[0]))) {
            // Help for the command line
            usage();
            System.exit(0);
        }
        /*
         * If -xsl-dir with directory has been provided, use the xsls in this
         * dir instead of the ones from .ganttproject Can be used in both
         * interacive and -html/-export modes From Pawel Lipinski. And set the
         * new path as default.
         */

        GanttOptions options = new GanttOptions(null, false);

        for (int i = 0; i < arg.length; i++) {
            if (arg[i].equals("-xsl-dir") && (arg.length > i + 1)) {
                options.setXslDir(arg[i + 1]);
                break;
            }
        }
        for (int i = 0; i < arg.length; i++) {
            if (arg[i].equals("-xsl-fo") && (arg.length > i + 1)) {
                options.setXslFo(arg[i + 1]);
                break;
            }
        }
        if (arg.length > 0
                && ("-html".equals(arg[0]) || "-htm".equals(arg[0]) || "-export"
                        .equals(arg[0]))) {
            if (checkProjectFile(arg))
                exportProject(arg);
            System.exit(0);
        } else if (arg.length > 0 && "-pdf".equals(arg[0])) {
            if (checkProjectFile(arg))
                exportPDF(arg);
            System.exit(0);
        } else if (arg.length > 0 && "-png".equals(arg[0])) {
            if (checkProjectFile(arg))
                exportPNG(arg);
            System.exit(0);
        } else if (arg.length > 0 && "-jpg".equals(arg[0])) {
            if (checkProjectFile(arg))
                exportJPG(arg);
            System.exit(0);
        } else if (arg.length > 0
                && ("-fig".equals(arg[0]) || "-xfig".equals(arg[0]))) {
            if (checkProjectFile(arg))
                exportXFIG(arg);
            System.exit(0);
        } else if (arg.length > 0 && ("-csv".equals(arg[0]))) {
            if (checkProjectFile(arg))
                exportCSV(arg);
            System.exit(0);
        }
        /* Splash image */
        GanttSplash splash = new GanttSplash();
        splash.setVisible(true);
        /** Create main frame */
        GanttProject ganttFrame = new GanttProject(false, null);
        ganttFrame.setVisible(true);
        if (arg.length > 0) {

            // todo [bbaranne] remove this hack!!!
            if (!arg[0].equals("ganttproject-eclipsito-config.xml")
                    && !arg[0].equals("-pdelaunch"))
                ganttFrame.openStartupDocument(arg[0]);
        } else {
            // ganttFrame.showNewProjectWizard();
        }
        splash.close();
    }

    static boolean checkProjectFile(String[] arg) {
        if (!(arg.length > 1)) {
            System.out.println("ERROR...\nMust pass a project file");
            return false;
        }
        return true;
    }

    /**
     * Possibility to export the project into HTML directly by command Line From
     * Dmitry Barashev
     */
    private static void exportProject(String[] args) {
        try {
            byCommandLine = true;
            GanttProject project = new GanttProject(false, args[1]);
            String exportPath;
            if (args.length > 2)
                exportPath = args[2];
            else
                exportPath = "htmlExport";
            File targetDir = new File(exportPath);
            if (!targetDir.exists()) {
                targetDir.mkdir();
            } else {
                if (!targetDir.isDirectory()) {
                    throw new RuntimeException("File " + args[2]
                            + " must be directory");
                }
            }
            System.out.println(targetDir.getAbsolutePath());
            String index = project.getProjectName();
            if (index == null || index.length() == 0) {
                index = new String("ganttproject");
            } else {
                index = project.getProjectName().toLowerCase();
            }
            File targetFile = new File(targetDir, index + ".html");
            System.err.println(targetFile.getAbsolutePath());
            if (!targetFile.exists()) {
                targetFile.createNewFile();
            }
            ExportFileInfo info = new ExportFileInfo(targetFile,
                    ExportFileInfo.FORMAT_HTML, new GanttExportSettings());
            project.doExport(info);
        } catch (IOException e) {
            throw new RuntimeException("IO error", e);
        }
    }

    /** Possibility to export the project into PDF directly by command Line */
    private static void exportPDF(String[] args) {
        byCommandLine = true;
        GanttProject project = new GanttProject(false, args[1]);
        String exportFile;
        if (args.length > 2)
            exportFile = args[2];
        else
            exportFile = "ganttproject.pdf";
        File targetFile = new File(exportFile);
        ExportFileInfo info = new ExportFileInfo(targetFile,
                ExportFileInfo.FORMAT_PDF, new GanttExportSettings());
        project.doExport(info);
    }

    /** Export directly in PNG image */
    private static void exportPNG(String[] args) {
        byCommandLine = true;
        GanttProject project = new GanttProject(false, args[1]);
        File targetFile = null;
        if (args.length > 2)
            targetFile = new File(args[2]);
        ExportFileInfo info = new ExportFileInfo(targetFile,
                ExportFileInfo.FORMAT_PNG, new GanttExportSettings());
        project.doExport(info);
    }

    /** Export directly in XFIG image */
    private static void exportXFIG(String[] args) {
        byCommandLine = true;
        GanttProject project = new GanttProject(false, args[1]);
        File targetFile = null;
        if (args.length > 2)
            targetFile = new File(args[2]);
        ExportFileInfo info = new ExportFileInfo(targetFile,
                ExportFileInfo.FORMAT_XFIG, new GanttExportSettings());
        project.doExport(info);
    }

    /** Export directly in CSV image */
    private static void exportCSV(String[] args) {
        byCommandLine = true;
        GanttProject project = new GanttProject(false, args[1]);
        File targetFile = null;
        if (args.length > 2)
            targetFile = new File(args[2]);
        ExportFileInfo info = new ExportFileInfo(targetFile,
                ExportFileInfo.FORMAT_CSV, new GanttExportSettings());
        project.doExport(info);
    }

    /** Export directly in JPG image */
    private static void exportJPG(String[] args) {
        byCommandLine = true;
        GanttProject project = new GanttProject(false, args[1]);
        File targetFile = null;
        if (args.length > 2)
            targetFile = new File(args[2]);
        ExportFileInfo info = new ExportFileInfo(targetFile,
                ExportFileInfo.FORMAT_JPG, new GanttExportSettings());
        project.doExport(info);
    }

    /**
     * The class able to export directly by command line From Dmitry Barashev
     */
    public static class ExportFileInfo {
        public final File myFile;

        public final int myFormat;

        public final GanttExportSettings myStorageOptions;

        public static final int FORMAT_HTML = 1;

        public static final int FORMAT_PNG = 2;

        public static final int FORMAT_JPG = 3;

        public static final int FORMAT_PDF = 4;

        public static final int FORMAT_XFIG = 5;

        public static final int FORMAT_CSV = 6;

        public static final int FORMAT_MSPROJECT = 7;

        public static final ExportFileInfo EMPTY = new ExportFileInfo(null, -1,
                null);

        public ExportFileInfo(File file, int format, GanttExportSettings options) {
            myFile = file;
            myFormat = format;
            myStorageOptions = options;
        }
    }

    public static final String HUMAN_RESOURCE_MANAGER_ID = "HUMAN_RESOURCE";

    // change
    // to
    // public

    public static final String ROLE_MANAGER_ID = "ROLE_MANAGER";

    private GPCalendar myFakeCalendar = new WeekendCalendarImpl();

    private final ZoomManager myZoomManager;

    // private GPCalendar myFakeCalendar = new AlwaysWorkingTimeCalendarImpl();
    private final TimeUnitStack myTimeUnitStack;

    private ScrollingManager myScrollingManager;

    private DocumentManager myDocumentManager;

    private ParserFactory myParserFactory;

    private GPUndoManager myUndoManager;

    private static WindowListener ourWindowListener;

    // ///////////////////////////////////////////////////////
    // IGanttProject implementation
    public String getProjectName() {
        return prjInfos._sProjectName;
    }

    public void setProjectName(String projectName) {
        prjInfos._sProjectName = projectName;
        setAskForSave(true);
    }

    public String getDescription() {
        return prjInfos.getDescription();
    }

    public void setDescription(String description) {
        prjInfos._sDescription = description;
        setAskForSave(true);
    }

    public String getOrganization() {
        return prjInfos.getOrganization();
    }

    public void setOrganization(String organization) {
        prjInfos._sOrganization = organization;
        setAskForSave(true);
    }

    public String getWebLink() {
        return prjInfos.getWebLink();
    }

    public void setWebLink(String webLink) {
        prjInfos._sWebLink = webLink;
        setAskForSave(true);
    }

    public ResourceManager getHumanResourceManager() {
        ResourceManager result = (ResourceManager) managerHash
                .get(HUMAN_RESOURCE_MANAGER_ID);
        if (result == null) {
            result = new HumanResourceManager(getRoleManager().getDefaultRole());
            // result.addView(getPeople());
            managerHash.put(HUMAN_RESOURCE_MANAGER_ID, result);
            result.addView(this);
        }
        return result;
    }

    public TaskManager getTaskManager() {
        return myTaskManager;
    }

    public RoleManager getRoleManager() {
        RoleManager result = (RoleManager) managerHash.get(ROLE_MANAGER_ID);
        if (result == null) {
            result = RoleManager.Access.getInstance();
            managerHash.put(ROLE_MANAGER_ID, result);
        }
        return result;
    }

    public Document getDocument() {
        return projectDocument;
    }

    public ZoomManager getZoomManager() {
        return myZoomManager;
    }

    public TimeUnitStack getTimeUnitStack() {
        return myTimeUnitStack;
    }

    public GanttLanguage getI18n() {
        return getLanguage();
    }

    public GPCalendar getActiveCalendar() {
        return myFakeCalendar;
    }

    public void setModified() {
        setAskForSave(true);
    }

    public void close() {
        closeProject(false);
    }

    private DocumentManager getDocumentManager() {
        if (myDocumentManager == null) {
            myDocumentManager = new DocumentCreator(this, getUIFacade(),
                    getParserFactory());
        }
        return myDocumentManager;
    }

    private ParserFactory getParserFactory() {
        if (myParserFactory == null) {
            myParserFactory = new ParserFactoryImpl();
        }
        return myParserFactory;
    }

    public GPUndoManager getUndoManager() {
        if (myUndoManager == null) {
            myUndoManager = new UndoManagerImpl((IGanttProject) this,
                    getParserFactory(), getDocumentManager());
            Mediator.registerUndoManager(myUndoManager);
        }
        return myUndoManager;
    }

    // ///////////////////////////////////////////////////////////////
    // ResourceView implementation
    public void resourceAdded(ResourceEvent event) {
        if (getStatusBar() != null) {
            // tabpane.setSelectedIndex(1);
            getStatusBar().setFirstText(
                    GanttProject.correctLabel(GanttLanguage.getInstance()
                            .getText("newHuman")), 2000);
            setAskForSave(true);
            refreshProjectInfos();
            // setQuickSave (true);
        }
    }

    public void resourcesRemoved(ResourceEvent event) {
        // setQuickSave (true);
    }

    // ///////////////////////////////////////////////////////////////
    // UIFacade
    public UIFacade getUIFacade() {
        return this;
    }

    public ScrollingManager getScrollingManager() {
        if (myScrollingManager == null) {
            myScrollingManager = new ScrollingManagerImpl(Mediator
                    .getTaskSelectionManager());
        }
        return myScrollingManager;
    }

    public ChartViewState getGanttChartViewState() {
        return getArea().getViewState();
    }

    // public JTabbedPane getTabpane() {
    // return getTabs();
    // }

    private class ParserFactoryImpl implements ParserFactory {
        public GPParser newParser() {
            return new GanttXMLOpen(prjInfos, getArea(), getTaskManager(),
                    getUIFacade());
        }

        public GPSaver newSaver() {
            return new GanttXMLSaver(GanttProject.this, (GanttTree2) getTree(),
                    getResourcePanel(), getArea(), getUIFacade());
        }

    }

    public void setRowHeight(int value) {
        tree.getTreeTable().getTable().setRowHeight(value);
    }

    public void changeOrder(DefaultListModel buttonList,
            DefaultListModel deletedButtonList) {
        iconList = new DefaultListModel();
        for (int i = 0; i < buttonList.size(); i++)
            iconList.addElement(buttonList.getElementAt(i));
        deletedIconList = new DefaultListModel();
        for (int i = 0; i < deletedButtonList.size(); i++)
            deletedIconList.addElement(deletedButtonList.getElementAt(i));
        addButtons();
        options.setIconList(getIconPositions(iconList));
        options.setDeletedIconList(getIconPositions(deletedIconList));
        setHiddens();
        refresh();
    }

    private String getIconPositions(DefaultListModel list) {
        String sIcons = "";
        DefaultListModel iconListValues = new DefaultListModel();
        if (list != null) {
            int i = 0;
            if (list.equals(deletedIconList))
                i++;
            for (; i < list.size(); i++) {
                if (!sIcons.equals(""))
                    sIcons = sIcons + ",";
                if (list.elementAt(i).equals(
                        language.getText("separator").toUpperCase()))
                    sIcons = sIcons + GanttOptions.SEPARATOR;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bExit)
                    sIcons = sIcons + GanttOptions.EXIT;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bNew)
                    sIcons = sIcons + GanttOptions.NEW;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bOpen)
                    sIcons = sIcons + GanttOptions.OPEN;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bSave)
                    sIcons = sIcons + GanttOptions.SAVE;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bSaveAs)
                    sIcons = sIcons + GanttOptions.SAVEAS;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bImport)
                    sIcons = sIcons + GanttOptions.IMPORT;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bExport)
                    sIcons = sIcons + GanttOptions.EXPORT;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bPrint)
                    sIcons = sIcons + GanttOptions.PRINT;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bPreviewPrint)
                    sIcons = sIcons + GanttOptions.PREVIEWPRINT;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bCut)
                    sIcons = sIcons + GanttOptions.CUT;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bCopy)
                    sIcons = sIcons + GanttOptions.COPY;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bPaste)
                    sIcons = sIcons + GanttOptions.PASTE;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bNewTask)
                    sIcons = sIcons + GanttOptions.NEWTASK;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bDelete)
                    sIcons = sIcons + GanttOptions.DELETE;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bProperties)
                    sIcons = sIcons + GanttOptions.PROPERTIES;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bUnlink)
                    sIcons = sIcons + GanttOptions.UNLINK;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bLink)
                    sIcons = sIcons + GanttOptions.LINK;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bInd)
                    sIcons = sIcons + GanttOptions.IND;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bUnind)
                    sIcons = sIcons + GanttOptions.UNIND;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bUp)
                    sIcons = sIcons + GanttOptions.UP;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bDown)
                    sIcons = sIcons + GanttOptions.DOWN;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bPrev)
                    sIcons = sIcons + GanttOptions.PREV;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bScrollCenter)
                    sIcons = sIcons + GanttOptions.CENTER;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bNext)
                    sIcons = sIcons + GanttOptions.NEXT;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bZoomOut)
                    sIcons = sIcons + GanttOptions.ZOOMOUT;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bZoomIn)
                    sIcons = sIcons + GanttOptions.ZOOMIN;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bUndo)
                    sIcons = sIcons + GanttOptions.UNDO;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bRedo)
                    sIcons = sIcons + GanttOptions.REDO;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bCritical)
                    sIcons = sIcons + GanttOptions.CRITICAL;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bAbout)
                    sIcons = sIcons + GanttOptions.ABOUT;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bSaveCurrent)
                    sIcons = sIcons + GanttOptions.SAVECURRENT;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bComparePrev)
                    sIcons = sIcons + GanttOptions.COMPAREPREV;
                else if ((TestGanttRolloverButton) list.elementAt(i) == bRefresh)
                    sIcons = sIcons + GanttOptions.REFRESH;
            }
        }
        return sIcons;
    }

    public DefaultListModel initIconList() {
        DefaultListModel list = new DefaultListModel();
        int[] icons = options.getIconList();
        for (int i = 0; i < icons.length; i++)
            addButton(icons[i], list);
        return list;
    }

    public DefaultListModel initDeletedIconList() {
        DefaultListModel list = new DefaultListModel();
        if (options.getDeletedIconList() != null) {
            int[] icons = options.getDeletedIconList();
            for (int i = 0; i < icons.length; i++)
                addButton(icons[i], list);
        }
        return list;
    }

    public void addButton(int icon, DefaultListModel list) {
        switch (icon) {
        case (GanttOptions.SEPARATOR):
            list.addElement(language.getText("separator").toUpperCase());
            break;
        case (GanttOptions.EXIT):
            list.addElement(bExit);
            break;
        case (GanttOptions.NEW):
            list.addElement(bNew);
            break;
        case (GanttOptions.OPEN):
            list.addElement(bOpen);
            break;
        case (GanttOptions.SAVE):
            list.addElement(bSave);
            break;
        case (GanttOptions.SAVEAS):
            list.addElement(bSaveAs);
            break;
        case (GanttOptions.IMPORT):
            list.addElement(bImport);
            break;
        case (GanttOptions.EXPORT):
            list.addElement(bExport);
            break;
        case (GanttOptions.PRINT):
            list.addElement(bPrint);
            break;
        case (GanttOptions.PREVIEWPRINT):
            list.addElement(bPreviewPrint);
            break;
        case (GanttOptions.CUT):
            list.addElement(bCut);
            break;
        case (GanttOptions.COPY):
            list.addElement(bCopy);
            break;
        case (GanttOptions.PASTE):
            list.addElement(bPaste);
            break;
        case (GanttOptions.NEWTASK):
            list.addElement(bNewTask);
            break;
        case (GanttOptions.DELETE):
            list.addElement(bDelete);
            break;
        case (GanttOptions.PROPERTIES):
            list.addElement(bProperties);
            break;
        case (GanttOptions.UNLINK):
            list.addElement(bUnlink);
            break;
        case (GanttOptions.LINK):
            list.addElement(bLink);
            break;
        case (GanttOptions.IND):
            list.addElement(bInd);
            break;
        case (GanttOptions.UNIND):
            list.addElement(bUnind);
            break;
        case (GanttOptions.UP):
            list.addElement(bUp);
            break;
        case (GanttOptions.DOWN):
            list.addElement(bDown);
            break;
        case (GanttOptions.PREV):
            list.addElement(bPrev);
            break;
        case (GanttOptions.CENTER):
            list.addElement(bScrollCenter);
            break;
        case (GanttOptions.NEXT):
            list.addElement(bNext);
            break;
        case (GanttOptions.ZOOMOUT):
            list.addElement(bZoomOut);
            break;
        case (GanttOptions.ZOOMIN):
            list.addElement(bZoomIn);
            break;
        case (GanttOptions.UNDO):
            list.addElement(bUndo);
            break;
        case (GanttOptions.REDO):
            list.addElement(bRedo);
            break;
        case (GanttOptions.CRITICAL):
            list.addElement(bCritical);
            break;
        case (GanttOptions.ABOUT):
            list.addElement(bAbout);
            break;
        case (GanttOptions.SAVECURRENT):
            list.addElement(bSaveCurrent);
            break;
        case (GanttOptions.COMPAREPREV):
            list.addElement(bComparePrev);
            break;
        case (GanttOptions.REFRESH):
            list.addElement(bRefresh);
            break;
        default:
            break;
        }
    }

    public void addButtons() {
        toolBar.removeAll();
        for (int i = 0; i < iconList.size(); i++) {
            if (iconList.elementAt(i).equals(
                    language.getText("separator").toUpperCase())) {
                int size = Integer.parseInt(options.getIconSize());
                // toolBar.addSeparator(new Dimension(size, size));
                ImageIcon icon;
                ImageIcon iconV = new ImageIcon(getClass().getResource(
                        "/icons/sepV_16.png"));
                ImageIcon iconH = new ImageIcon(getClass().getResource(
                        "/icons/sepH_16.png"));
                if (toolBar.getOrientation() == JToolBar.HORIZONTAL)
                    icon = iconV;
                else
                    icon = iconH;
                toolBar.add(new JLabel(icon));
            } else {
                toolBar.add((JButton) iconList.elementAt(i));
            }
        }
    }

    public DefaultListModel getButtonList() {
        return iconList;
    }

    public DefaultListModel getDeletedButtonList() {
        return deletedIconList;
    }

    public Chart getActiveChart() {
        Chart resourcesChart = getResourceChart();
        Chart ganttChart = getGanttChart();
        Chart visibleChart = (getTabs().getSelectedIndex() == UIFacade.RESOURCES_INDEX) ? resourcesChart
                : ganttChart;
        return visibleChart;
    }

    public Chart getGanttChart() {
        return (Chart) getArea();
    }

    public Chart getResourceChart() {
        return (Chart) getResourcePanel().area;
    }

    public void setStatusText(String text) {
        statusBar.setFirstText(text, 2000);
    }

    public void repaint2() {
        // try
        // {
        // Thread.sleep(10);
        // }
        // catch (InterruptedException e2)
        // {
        // // TODO Auto-generated catch block
        // e2.printStackTrace();
        // }
        try {
            getTaskManager().getAlgorithmCollection()
                    .getRecalculateTaskScheduleAlgorithm().run();

            // there was a bug when importing a project where a FF dependency
            // was under a supertask.
            // The supertask length was calculated as if it was a FS dependency.
            List l = tree.getAllGanttTasks();
            Task[] t = new Task[l.size()];
            l.toArray(t);
            getTaskManager().getAlgorithmCollection()
                    .getAdjustTaskBoundsAlgorithm().run(t);
        } catch (TaskDependencyException e1) {
            e1.printStackTrace();
        }
        recalculateCriticalPath(); 

        try {
            getTaskManager().getAlgorithmCollection()
                    .getRecalculateTaskScheduleAlgorithm().run();
        } catch (TaskDependencyException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        getResourcePanel().getResourceTreeTableModel().updateResources();
        getResourcePanel().getResourceTreeTable().setRowHeight(20);
        myDelayManager.fireDelayObservation();
        super.repaint();
    }

    public void recalculateCriticalPath() {
        if (myUIConfiguration.isCriticalPathOn()) {
            getTaskManager().processCriticalPath((TaskNode) tree.getRoot());
            ArrayList projectTasks = tree.getProjectTasks();
            if (projectTasks.size() != 0) {
                for (int i = 0; i < projectTasks.size(); i++)
                    getTaskManager().processCriticalPath(
                            (TaskNode) projectTasks.get(i));
            }
            repaint();
        }
    }

    public int getViewIndex() {
        if (getTabs() == null)
            return -1;
        return getTabs().getSelectedIndex();
    }

    public void setViewIndex(int viewIndex) {
        if (myTabPane.getTabCount() > viewIndex)
            myTabPane.setSelectedIndex(viewIndex);
    }

    public static void setWindowListener(WindowListener windowListener) {
        ourWindowListener = windowListener;
    }

    public int getGanttDividerLocation() {
        return mySplitPane.getDividerLocation();
    }

    public void setGanttDividerLocation(int location) {
        mySplitPane.setDividerLocation(location);
    }

    public int getResourceDividerLocation() {
        return getResourcePanel().getDividerLocation();
    }

    public void setResourceDividerLocation(int location) {
        getResourcePanel().setDividerLocation(location);
    }

    public void refresh() {
        // // -------------------------
        //        
        // Component glassPane = this.getGlassPane();
        // setGlassPane(glassPane);
        // glassPane.setVisible(true); // before worker thread
        //
        // glassPane.addKeyListener(new KeyListener() {
        //
        // public void keyPressed(KeyEvent arg0) {
        // System.out.println("keyPressed");
        //                
        // }
        //
        // public void keyReleased(KeyEvent arg0) {
        // System.out.println("keyReleased");
        //                
        // }
        //
        // public void keyTyped(KeyEvent arg0) {
        // System.out.println("keyTyped");
        //                
        // }
        // });
        // glassPane.addMouseListener(new MouseListener() {
        //
        // public void mouseClicked(MouseEvent arg0) {
        // // TODO Auto-generated method stub
        //                
        // }
        //
        // public void mouseEntered(MouseEvent arg0) {
        // // TODO Auto-generated method stub
        //                
        // }
        //
        // public void mouseExited(MouseEvent arg0) {
        // // TODO Auto-generated method stub
        //                
        // }
        //
        // public void mousePressed(MouseEvent arg0) {
        // System.out.println("mousePressed");
        //                
        // }
        //
        // public void mouseReleased(MouseEvent arg0) {
        // // TODO Auto-generated method stub
        //                
        // }
        // });
        //        
        // // -------------------------

        try {
            getTaskManager().getAlgorithmCollection()
                    .getRecalculateTaskScheduleAlgorithm().run();

            List l = tree.getAllGanttTasks();
            Task[] t = new Task[l.size()];
            l.toArray(t);
            getTaskManager().getAlgorithmCollection()
                    .getAdjustTaskBoundsAlgorithm().run(t);

            for (int i = 0; i < t.length; i++)
                getTaskManager().getAlgorithmCollection()
                        .getRecalculateTaskCompletionPercentageAlgorithm().run(
                                t[i]);

        } catch (TaskDependencyException e1) {
            e1.printStackTrace();
        }

        getTaskManager().processCriticalPath((TaskNode) tree.getRoot());
        ArrayList projectTasks = tree.getProjectTasks();
        if (projectTasks.size() != 0) {
            for (int i = 0; i < projectTasks.size(); i++)
                getTaskManager().processCriticalPath(
                        (TaskNode) projectTasks.get(i));
        }

        try {
            getTaskManager().getAlgorithmCollection()
                    .getRecalculateTaskScheduleAlgorithm().run();
        } catch (TaskDependencyException e) {
            e.printStackTrace();
        }

        getResourcePanel().getResourceTreeTableModel().updateResources();
        getResourcePanel().getResourceTreeTable().setRowHeight(20);
        if (myDelayManager != null)
            myDelayManager.fireDelayObservation();
        super.repaint();
    }

    public void showHiddenButtonsPaneMenu() {
        menu.applyComponentOrientation(language.getComponentOrientation());
        menu.show(toolBar, bShowHiddens.getX(), bShowHiddens.getY());
    }

    public void setHiddens() {
        menu.removeAll();
        addButtons();

        int separatorSize = Integer.parseInt(options.getIconSize());

        double toolBarlength = 0.;
        int buttonSize = 0;
        int lastDisplayedIndex = 0;
        Component[] buttons = toolBar.getComponents();

        if (toolBar.getOrientation() == JToolBar.HORIZONTAL)
            toolBarlength = toolBar.getSize().getWidth();
        else
            toolBarlength = toolBar.getSize().getHeight();
        int position = 10;

        // searching for hidden buttons
        for (int i = 0; i < buttons.length; i++) {
            if (buttons[i].getClass() == TestGanttRolloverButton.class) {
                if (toolBar.getOrientation() == JToolBar.HORIZONTAL)
                    buttonSize = buttons[i].getWidth();
                else
                    buttonSize = buttons[i].getHeight();
                position = position + buttonSize;
                if (position + 2 * bShowHiddens.getWidth() / 3 < toolBarlength) {
                    lastDisplayedIndex = i;
                } else {
                    menu.add(buttons[i]);
                }
            } else
                position = position + separatorSize;
        }

        // if there is hidden buttons
        if (menu.getComponentCount() != 0) {
            for (int i = lastDisplayedIndex + 1; i < buttons.length; i++) {
                toolBar.remove(buttons[i]);
            }
            toolBar.add(bShowHiddens);
        }
    }
}
