/*
 * Created on 05.04.2005
 */
package net.sourceforge.ganttproject;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.PrintWriter;
import java.io.StringWriter;

import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.KeyStroke;
import javax.swing.WindowConstants;

import net.sourceforge.ganttproject.action.CancelAction;
import net.sourceforge.ganttproject.action.OkAction;
import net.sourceforge.ganttproject.chart.Chart;
import net.sourceforge.ganttproject.chart.ChartModelImpl;
import net.sourceforge.ganttproject.gui.DialogAligner;
import net.sourceforge.ganttproject.gui.GanttDialogInfo;
import net.sourceforge.ganttproject.gui.GanttTabbedPane;
import net.sourceforge.ganttproject.gui.options.model.GPOptionChangeListener;
import net.sourceforge.ganttproject.language.GanttLanguage;

import org.eclipse.core.runtime.IAdaptable;

/**
 * This class is designed to be a GanttProject-after-refactorings. I am going to
 * refactor GanttProject in order to make true view communicating with other
 * views through interfaces. This class is intentionally package local to
 * prevent using it in other packages (use interfaces rather than concrete
 * implementations!)
 * 
 * @author dbarashev
 */
class GanttProjectBase extends JFrame {
    private GPViewManager myViewManager;

    protected GanttProjectBase() {
        super("Gantt Chart");
        myTabPane = new GanttTabbedPane(); 
        myViewManager = new ViewManagerImpl(myTabPane);
    }

    private GanttLanguage getLanguage() {
        return GanttLanguage.getInstance();
    }

    // ////////////////////////////////////////////////////////////////
    // UIFacade
    public void showDialog(Component content, Action[] actions) {
        JDialog result = new JDialog(this, true);
        result.setResizable(false);

        final Commiter commiter = new Commiter();
        Action okAction = null;
        Action cancelAction = null;
        Box buttonBox = Box.createHorizontalBox();
        for (int i = 0; i < actions.length; i++) {
            Action nextAction = actions[i];
            JButton nextButton = null;
            if (nextAction instanceof OkAction) {
                nextAction = createOkAction(nextAction, result, commiter);
                okAction = nextAction;
                nextButton = new JButton(nextAction);
                result.getRootPane().setDefaultButton(nextButton);
            }
            if (nextAction instanceof CancelAction) {
                nextAction = createCancelAction(nextAction, result, commiter);
                cancelAction = nextAction;
                result.getRootPane().getInputMap().put(
                        KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
                        nextAction.getValue(Action.NAME));
                result.getRootPane().getActionMap().put(
                        nextAction.getValue(Action.NAME), nextAction);
            }
            if (nextButton == null) {
                nextButton = new JButton(nextAction);
            }
            buttonBox.add(nextButton);
            if (i < actions.length - 1) {
                buttonBox.add(Box.createHorizontalStrut(5));
            }
        }
        result.getContentPane().setLayout(new BorderLayout());
        result.getContentPane().add(content, BorderLayout.CENTER);
        //
        JPanel buttonPanel = new JPanel(new BorderLayout());
        buttonPanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 5, 5));
        buttonPanel.add(buttonBox, BorderLayout.EAST);
        result.getContentPane().add(buttonPanel, BorderLayout.SOUTH);
        //
        result.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        final Action localOkAction = okAction;
        final Action localCancelAction = cancelAction;
        result.addWindowListener(new WindowAdapter() {
            public void windowClosed(WindowEvent e) {
                if (localCancelAction != null && !commiter.isCommited()) {
                    localCancelAction.actionPerformed(null);
                }
            }
        });
        result.pack();
        // setSize(300, 300);
        DialogAligner.center(result, this);
        result.setVisible(true);
    }

    private static class Commiter {
        void commit() {
            isCommited = true;
        }

        boolean isCommited() {
            return isCommited;
        }

        private boolean isCommited;
    }

    private class ProxyOkAction extends OkAction implements
            PropertyChangeListener {
        private Action myRealAction;

        private JDialog myDialog;

        private Commiter myCommiter;

        private ProxyOkAction(final Action realAction, final JDialog dialog,
                final Commiter commiter) {
            realAction.addPropertyChangeListener(this);
            myRealAction = realAction;
            myDialog = dialog;
            myCommiter = commiter;
            setEnabled(realAction.isEnabled());
        }

        public void actionPerformed(ActionEvent e) {
            myRealAction.removePropertyChangeListener(this);
            myRealAction.actionPerformed(e);
            myCommiter.commit();
            if (myDialog.isVisible()) {
                myDialog.setVisible(false);
                myDialog.dispose();
            }
        }

        public void propertyChange(PropertyChangeEvent evt) {
            setEnabled(myRealAction.isEnabled());
        }

    }

    private Action createOkAction(final Action realAction,
            final JDialog dialog, final Commiter commiter) {
        return new ProxyOkAction(realAction, dialog, commiter);
    }

    private Action createCancelAction(final Action realAction,
            final JDialog result, final Commiter commiter) {
        return new CancelAction() {
            public void actionPerformed(ActionEvent e) {
                realAction.actionPerformed(e);
                commiter.commit();
                result.setVisible(false);
                result.dispose();
            }

        };
    }

    public boolean showConfirmationDialog(String message, String title) {
        GanttDialogInfo gdi = new GanttDialogInfo(this,
                GanttDialogInfo.WARNING, GanttDialogInfo.YES_NO_OPTION,
                getLanguage().getText("msg13"), getLanguage()
                        .getText("warning"));
        gdi.show();
        return gdi.res == GanttDialogInfo.YES;
    }

    public void showErrorDialog(String message) {
        if (isVisible()) {
            GanttDialogInfo gdi = new GanttDialogInfo(this,
                    GanttDialogInfo.ERROR, GanttDialogInfo.YES_OPTION, message,
                    getLanguage().getText("error"));
            gdi.show();
        } else {
            System.err.println("[GanttProjectBase] showErrorDialog:\n message");
        }
    }

    public void showErrorDialog(Exception e) {
        showErrorDialog(getExceptionReport(e));
    }

    public void showPopupMenu(Component invoker, Action[] actions, int x, int y) {
        JPopupMenu menu = new JPopupMenu();
        for (int i = 0; i < actions.length; i++) {
            Action next = actions[i];
            if (next == null) {
                menu.addSeparator();
            } else {
                menu.add(next);
            }
        }
        menu.applyComponentOrientation(getLanguage().getComponentOrientation());
        menu.show(invoker, x, y);
    }

    protected GPViewManager getViewManager() {
        return myViewManager;
    }

    private static String getExceptionReport(Exception e) {
        StringBuffer result = new StringBuffer();
        result.append(e.getMessage() + "\n\n");
        StringWriter stringWriter = new StringWriter();
        PrintWriter writer = new PrintWriter(stringWriter);
        e.printStackTrace(writer);
        writer.close();
        result.append(stringWriter.getBuffer().toString());
        return result.toString();
    }

    private static class ViewManagerImpl implements GPViewManager {
        private GanttTabbedPane myTabs;

        ViewManagerImpl(GanttTabbedPane tabs) {
            myTabs = tabs;
        }

        public GPView createView(IAdaptable adaptable, Icon icon) {
            return new GPViewImpl(myTabs, (Container) adaptable
                    .getAdapter(Container.class), icon);
        }
    }

    private static class GPViewImpl implements GPView {
        private GanttTabbedPane myTabs;

        private int myIndex;

        private Container myComponent;

        private boolean isVisible;

        private Icon myIcon;

        GPViewImpl(GanttTabbedPane tabs, Container component, Icon icon) {
            myTabs = tabs;
            myComponent = component;
            myIcon = icon;
        }

        public void setVisible(boolean isVisible) {
            String tabName = "New view";
            JScrollPane jsp = new JScrollPane();
            if (isVisible) {
                if (myComponent instanceof Chart) {
                    ((Chart) myComponent).setTaskManager(Mediator
                            .getTaskManager());
                    tabName = ((Chart) myComponent).getName();
                }
                jsp.getViewport().add(myComponent);

                myTabs.addTab(tabName, myIcon, jsp, tabName, myComponent);
                myTabs.setSelectedComponent(jsp);
                myIndex = myTabs.getSelectedIndex();

            } else {
                myTabs.remove(myIndex);
            }
            this.isVisible = isVisible;
        }

        public boolean isVisible() {
            return isVisible;
        }

    }

    protected static class RowHeightAligner implements GPOptionChangeListener {
        private ChartModelImpl myGanttViewModel;

        private GanttTree2 myTreeView;

        // TODO: 1.12 refactor and get rid of using concrete implementations of
        // gantt view model
        // and tree view
        public RowHeightAligner(GanttTree2 treeView,
                ChartModelImpl ganttViewModel) {
            myGanttViewModel = ganttViewModel;
            myTreeView = treeView;
        }

        public void optionsChanged() {
            myTreeView.getTable().setRowHeight(myGanttViewModel.setRowHeight());
        }

    }

    /** The tabbed pane with the differents parts of the project */
    protected GanttTabbedPane myTabPane;
    public GanttTabbedPane getTabs() {
        return myTabPane;
    }
}