package net.sourceforge.ganttproject;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;

import javax.swing.DefaultCellEditor;
import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JScrollBar;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.table.TableColumn;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import net.sourceforge.ganttproject.resource.ProjectResource;
import net.sourceforge.ganttproject.resource.ResourceNode;
import net.sourceforge.ganttproject.roles.Role;

import org.jdesktop.jdnc.JNTreeTable;
import org.jdesktop.swing.decorator.AlternateRowHighlighter;
import org.jdesktop.swing.decorator.HierarchicalColumnHighlighter;
import org.jdesktop.swing.decorator.Highlighter;
import org.jdesktop.swing.decorator.HighlighterPipeline;
import org.jdesktop.swing.table.TableColumnExt;
import org.jdesktop.swing.treetable.TreeTableModel;

public class ResourceTreeTable extends JNTreeTable {
    // /**
    // * model of the treetable.
    // */
    // private ResourceTreeTableModel ttModel;

    /**
     * Creates an instance of GanttTreeTable with the given TreeTableModel.
     * 
     * @param model
     *            TreeTableModel.
     */
    public ResourceTreeTable(TreeTableModel model) {
        super(model);

        this.setTreeTableModel(model);
        initTreeTable();

    }

    public boolean isVisible(DefaultMutableTreeNode node) {
        return this.getTreeTable().getTree().isVisible(
                new TreePath(node.getPath()));
    }

    public void setRowHeight(int rowHeight) {
        super.setRowHeight(rowHeight);
    }

    /**
     * Initialize the treetable. Addition of various listeners, tree's icons,
     */
    public void initTreeTable() {
        // ttModel = (ResourceTreeTableModel) this.getTreeTableModel();
        Enumeration enumeration = getTable().getColumnModel().getColumns();
        Collection lToDel = new ArrayList();
        while (enumeration.hasMoreElements()) {
            TableColumnExt tc = (TableColumnExt) enumeration.nextElement();
            lToDel.add(tc);
        }

        Iterator it = lToDel.iterator();
        while (it.hasNext())
            getTable().removeColumn((TableColumn) it.next());

        getTable().setAutoCreateColumnsFromModel(false);
        getTable().setAutoResizeMode(JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS);

        TableColumnExt tce1 = new TableColumnExt(0);
        tce1.setTitle(ResourceTreeTableModel.strResourceName);
        TableColumnExt tce2 = new TableColumnExt(1);
        tce2.setTitle(ResourceTreeTableModel.strResourceRole);
        TableColumnExt tce3 = new TableColumnExt(2);
        tce3.setTitle(ResourceTreeTableModel.strResourceEMail);
        TableColumnExt tce4 = new TableColumnExt(3);
        tce4.setTitle(ResourceTreeTableModel.strResourcePhone);
        TableColumnExt tce5 = new TableColumnExt(4);
        tce5.setTitle(ResourceTreeTableModel.strResourceRoleForTask);

        this.addColumn(tce1);
        this.addColumn(tce2);
        this.addColumn(tce3);
        this.addColumn(tce4);
        this.addColumn(tce5);

        initColumnsAlignements();

        // Highlighters to ease the reading of the table.
        setHighlighters(new HighlighterPipeline(new Highlighter[] {
                AlternateRowHighlighter.quickSilver,
                new HierarchicalColumnHighlighter() }));

        setShowHorizontalLines(true);
        setHasColumnControl(true);

        ImageIcon icon = new ImageIcon(getClass().getResource(
                "/icons/simple_task.gif"));

        setOpenIcon(icon);
        setClosedIcon(icon);
        setCollapsedIcon(new ImageIcon(getClass()
                .getResource("/icons/plus.gif")));
        setExpandedIcon(new ImageIcon(getClass()
                .getResource("/icons/minus.gif")));
        setLeafIcon(icon);

        // calculateWidth();
        this.getTreeTable().getParent().setBackground(Color.WHITE);
        setUpRolesRenderer();
        setUpAssignementRolesRenderer();

        // getTable().getTableHeader().addMouseListener(new MouseListener(){
        //
        // public void mouseClicked(MouseEvent arg0) {
        // // TODO Auto-generated method stub
        //                
        // }
        //
        // public void mousePressed(MouseEvent arg0) {
        // // TODO Auto-generated method stub
        //                
        // }
        //
        // public void mouseReleased(MouseEvent e) {
        // Enumeration en = getTable().getColumnModel().getColumns();
        // while(en.hasMoreElements())
        // {
        // TableColumn tc = (TableColumn)en.nextElement();
        // tc.setPreferredWidth(tc.getWidth());
        // }
        // }
        //
        // public void mouseEntered(MouseEvent arg0) {
        // // TODO Auto-generated method stub
        //                
        // }
        //
        // public void mouseExited(MouseEvent arg0) {
        // // TODO Auto-generated method stub
        //                
        // }
        //            
        // });

        this.getTreeTable().getTree().addTreeExpansionListener(
                new TreeExpansionListener() {
                    public void treeExpanded(TreeExpansionEvent arg0) {
                        Mediator.getGanttProjectSingleton().repaint2();

                    }

                    public void treeCollapsed(TreeExpansionEvent arg0) {
                        Mediator.getGanttProjectSingleton().repaint2();
                    }
                });
        this.getTreeTable().getParent().addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2
                        && e.getButton() == MouseEvent.BUTTON1)
                    Mediator.getGanttProjectSingleton().newHumanResource();
            }
        });
        this.setPreferredSize(new Dimension(this.getPreferredSize().width, 0));

    }

    void setUpRolesRenderer() {
        Role roles[] = Mediator.getRoleManager().getEnabledRoles();
        final JComboBox comboBox = new JComboBox();
        for (int i = 0; i < roles.length; i++)
            comboBox.addItem(roles[i]);

        try {
            TableColumn roleColumn = this
                    .getColumn(ResourceTreeTableModel.strResourceRole);
            comboBox.setEditable(false);
            roleColumn.setCellEditor(new DefaultCellEditor(comboBox));
        } catch (IllegalArgumentException ex) {

        }
    }

    void setUpAssignementRolesRenderer() {
        // Role roles[] = Mediator.getRoleManager().getEnabledRoles();
        // final JComboBox comboBox = new JComboBox();
        // for (int i = 0; i < roles.length; i++)
        // comboBox.addItem(roles[i]);
        final JComboBox comboBox = new JComboBox(Mediator.getRoleManager()
                .getEnabledRoles());

        try {
            TableColumn roleColumn = this
                    .getColumn(ResourceTreeTableModel.strResourceRoleForTask);
            comboBox.setEditable(false);
            roleColumn.setCellEditor(new DefaultCellEditor(comboBox));
        } catch (IllegalArgumentException ex) {

        }
    }

    private void initColumnsAlignements() {
        // setColumnHorizontalAlignment(ResourceTreeTableModel.strResourceName,
        // SwingConstants.LEFT);
        setColumnHorizontalAlignment(ResourceTreeTableModel.strResourceRole,
                SwingConstants.LEFT);
        setColumnHorizontalAlignment(ResourceTreeTableModel.strResourceEMail,
                SwingConstants.LEFT);
        setColumnHorizontalAlignment(ResourceTreeTableModel.strResourcePhone,
                SwingConstants.RIGHT);
        setColumnHorizontalAlignment(
                ResourceTreeTableModel.strResourceRoleForTask,
                SwingConstants.LEFT);

        // Set the columns widths
        getTable().getColumnExt(ResourceTreeTableModel.strResourceName)
                .setPreferredWidth(150);
        getTable().getColumnExt(ResourceTreeTableModel.strResourceRole)
                .setPreferredWidth(120);
        getTable().getColumnExt(ResourceTreeTableModel.strResourceEMail)
                .setPreferredWidth(100);
        getTable().getColumnExt(ResourceTreeTableModel.strResourcePhone)
                .setPreferredWidth(100);
        getTable().getColumnExt(ResourceTreeTableModel.strResourceRoleForTask)
                .setPreferredWidth(100);
        // getTable().getColumnExt(ResourceTreeTableModel.strResourceName)
        // .setMaxWidth(300);
        // getTable().getColumnExt(ResourceTreeTableModel.strResourceRole)
        // .setMaxWidth(300);
        // getTable().getColumnExt(ResourceTreeTableModel.strResourceEMail)
        // .setMaxWidth(300);
        // getTable().getColumnExt(ResourceTreeTableModel.strResourcePhone)
        // .setMaxWidth(300);
        // getTable().getColumnExt(ResourceTreeTableModel.strResourceRoleForTask)
        // .setMaxWidth(300);
    }

    /** @return the list of the selected nodes. */
    public DefaultMutableTreeNode[] getSelectedNodes() {
        TreePath[] currentSelection = getTreeTable().getTree()
                .getSelectionPaths();

        if (currentSelection == null || currentSelection.length == 0)
            return null;

        DefaultMutableTreeNode[] dmtnselected = new DefaultMutableTreeNode[currentSelection.length];

        for (int i = 0; i < currentSelection.length; i++)
            dmtnselected[i] = (DefaultMutableTreeNode) currentSelection[i]
                    .getLastPathComponent();

        return dmtnselected;
    }

    private void calculateWidth() {
        int width = 0;

        int nbCol = getTable().getColumnCount();

        for (int i = 0; i < nbCol; i++) {
            TableColumnExt tce = getTable().getColumnExt(i);
            if (tce.isVisible())
                width += tce.getPreferredWidth();
        }

        getTable().setPreferredScrollableViewportSize(new Dimension(width, 0));
    }

    public boolean isExpanded(ProjectResource pr) {
        ResourceNode node = ((ResourceTreeTableModel) getTreeTableModel())
                .exists(pr);
        if (node != null)
            return getTreeTable().isExpanded(new TreePath(node.getPath()));
        return false;
    }

    public void addKeyListener(KeyListener listener) {
        getTreeTable().addKeyListener(listener);
    }

    public void addMouseListener(MouseListener listener) {
        getTreeTable().addMouseListener(listener);
    }

    public JScrollBar getVerticalScrollBar() {
        return scrollPane.getVerticalScrollBar();
    }
}
