/*
 * Created on 01.05.2005
 */
package net.sourceforge.ganttproject.export;

import java.io.File;
import java.io.FileOutputStream;

import net.sourceforge.ganttproject.GanttOptions;
import net.sourceforge.ganttproject.IGanttProject;
import net.sourceforge.ganttproject.Mediator;
import net.sourceforge.ganttproject.GanttProject.ExportFileInfo;
import net.sourceforge.ganttproject.chart.Chart;
import net.sourceforge.ganttproject.document.Document;
import net.sourceforge.ganttproject.gui.UIFacade;
import net.sourceforge.ganttproject.gui.projectwizard.WizardImpl;
import net.sourceforge.ganttproject.io.GanttCSVExport;
import net.sourceforge.ganttproject.io.GanttHTMLExport;
import net.sourceforge.ganttproject.io.GanttXFIGSaver;
import net.sourceforge.ganttproject.language.GanttLanguage;

/**
 * @author bard
 */
public class ExportFileWizardImpl extends WizardImpl {
    private ExportFileInfo myExportFileInfo;

    private UIFacade myUIFacade;

    private IGanttProject myProject;

    private Chart myGanttChart;

    private Chart myResourceChart;

    private Chart myVisibleChart;

    private GanttOptions myOptions;

    private State myState;

    public ExportFileWizardImpl(UIFacade uiFacade, IGanttProject project,
            GanttOptions options) {
        this(ExportFileInfo.EMPTY, uiFacade, project, uiFacade.getGanttChart(),
                uiFacade.getResourceChart(), uiFacade.getActiveChart(), options);
    }

    public ExportFileWizardImpl(ExportFileInfo info, UIFacade uiFacade,
            IGanttProject project, Chart ganttChart, Chart resourceChart,
            Chart visibleChart, GanttOptions options) {
        super(uiFacade, "Export project wizard");
        myExportFileInfo = info;
        myUIFacade = uiFacade;
        myProject = project;
        myGanttChart = ganttChart;
        myResourceChart = resourceChart;
        myVisibleChart = visibleChart;
        myOptions = options;
        myState = new State(project.getDocument());
        Exporter[] exporters = Mediator.getPluginManager().getExporters();
        for (int i = 0; i < exporters.length; i++) {
            exporters[i].setContext(project, uiFacade);
        }
        addPage(new ExporterChooserPage(exporters, myState));
        addPage(new FileChooserPage(myState, myProject));
    }

    protected boolean canFinish() {
        return myState.myExporter != null && myState.myFile != null;
    }

    protected void onOkPressed() {
        super.onOkPressed();
        try {
            myState.myExporter.run(myState.getFile());
        } catch (Exception e) {
            getUIFacade().showErrorDialog(e);
        }
    }

    public void doExport() {
        try {
            doExport(myExportFileInfo);
        } catch (Exception e) {
            e.printStackTrace();
            getUIFacade().showErrorDialog(e);
        }
    }

    private void doExport(ExportFileInfo info) throws Exception {
        switch (info.myFormat) {
        case ExportFileInfo.FORMAT_HTML: {
            setStatusText(getLanguage().getText("htmlexport"));
            GanttHTMLExport.save(info.myFile, myProject, myGanttChart,
                    myResourceChart, info.myStorageOptions, myOptions);
            break;
        }
        case ExportFileInfo.FORMAT_PDF: {
            DeprecatedProjectExportData exportData = new DeprecatedProjectExportData(
                    myProject, info.myFile.toString(), myGanttChart,
                    myResourceChart, info.myStorageOptions, myOptions
                            .getXslFo());
            ProjectExportProcessor processor = new PDFExportProcessor();
            processor.doExport(exportData);
            break;
        }
        case ExportFileInfo.FORMAT_XFIG: {
            // show a message on the status bar
            setStatusText(getLanguage().getText("xfigexport"));
            String filename = info.myFile.toString();
            if (!filename.toUpperCase().endsWith(".FIG"))
                filename += ".fig";

            GanttXFIGSaver saver = new GanttXFIGSaver(myProject);
            saver.save(new FileOutputStream(new File(filename)));
            break;
        }

        case ExportFileInfo.FORMAT_CSV: {
            // show a message on the status bar
            setStatusText(getLanguage().getText("csvexport"));
            String filename = info.myFile.toString();
            if (!filename.toUpperCase().endsWith(".CSV"))
                filename += ".csv";

            GanttCSVExport saver = new GanttCSVExport(myProject, myOptions
                    .getCSVOptions());
            saver.save(new FileOutputStream(new File(filename)));
            break;
        }
        }
    }

    private GanttLanguage getLanguage() {
        return GanttLanguage.getInstance();
    }

    private void setStatusText(String text) {
        getUIFacade().setStatusText(text);
    }

    class State {
        final Document myProjectDocument;

        Exporter myExporter;

        File myFile;

        State(Document projectDocument) {
            myProjectDocument = projectDocument;
        }

        public void setFile(File file) {
            myFile = file;
            ExportFileWizardImpl.this.adjustButtonState();
        }

        public File getFile() {
            return myFile;
        }
    }
}