package net.sourceforge.ganttproject.export;

import java.awt.BorderLayout;
import java.awt.Component;
import java.io.File;

import javax.swing.JComponent;
import javax.swing.JPanel;

import net.sourceforge.ganttproject.IGanttProject;
import net.sourceforge.ganttproject.document.Document;
import net.sourceforge.ganttproject.export.ExportFileWizardImpl.State;
import net.sourceforge.ganttproject.filter.ExtensionBasedFileFilter;
import net.sourceforge.ganttproject.gui.TextFieldAndFileChooserComponent;
import net.sourceforge.ganttproject.gui.options.OptionsPageBuilder;
import net.sourceforge.ganttproject.gui.options.model.GPOptionGroup;
import net.sourceforge.ganttproject.gui.projectwizard.WizardPage;

class FileChooserPage implements WizardPage {

    private State myState;

    private TextFieldAndFileChooserComponent myChooser;

    private IGanttProject myProject;

    private JPanel myComponent;

    private JComponent mySecondaryOptionsComponent;
    
    private OptionsPageBuilder myOptionsBuilder;

    private GPOptionGroup[] mySecondaryOptions;
    
    FileChooserPage(State state, IGanttProject project) {
        myState = state;
        myProject = project;
        myOptionsBuilder = new OptionsPageBuilder();
    }

    public String getTitle() {
        return "Select file to export to";
    }

    public Component getComponent() {
        myComponent = new JPanel(new BorderLayout());
        myChooser = new TextFieldAndFileChooserComponent("Filename:",
                "Select file to export to") {
            protected void onFileChosen(File file) {
                String proposedExtension = myState.myExporter
                        .proposeFileExtension();
                if(proposedExtension != null)
                if (false == file.getName().toLowerCase().endsWith(
                        proposedExtension)) {
                    file = new File(file.getAbsolutePath() + "."
                            + proposedExtension);
                }
                super.onFileChosen(file);
                myState.setFile(file);
            }
        };
        myComponent.add(myChooser.getComponent(), BorderLayout.NORTH);
        return myComponent;
    }

    public void setActive(boolean b) {
        if (b == false) {
            myState.setFile(myChooser.getFile());
            if (mySecondaryOptions!=null) {
                for (int i=0; i<mySecondaryOptions.length; i++) {
                    mySecondaryOptions[i].commit();
                }
            }
        } else {
            mySecondaryOptions = myState.myExporter.getSecondaryOptions(); 
            if (mySecondaryOptions!=null) {
                for (int i=0; i<mySecondaryOptions.length; i++) {
                    mySecondaryOptions[i].lock();
                }
                mySecondaryOptionsComponent = new JPanel(new BorderLayout()); 
                mySecondaryOptionsComponent.add(myOptionsBuilder.buildPlanePage(myState.myExporter.getSecondaryOptions()), BorderLayout.NORTH); 
                myComponent.add(mySecondaryOptionsComponent, BorderLayout.CENTER);
            }
            else if (mySecondaryOptionsComponent!=null){
                myComponent.remove(mySecondaryOptionsComponent);
            }
            myComponent.invalidate();
            myChooser.setFileFilter(new ExtensionBasedFileFilter(
                    myState.myExporter.getFileNamePattern(), myState.myExporter
                            .getFileTypeDescription()));
            File proposedFile = proposeOutputFile(myProject);
            if (proposedFile == null) {
                myChooser.setFile(myState.getFile());
                return;
            }
            if (false == proposedFile.equals(myChooser.getFile())) {
                myChooser.setFile(proposedFile);
                myState.setFile(proposedFile);
            }
        }
    }

    public File proposeOutputFile(IGanttProject project) {
        String proposedExtension = myState.myExporter.proposeFileExtension();
        if (proposedExtension == null) {
            return null;
        }
        File result = new File(project.getProjectName() + "."
                + proposedExtension);
        Document projectDocument = project.getDocument();
        if (projectDocument != null) {
            File localFile = new File(projectDocument.getFilePath());
            if (localFile.exists()) {
                String name = localFile.getAbsolutePath();
                int lastDot = name.lastIndexOf('.');
                name = name.substring(0, lastDot) + "." + proposedExtension;
                result = new File(name);
            } else {
                File directory = localFile.getParentFile();
                if (directory.exists()) {
                    result = new File(directory, project.getProjectName() + "."
                            + proposedExtension);
                }
            }
        }
        return result;
    }

}
