package net.sourceforge.ganttproject.gui;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Date;
import java.util.GregorianCalendar;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import net.sourceforge.ganttproject.GanttCalendar;
import net.sourceforge.ganttproject.Mediator;
import net.sourceforge.ganttproject.language.GanttLanguage;
import net.sourceforge.ganttproject.task.CustomColumn;
import net.sourceforge.ganttproject.task.CustomColumnsStorage;

/**
 * Dialog opened to create a new customColumn.
 * 
 * @author bbaranne Mar 2, 2005
 */
public class GanttDialogCustomColumn extends JDialog {
    private static GanttLanguage language = GanttLanguage.getInstance();

    private static String cardInteger = language.getText("integer");

    private static String cardText = language.getText("text");

    private static String cardDouble = language.getText("double");

    private static String cardDate = language.getText("date");

    private static String cardBoolean = language.getText("boolean");

    /**
     * Created CustomColumn.
     */
    private CustomColumn customColumn = null;

    private JTextField fieldName = null;

    private JComboBox comboTypes = null;

    private JPanel panelDefaultValue = null;

    private CardLayout cardLayoutDefaultValue = null;

    private JPanel panelText = null;

    private JPanel panelInteger = null;

    private JPanel panelDouble = null;

    private JPanel panelDate = null;

    private JPanel panelBoolean = null;

    private JTextArea textArea = null;

    private GTextField fieldInteger = null;

    private GTextField fieldDouble = null;

    private JLabel labelDate = null;

    private JButton buttonDate = null;

    private JRadioButton radioTrue = null;

    private JRadioButton radioFalse = null;

    private JLabel labelName = null;

    private JLabel labelType = null;

    private JLabel labelDefaultValue = null;

    private JButton buttonOK = null;

    private JButton buttonCancel = null;

    private GanttCalendar date = null;

    public GanttDialogCustomColumn(Frame parent, CustomColumn customCol) {
        super(parent, language.getText("newCustomColumn"), true);
        customColumn = customCol;
        initComponents();
        DialogAligner.center(this, parent);
    }

    private void initComponents() {
        GridBagConstraints constraints = new GridBagConstraints();

        labelName = new JLabel(language.getText("name") + ": ");
        labelType = new JLabel(language.getText("typeClass") + ": ");
        labelDefaultValue = new JLabel(language.getText("defaultValue") + ": ");

        // Text
        textArea = new JTextArea();
        textArea.setWrapStyleWord(true);
        textArea.setLineWrap(true);
        textArea.setColumns(25);
        textArea.setRows(5);
        JScrollPane jsp = new JScrollPane(textArea);
        panelText = new JPanel(new BorderLayout());
        panelText.add(jsp, BorderLayout.CENTER);

        // Boolean
        radioTrue = new JRadioButton(language.getText("yes"));
        radioFalse = new JRadioButton(language.getText("no"));
        ButtonGroup group = new ButtonGroup();
        group.add(radioFalse);
        group.add(radioTrue);
        radioTrue.setSelected(true);
        panelBoolean = new JPanel(new GridBagLayout());
        constraints.gridx = 0;
        constraints.gridy = 0;
        panelBoolean.add(radioTrue, constraints);
        constraints.gridx = 0;
        constraints.gridy = 1;
        panelBoolean.add(radioFalse, constraints);

        // Integer
        fieldInteger = new GTextField();
        fieldInteger.setPattern(GTextField.PATTERN_INTEGER);
        fieldInteger.setColumns(10);
        panelInteger = new JPanel(new GridBagLayout());
        constraints = new GridBagConstraints();
        constraints.gridx = 0;
        constraints.gridy = 0;
        panelInteger.add(fieldInteger, constraints);

        // Double
        fieldDouble = new GTextField();
        fieldDouble.setPattern(GTextField.PATTERN_DOUBLE);
        fieldDouble.setColumns(10);
        panelDouble = new JPanel(new GridBagLayout());
        constraints = new GridBagConstraints();
        constraints.gridx = 0;
        constraints.gridy = 0;
        panelDouble.add(fieldDouble, constraints);

        // Date
        ImageIcon icon = new ImageIcon(getClass().getResource(
                "/icons/calendar_16.gif"));

        date = new GanttCalendar(new Date());
        final GanttDialogDate dd = new GanttDialogDate(this, date, false);
        labelDate = new JLabel(date.toString());
        buttonDate = new JButton(icon);
        buttonDate.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                dd.show();
                date = dd.getDate();
                labelDate.setText(date.toString());
            }

        });
        panelDate = new JPanel(new GridBagLayout());
        constraints = new GridBagConstraints();
        constraints.gridx = 0;
        constraints.gridy = 0;
        panelDate.add(labelDate, constraints);
        constraints.gridx = 1;
        constraints.gridy = 0;
        constraints.insets = new Insets(0, 5, 0, 5);
        panelDate.add(buttonDate, constraints);

        date = dd.getDate();
        labelDate.setText(date.toString());

        cardLayoutDefaultValue = new CardLayout();
        panelDefaultValue = new JPanel(cardLayoutDefaultValue);
        panelDefaultValue.add(cardText, panelText);
        panelDefaultValue.add(cardBoolean, panelBoolean);
        panelDefaultValue.add(cardInteger, panelInteger);
        panelDefaultValue.add(cardDouble, panelDouble);
        panelDefaultValue.add(cardDate, panelDate);

        fieldName = new JTextField();
        comboTypes = new JComboBox(CustomColumnsStorage.availableTypes);
        comboTypes.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent e) {
                /*
                 * 0 -> Text 1 -> Integer 2 -> Double 3 -> Date 4 -> Boolean
                 */
                int selectedIndex = comboTypes.getSelectedIndex();

                switch (selectedIndex) {
                case 0:
                    cardLayoutDefaultValue.show(panelDefaultValue, cardText);
                    break;
                case 1:
                    cardLayoutDefaultValue.show(panelDefaultValue, cardInteger);
                    break;
                case 2:
                    cardLayoutDefaultValue.show(panelDefaultValue, cardDouble);
                    break;
                case 3:
                    cardLayoutDefaultValue.show(panelDefaultValue, cardDate);
                    break;
                case 4:
                    cardLayoutDefaultValue.show(panelDefaultValue, cardBoolean);
                    break;

                }
            }

        });

        buttonOK = new JButton();
        buttonOK.setHorizontalTextPosition(SwingConstants.CENTER);
        buttonOK.setText(language.getText("ok"));
        buttonOK.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                ok();
            }
        });
        getRootPane().setDefaultButton(buttonOK);

        buttonCancel = new JButton();
        buttonCancel.setHorizontalAlignment(SwingConstants.CENTER);
        buttonCancel.setText(language.getText("cancel"));
        buttonCancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                cancel();
            }
        });

        setResizable(false);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                closeDialog();
            }
        });

        constraints.fill = GridBagConstraints.HORIZONTAL;
        JPanel panelMain = new JPanel(new GridBagLayout());

        // name
        constraints = new GridBagConstraints();
        constraints.gridx = 0;
        constraints.gridy = 0;
        constraints.insets = new Insets(5, 5, 10, 10);
        panelMain.add(labelName, constraints);
        constraints.gridx = 1;
        constraints.gridy = 0;
        constraints.gridwidth = 5;
        constraints.ipadx = 100;
        panelMain.add(fieldName, constraints);

        // type
        constraints = new GridBagConstraints();
        constraints.gridx = 0;
        constraints.gridy = 1;
        constraints.insets = new Insets(5, 5, 10, 10);
        panelMain.add(labelType, constraints);
        constraints.gridx = 1;
        constraints.gridy = 1;
        constraints.gridwidth = 5;
        panelMain.add(comboTypes, constraints);

        // panelDefaultValue
        constraints = new GridBagConstraints();
        constraints.gridx = 0;
        constraints.gridy = 2;
        constraints.insets = new Insets(5, 5, 10, 10);
        panelMain.add(labelDefaultValue, constraints);
        constraints.gridx = 1;
        constraints.gridy = 2;
        constraints.gridwidth = 5;
        panelMain.add(panelDefaultValue, constraints);

        // buttonOK
        constraints = new GridBagConstraints();
        constraints.gridx = 3;
        constraints.gridy = 3;
        constraints.anchor = java.awt.GridBagConstraints.EAST;
        constraints.insets = new Insets(10, 10, 10, 10);
        panelMain.add(buttonOK, constraints);

        // buttonCancel
        constraints = new java.awt.GridBagConstraints();
        constraints.gridx = 4;
        constraints.gridy = 3;
        constraints.anchor = java.awt.GridBagConstraints.EAST;
        constraints.insets = new Insets(10, 10, 10, 10);
        panelMain.add(buttonCancel, constraints);

        // All
        getContentPane().setLayout(new GridBagLayout());
        constraints = new java.awt.GridBagConstraints();
        constraints.gridx = 0;
        constraints.gridy = 0;
        constraints.gridwidth = 5;
        constraints.gridheight = 5;
        constraints.fill = java.awt.GridBagConstraints.BOTH;
        constraints.weightx = 1.0;
        constraints.weighty = 1.0;
        getContentPane().add(panelMain, constraints);
        cardLayoutDefaultValue.show(panelDefaultValue, cardText);
        pack();

    } // TODO change the default value for custom columns.

    private void ok() {
        Mediator.getGanttProjectSingleton().setAskForSave(true);
        String colName = fieldName.getText().trim();
        Object defValue = null;

        if (colName.length() != 0) {
            Class colClass;
            int colType = comboTypes.getSelectedIndex();
            switch (colType) {
            case 0:
                colClass = String.class;
                defValue = textArea.getText();
                break;
            case 1:
                colClass = Integer.class;
                String ti = fieldInteger.getText();
                if (ti.trim().length() == 0)
                    ti = "0";
                defValue = new Integer(Integer.parseInt(ti));
                break;
            case 2:
                colClass = Double.class;
                String td = fieldDouble.getText();
                if (td.trim().length() == 0)
                    td = "0.0";
                defValue = new Double(Double.parseDouble(td));
                break;
            case 3:
                colClass = GregorianCalendar.class;
                defValue = date;
                break;
            case 4:
                colClass = Boolean.class;
                defValue = new Boolean(radioTrue.isSelected());
                break;
            default: // normally never reached.
                colClass = String.class;
                defValue = "default";
            }

            if (customColumn != null) {
                customColumn.setName(colName);
                customColumn.setType(colClass);
                customColumn.setDefaultValue(defValue);
            }
            closeDialog();
        } else
        // colName is empty
        {
            fieldName.requestFocus();
            // nothing (the dialog stays opened)
        }
    }

    private void cancel() {
        closeDialog();
    }

    private void closeDialog() {
        setVisible(false);
        dispose();
    }
}
