/**
 * 
 */
package net.sourceforge.ganttproject.gui;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Date;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JList;

import net.sourceforge.ganttproject.GanttCalendar;
import net.sourceforge.ganttproject.GanttProject;
import net.sourceforge.ganttproject.language.GanttLanguage;

/**
 * @author nbohn
 */
public class GanttDialogPublicHoliday extends JDialog implements ActionListener {
    private GanttLanguage lang = GanttLanguage.getInstance();

    private GanttPublicHolidayBean publicHolidayBean;

    private GanttProject myProject;

    private DefaultListModel publicHolidays;

    private boolean isChanged = false;

    public GanttDialogPublicHoliday(GanttProject project) {
        super(project, GanttLanguage.getInstance()
                .getText("editPublicHolidays"), true);
        publicHolidays = new DefaultListModel();
        for (int i = 0; i < project.getActiveCalendar().getPublicHolidays()
                .size(); i++) {
            Date d = (Date) project.getActiveCalendar().getPublicHolidays()
                    .get(i);
            if (d.getYear() == 1 - 1900) {
                publicHolidays.addElement(new GanttCalendar(1, d.getMonth(), d
                        .getDate()));
            } else {

                publicHolidays.addElement(new GanttCalendar(d.getYear() + 1900,
                        d.getMonth(), d.getDate()));
            }
        }

        publicHolidayBean = new GanttPublicHolidayBean(this, publicHolidays);
        myProject = project;
        Container cp = getContentPane();

        publicHolidayBean.addActionListener(this);

        cp.add(publicHolidayBean, BorderLayout.CENTER);

        this.pack();
        setResizable(false);
        DialogAligner.center(this, getParent());

        applyComponentOrientation(lang.getComponentOrientation());
    }

    public void actionPerformed(ActionEvent evt) {
        if (evt.getSource() instanceof JButton) {

            JButton button = (JButton) evt.getSource();
            if (button.getName().equals("ok")) {
                myProject.getActiveCalendar().getPublicHolidays().clear();
                for (int i = 0; i < publicHolidays.size(); i++)
                    myProject.getActiveCalendar().setPublicHoliDayType(
                            ((GanttCalendar) publicHolidays.getElementAt(i))
                                    .getTime());
                this.setVisible(false);
                dispose();
                myProject.repaint();
            }

            else if (button.getName().equals("cancel")) {
                this.setVisible(false);
                dispose();

            } else if (button.getName().equals("delete")) {
                JList list = publicHolidayBean.getJList();
                Object[] objects = list.getSelectedValues();
                if ((objects.length > 0) && (!isChanged))
                    isChanged = true;
                for (int i = 0; i < objects.length; i++) {
                    publicHolidays.removeElement(objects[i]);
                }
            } else if (button.getName().equals("startDate")) {
                GanttCalendar date = new GanttCalendar();

                if (publicHolidayBean.getStartDate() != null)
                    date = publicHolidayBean.getStartDate().Clone();

                GanttDialogDate dd = new GanttDialogDate(this, date, true);

                dd.show();

                publicHolidayBean.setStartDate(dd.getDate());

                publicHolidayBean.getAddButton().setEnabled(true);
            } else if (button.getName().equals("finishDate")) {
                GanttCalendar date = new GanttCalendar();
                if (publicHolidayBean.getFinishDate() != null)
                    date = publicHolidayBean.getFinishDate().Clone();
                GanttDialogDate dd = new GanttDialogDate(this, date, false);

                dd.show();

                publicHolidayBean.setFinishDate(dd.getDate());

                publicHolidayBean.getAddButton().setEnabled(true);
            } else if (button.getName().equals("add")) {
                GanttCalendar startDate = publicHolidayBean.getStartDate();
                GanttCalendar finishDate = publicHolidayBean.getFinishDate();
                if (finishDate.before(startDate)
                        || finishDate.equals(startDate)) {
                    GanttDialogInfo gdi = new GanttDialogInfo(myProject,
                            GanttDialogInfo.ERROR, GanttDialogInfo.YES_OPTION,
                            lang.getText("msg24"), lang.getText("error"));
                    gdi.show();
                } else {
                    for (int i = 0; i < finishDate.diff(startDate); i++) {
                        addANewHoliDay(startDate.newAdd(i));
                    }
                    publicHolidayBean.resetDates();
                }

                // boolean isUnique = true;
                // for (int i = 0; i < publicHolidays.size(); i++) {
                // Date dateToCompare = ((GanttCalendar) publicHolidays
                // .getElementAt(i)).getTime();
                // if ((startDate.isFixed())
                // && (startDate.getTime().getMonth() == dateToCompare
                // .getMonth())
                // && (startDate.getTime().getDate() == dateToCompare
                // .getDate())) {
                // isUnique = false;
                // break;
                // }
                //
                // else if (startDate.getTime().equals(dateToCompare)) {
                // isUnique = false;
                // break;
                // }
                // }
                // if (isUnique) {
                // if (startDate.isFixed())
                // startDate.setYear(1);
                // publicHolidays.addElement(startDate);
                // if (!isChanged)
                // isChanged = true;
                // }

            }
            if (publicHolidays.size() == 0)
                publicHolidayBean.getDeleteButton().setEnabled(false);
            else
                publicHolidayBean.getDeleteButton().setEnabled(true);
        }
    }

    private void addANewHoliDay(GanttCalendar calendar) {
        for (int i = 0; i < publicHolidays.size(); i++) {
            Date dateToCompare = ((GanttCalendar) publicHolidays
                    .getElementAt(i)).getTime();
            if ((calendar.isFixed())
                    && (calendar.getTime().getMonth() == dateToCompare
                            .getMonth())
                    && (calendar.getTime().getDate() == dateToCompare.getDate())) {
                publicHolidays.remove(i);
            }

            else if (calendar.getTime().equals(dateToCompare)) {
                publicHolidays.remove(i);
            }
        }
        if (calendar.isFixed())
            calendar.setYear(1);
        publicHolidays.addElement(calendar);
        if (!isChanged)
            isChanged = true;
    }

}
