/**
 * 
 */
package net.sourceforge.ganttproject.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionListener;

import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionListener;

import net.sourceforge.ganttproject.GanttCalendar;
import net.sourceforge.ganttproject.GanttProject;
import net.sourceforge.ganttproject.language.GanttLanguage;

/**
 * @author nbohn
 */
public class GanttPublicHolidayBean extends JPanel {
    private GanttDialogPublicHoliday parent;

    private JPanel generalPanel;

    private JPanel southPanel;

    private JPanel deletePanel;

    private JPanel addPanel;

    private GridBagConstraints gbc = new GridBagConstraints();

    private JButton okButton, cancelButton, deleteButton, startDateButton,
            finishDateButton, addButton;

    private GanttLanguage lang = GanttLanguage.getInstance();

    private GanttProject myProject;

    private DefaultListModel publicHolidays;

    private JTextField startDateField;

    private JTextField finishDateField;

    private JLabel startDateLabel;

    private JLabel finishDateLabel;

    private GanttCalendar startDate;

    private GanttCalendar finishDate;

    private JScrollPane scrollPane;

    private JList list;

    public GanttPublicHolidayBean(GanttDialogPublicHoliday parent,
            DefaultListModel publicHolidays) {
        this.parent = parent;
        this.publicHolidays = publicHolidays;
        init();
    }

    public void init() {
        constructGeneralPanel();

        setLayout(new BorderLayout());

        // add(generalPanel, BorderLayout.NORTH);

        add(constructDeletePanel(), BorderLayout.CENTER);
        add(constructAddPanel(), BorderLayout.NORTH);
        add(constructSouthPanel(), BorderLayout.SOUTH);
    }

    /** Construct the general panel */
    private void constructGeneralPanel() {
        generalPanel = new JPanel(new GridBagLayout());

        generalPanel.setLayout(new GridBagLayout());
        gbc.weightx = 0;
        gbc.weighty = 0;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.insets.right = 15;
        gbc.insets.left = 10;
        gbc.insets.top = 10;

    }

    /** Construct the south panel */

    private JPanel constructSouthPanel() {
        okButton = new JButton(lang.getText("ok"));

        okButton.setName("ok");

        if (getRootPane() != null)
            getRootPane().setDefaultButton(okButton); // set ok the defuault
        // button when press
        // "enter" --> check
        // because
        // getRootPane()==null
        // !!!

        cancelButton = new JButton(lang.getText("cancel"));

        cancelButton.setName("cancel");

        southPanel = new JPanel(new FlowLayout(FlowLayout.TRAILING, 50, 10));

        southPanel.add(okButton);

        southPanel.add(cancelButton);

        return southPanel;
    }

    private JPanel constructDeletePanel() {
        list = new JList(publicHolidays);
        list.setName("list");

        scrollPane = new JScrollPane(list);
        scrollPane.setPreferredSize(new Dimension(120, 200));
        deleteButton = new JButton(lang.getText("delete"));
        deleteButton.setName("delete");
        if (publicHolidays.size() == 0)
            deleteButton.setEnabled(false);
        deletePanel = new JPanel(new FlowLayout(FlowLayout.TRAILING, 30, 10));
        deletePanel.add(scrollPane);
        deletePanel.add(deleteButton);
        deletePanel.setBorder(new TitledBorder(new EtchedBorder(), lang
                .getText("delete")));
        return deletePanel;
    }

    private JPanel constructAddPanel() {
        startDateLabel = new JLabel(lang.getText("dateOfBegining"));
        finishDateLabel = new JLabel(lang.getText("dateOfEnd"));
        startDateField = new JTextField(12);
        startDateField.setEditable(false);
        finishDateField = new JTextField(12);
        finishDateField.setEditable(false);
        ImageIcon icon = new ImageIcon(getClass().getResource(
                "/icons/calendar_16.gif"));
        startDateButton = new TestGanttRolloverButton(icon);
        startDateButton.setName("startDate");
        startDateButton.setToolTipText(GanttProject.getToolTip(lang
                .getText("chooseDate")));
        finishDateButton = new TestGanttRolloverButton(icon);
        finishDateButton.setName("finishDate");
        finishDateButton.setToolTipText(GanttProject.getToolTip(lang
                .getText("chooseDate")));
        finishDateButton.setEnabled(false);
        addButton = new JButton(lang.getText("add"));
        addButton.setName("add");
        addButton.setEnabled(false);
        addPanel = new JPanel(new FlowLayout(FlowLayout.TRAILING, 30, 10));
        JPanel startDatePanel = new JPanel(new FlowLayout(FlowLayout.TRAILING,
                10, 10));
        startDatePanel.add(startDateLabel);
        startDatePanel.add(startDateField);
        startDatePanel.add(startDateButton);
        JPanel finishDatePanel = new JPanel(new FlowLayout(FlowLayout.TRAILING,
                10, 10));
        finishDatePanel.add(finishDateLabel);
        finishDatePanel.add(finishDateField);
        finishDatePanel.add(finishDateButton);

        JPanel datesPanel = new JPanel(new BorderLayout());
        datesPanel.add(startDatePanel, BorderLayout.NORTH);
        datesPanel.add(finishDatePanel, BorderLayout.SOUTH);
        addPanel.add(datesPanel);
        addPanel.add(addButton);
        addPanel.setBorder(new TitledBorder(new EtchedBorder(), lang
                .getText("add")));
        return addPanel;
    }

    private void addUsingGBL(Container container, Component component,

    GridBagConstraints gbc, int x,

    int y, int w, int h) {
        gbc.gridx = x;
        gbc.gridy = y;
        gbc.gridwidth = w;
        gbc.gridheight = h;
        gbc.weighty = 0;
        container.add(component, gbc);
    }

    public void addActionListener(ActionListener l) {

        okButton.addActionListener(l);

        cancelButton.addActionListener(l);

        deleteButton.addActionListener(l);

        startDateButton.addActionListener(l);

        finishDateButton.addActionListener(l);

        addButton.addActionListener(l);

    }

    public void addListSelectionListener(ListSelectionListener l) {

        list.addListSelectionListener(l);

    }

    public JList getJList() {
        return list;
    }

    public void setStartDate(GanttCalendar date) {
        this.startDate = date.Clone();
        startDateField.setText(date.toString());
        if (finishDate == null) {
            finishDate = startDate.newAdd(1);
            finishDateField.setText(finishDate.toString());
        }
        setAnnual(startDate.isFixed());
        finishDateButton.setEnabled(true);
    }

    public GanttCalendar getStartDate() {
        return startDate;
    }

    public void setFinishDate(GanttCalendar date) {
        this.finishDate = date.Clone();
        finishDateField.setText(date.toString());
    }

    public GanttCalendar getFinishDate() {
        return finishDate;
    }

    public JButton getDeleteButton() {
        return deleteButton;
    }

    public JButton getAddButton() {
        return addButton;
    }

    public void setAnnual(boolean isAnnual) {
        startDateField.setForeground(isAnnual ? Color.BLACK : Color.GRAY);
        finishDateField.setForeground(isAnnual ? Color.BLACK : Color.GRAY);
    }

    public void resetDates() {
        startDateField.setText("");
        finishDateField.setText("");
        finishDate = null;
        finishDateButton.setEnabled(false);
    }

}