/***************************************************************************
 HTMLSettingsPanel.java 
 ------------------------------------------
 begin                : 27 juin 2004
 copyright            : (C) 2004 by Thomas Alexandre
 email                : alexthomas(at)ganttproject.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
package net.sourceforge.ganttproject.gui.options;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.security.AccessControlException;

import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.filechooser.FileFilter;

import net.sourceforge.ganttproject.GanttProject;
import net.sourceforge.ganttproject.gui.TextFieldAndFileChooserComponent;
import net.sourceforge.ganttproject.language.GanttLanguage;

/**
 * @author athomas Choose the xsl directory for the html export
 */
public class HTMLSettingsPanel extends GeneralOptionPanel implements
        ActionListener {

    // JButton buttonXsl;
    // JTextField tfXsl;
    JCheckBox cbDefault;

    TextFieldAndFileChooserComponent myChooser;

    private GanttProject appli;

    /** Constructor. */
    public HTMLSettingsPanel(GanttProject parent) {
        super(GanttProject.correctLabel(GanttLanguage.getInstance().getText(
                "htmlexport")), GanttLanguage.getInstance().getText(
                "settingsHTMLExport"), parent);

        appli = parent;

        myChooser = new TextFieldAndFileChooserComponent(this, language
                .getText("selectThemeDirectory")) {
            protected void onFileChosen(File file) {
                bHasChange = true;
            }
        };
        myChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        myChooser.setFileFilter(new FileFilter() {
            public boolean accept(File f) {
                return f.isDirectory();
            }

            public String getDescription() {
                return "Directories";
            }
        });

        vb.add(myChooser.getComponent());
        vb.add(new JPanel());
        JPanel defaultPanel = new JPanel(new BorderLayout());
        defaultPanel.add(cbDefault = new JCheckBox(), BorderLayout.WEST);
        defaultPanel.add(new JLabel(language.getText("xslDirectory")),
                BorderLayout.CENTER);
        vb.add(defaultPanel);
        vb.add(new JPanel());

        cbDefault.addActionListener(this);

        applyComponentOrientation(language.getComponentOrientation());
    }

    /** This method check if the value has changed, and assk for commit changes. */
    public boolean applyChanges(boolean askForApply) {
        if (bHasChange)
            if (!askForApply || (askForApply && askForApplyChanges()))
                appli.getOptions().setXslDir(
                        getXslDirectory(cbDefault.isSelected()));

        return bHasChange;
    }

    /** Initialize the component. */
    public void initialize() {
        try {
            File xslDir = new File(
                    appli.getOptions().getXslDir() != null ? appli.getOptions()
                            .getXslDir() : System.getProperty("user.home"));
            myChooser.setFile(xslDir);

        } catch (AccessControlException e) {
            // This can happen when running in a sandbox (Java WebStart)
            System.err.println(e + ": " + e.getMessage());
        }
    }

    /** Return the xsl directory */
    public String getXslDirectory(boolean useDefault) {
        if (!useDefault) {
            if (myChooser.getFile() != null && myChooser.getFile().exists()) {
                return myChooser.getFile().getAbsolutePath();
            }
        }
        return HTMLSettingsPanel.class.getResource("/xslt").toString();
    }

    /** Action callback. */
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == cbDefault) {
            bHasChange = true;
        }
    }
}
