/*
 * Created on 02.04.2005
 */
package net.sourceforge.ganttproject.gui.options;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.ComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SpringLayout;
import javax.swing.border.EmptyBorder;

import net.sourceforge.ganttproject.gui.options.model.BooleanOption;
import net.sourceforge.ganttproject.gui.options.model.EnumerationOption;
import net.sourceforge.ganttproject.gui.options.model.GPOption;
import net.sourceforge.ganttproject.gui.options.model.GPOptionGroup;
import net.sourceforge.ganttproject.language.GanttLanguage;

/**
 * @author bard
 */
public class OptionsPageBuilder {
    I18N myi18n = new I18N();

    public JComponent buildPage(GPOptionGroup[] optionGroups, String pageID) {
        JPanel result = new JPanel(new BorderLayout());
        result.setBorder(new EmptyBorder(0, 5, 0, 5));
        TopPanel topPanel = new TopPanel(myi18n.getPageTitle(pageID), myi18n
                .getPageDescription(pageID));
        topPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        result.add(topPanel, BorderLayout.NORTH);
        JComponent planePage = buildPlanePage(optionGroups);
        result.add(planePage, BorderLayout.CENTER);
        return result;
    }

    public JComponent buildPlanePage(GPOptionGroup[] optionGroups) {
        JComponent optionsPanel = new JPanel(new SpringLayout());
        for (int i = 0; i < optionGroups.length; i++) {
            optionsPanel.add(createGroupComponent(optionGroups[i]));
        }
        SpringUtilities.makeCompactGrid(optionsPanel, optionGroups.length, 1,
                0, 0, 5, 5);
        return optionsPanel;
    }

    private Component createGroupComponent(GPOptionGroup group) {
        JPanel optionsPanel = new JPanel(new SpringLayout());
        if (group.isTitled()) {
            optionsPanel.setBorder(BorderFactory.createTitledBorder(myi18n
                    .getOptionGroupLabel(group)));
        }
        GPOption[] options = group.getOptions();
        for (int i = 0; i < options.length; i++) {
            JLabel nextLabel = new JLabel(GanttLanguage.getInstance().getText(
                    "option." + options[i].getID() + ".label"));
            nextLabel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 3));
            optionsPanel.add(nextLabel);
            optionsPanel.add(createOptionComponent(options[i]));
        }
        if (options.length > 0) {
            SpringUtilities.makeCompactGrid(optionsPanel, options.length, 2, 0,
                    0, 0, 0);
        }
        return optionsPanel;
    }

    private Component createOptionComponent(GPOption option) {
        Component result = null;
        if (option instanceof EnumerationOption) {
            result = createEnumerationComponent((EnumerationOption) option);
        } else if (option instanceof BooleanOption) {
            result = createBooleanComponent((BooleanOption) option);
        }
        else if (option instanceof GPOptionGroup) {
            result = createButtonComponent((GPOptionGroup)option);
        }
        if (result == null) {
            result = new JLabel("Unknown option class=" + option.getClass());
        }
        return result;
    }

    private Component createButtonComponent(GPOptionGroup optionGroup) {
        Action action = new AbstractAction(myi18n.getAdvancedActionTitle()) {
            public void actionPerformed(ActionEvent e) {
                System.err
                        .println("[OptionsPageBuilder] createButtonComponent: ");
            }
            
        };
        JButton result = new JButton(action);
        return result;
    }
    private Component createBooleanComponent(BooleanOption option) {
        JCheckBox result = new JCheckBox(new BooleanOptionAction(option));
        result.setSelected(option.isChecked());
        return result;
    }

    private JComboBox createEnumerationComponent(EnumerationOption option) {
        ComboBoxModel model = new EnumerationOptionComboBoxModel(option);
        JComboBox result = new JComboBox(model);
        return result;
    }

    private static class I18N {
        String getPageTitle(String pageID) {
            return GanttLanguage.getInstance().getText(
                    "optionPage." + pageID + ".title");
        }

        String getPageDescription(String pageID) {
            return GanttLanguage.getInstance().getText(
                    "optionPage." + pageID + ".description");
        }

        String getOptionGroupLabel(GPOptionGroup group) {
            return GanttLanguage.getInstance().getText(
                    "optionGroup." + group.getID() + ".label");
        }

        String getOptionLabel(GPOption option) {
            return GanttLanguage.getInstance().getText(
                    "option." + option.getID() + ".label");
        }

        String getAdvancedActionTitle() {
            return GanttLanguage.getInstance().getText("optionAdvanced.label");
        }
    }
}
