/***************************************************************************
 PDFSettingsPanel.java 
 ------------------------------------------
 begin                : 28 juin 2004
 copyright            : (C) 2004 by Thomas Alexandre
 email                : alexthomas(at)ganttproject.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
package net.sourceforge.ganttproject.gui.options;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.security.AccessControlException;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import net.sourceforge.ganttproject.GanttProject;
import net.sourceforge.ganttproject.filter.GanttXSLFileFilter;
import net.sourceforge.ganttproject.gui.TextFieldAndFileChooserComponent;
import net.sourceforge.ganttproject.language.GanttLanguage;

/**
 * @author athomas Panel for options of PDF export
 */
public class PDFSettingsPanel extends GeneralOptionPanel implements
        ActionListener {
    JCheckBox cbDefault;

    private GanttProject appli;

    private TextFieldAndFileChooserComponent myChooser;

    /** Constructor. */
    public PDFSettingsPanel(GanttProject parent) {
        super(GanttProject.correctLabel(GanttLanguage.getInstance().getText(
                "pdfexport")), GanttLanguage.getInstance().getText(
                "settingsPDFExport"), parent);

        appli = parent;
        myChooser = new TextFieldAndFileChooserComponent(this, language
                .getText("selectThemeFile")) {
            protected void onFileChosen(File file) {
                bHasChange = true;
            }
        };
        myChooser.setFileFilter(new GanttXSLFileFilter());
        vb.add(myChooser.getComponent());
        vb.add(new JPanel());
        JPanel defaultPanel = new JPanel(new BorderLayout());
        defaultPanel.add(cbDefault = new JCheckBox(), BorderLayout.WEST);
        defaultPanel.add(new JLabel(language.getText("xslFoFile")),
                BorderLayout.CENTER);
        vb.add(defaultPanel);
        vb.add(new JPanel());

        cbDefault.addActionListener(this);

        applyComponentOrientation(language.getComponentOrientation());
    }

    /** This method check if the value has changed, and assk for commit changes. */
    public boolean applyChanges(boolean askForApply) {
        if (bHasChange)
            if (!askForApply || (askForApply && askForApplyChanges())) {
                appli.getOptions().setXslFo(
                        getXslFoFile(cbDefault.isSelected()));
            }
        return bHasChange;
    }

    /** Initialize the component. */
    public void initialize() {
        try {
            File xslFile = new File(
                    appli.getOptions().getXslFo() != null ? appli.getOptions()
                            .getXslFo() : System.getProperty("user.home"));
            myChooser.setFile(xslFile);
        } catch (AccessControlException e) {
            // This can happen when running in a sandbox (Java WebStart)
            System.err.println(e + ": " + e.getMessage());
        }
    }

    /** @return the xmlfo file for pdf export. */
    public String getXslFoFile(boolean useDefault) {
        if (!useDefault) {
            if (myChooser.getFile() != null && myChooser.getFile().exists()) {
                return myChooser.getFile().getAbsolutePath();
            }
        }
        return PDFSettingsPanel.class.getResource("/xslfo/ganttproject.xsl")
                .toString();
    }

    /** Action callback. */
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == cbDefault) {
            bHasChange = true;
        }
    }
}
