/*
 * Created on 18.06.2005
 */
package net.sourceforge.ganttproject.gui.options.model;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

public class DefaultEnumerationOption extends GPAbstractOption implements
        EnumerationOption, ChangeValueDispatcher {

    private List myListeners = new ArrayList();

    private final List myValues;

    private String myValue;

    private String myLockedValue;

    public DefaultEnumerationOption(String id, String[] values) {
        super(id);
        myValues = Arrays.asList(values);
    }

    public DefaultEnumerationOption(String id, List values) {
        super(id);
        myValues = values;
    }

    public String[] getAvailableValues() {
        return (String[]) myValues.toArray(new String[0]);
    }

    public void setValue(String value) {
        if (!isLocked()) {
            throw new IllegalStateException("Lock option before setting value");
        }

        ChangeValueEvent event = new ChangeValueEvent(getID(), myLockedValue,
                value);
        myLockedValue = value;
        fireChangeValueEvent(event);
    }

    public String getValue() {
        return myValue;
    }

    public void commit() {
        super.commit();
        myValue = myLockedValue;
    }

    public void addChangeValueListener(ChangeValueListener listener) {
        myListeners.add(listener);
    }

    public void fireChangeValueEvent(ChangeValueEvent event) {
        Iterator it = myListeners.iterator();
        while (it.hasNext()) {
            ((ChangeValueListener) it.next()).changeValue(event);
        }
    }

}
