/*
 * Created on 30.04.2005
 */
package net.sourceforge.ganttproject.importer;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import net.sourceforge.ganttproject.GanttProject;
import net.sourceforge.ganttproject.gui.UIFacade;
import net.sourceforge.ganttproject.gui.projectwizard.WizardImpl;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.Platform;

/**
 * @author bard
 */
public class ImportFileWizardImpl extends WizardImpl {
    private State myState;

    private GanttProject myProject;

    public ImportFileWizardImpl(UIFacade uiFacade, GanttProject project) {
        super(uiFacade, "");
        myProject = project;
        myState = new State();
        Importer[] importers = getImporters();
        addPage(new ImporterChooserPage(importers, myState));
        addPage(new FileChooserPage(myState));
    }

    private Importer[] getImporters() {
        List result = new ArrayList();
        String extensionPointID = "net.sourceforge.ganttproject.importer";
        IConfigurationElement[] configElements = Platform
                .getExtensionRegistry().getConfigurationElementsFor(
                        extensionPointID);
        for (int i = 0; i < configElements.length; i++) {
            try {
                Object nextImporter = configElements[i]
                        .createExecutableExtension("class");
                if (nextImporter instanceof Importer == false) {
                    throw new RuntimeException("Extension of point="
                            + extensionPointID + " is expected to implement "
                            + Importer.class);
                }
                ((Importer) nextImporter)
                        .addPreImporterProcessor(new PreImporterProcessor() {
                            public void run() {
                                myProject.getTree().addBlankLine(
                                        null,
                                        myProject.getTree().getRoot()
                                                .getChildCount());
                            }
                        });

                result.add(nextImporter);
            } catch (CoreException e) {
                e.printStackTrace();
                throw new RuntimeException("Failed to instantiate importer", e);
            }
        }
        return (Importer[]) result.toArray(new Importer[0]);
    }

    protected void onOkPressed() {
        super.onOkPressed();
        myState.myImporter.run(myProject, myState.getFile());
    }

    protected boolean canFinish() {
        return myState.myImporter != null && myState.getFile() != null;
    }

    class State {
        Importer myImporter;

        private File myFile;

        void setFile(File myFile) {
            this.myFile = myFile;
            ImportFileWizardImpl.this.adjustButtonState();
        }

        File getFile() {
            return myFile;
        }
    }
}
