package net.sourceforge.ganttproject.plugins;

import java.util.ArrayList;
import java.util.List;

import net.sourceforge.ganttproject.chart.Chart;
import net.sourceforge.ganttproject.export.Exporter;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;

/**
 * Very basic Plugin Manager
 * @author bbaranne
 *
 */
public class PluginManager {

    private static final String EXTENSION_POINT_ID_CHART = "net.sourceforge.ganttproject.chart";

    private static final String EXTENSION_POINT_ID_EXPORTER = "net.sourceforge.ganttproject.exporter";

    private Chart[] myCharts;

    private Exporter[] myExporters;

    public Chart[] getCharts() {
        if (myCharts == null) {
            List result = new ArrayList();
            String extensionPointID = EXTENSION_POINT_ID_CHART;
            IExtensionRegistry extensionRegistry = Platform
                    .getExtensionRegistry();
            if (extensionRegistry == null) {
                return new Chart[0];
            }
            IConfigurationElement[] configElements = extensionRegistry
                    .getConfigurationElementsFor(extensionPointID);
            for (int i = 0; i < configElements.length; i++) {
                try {
                    Object nextChart = configElements[i]
                            .createExecutableExtension("class");
                    if (!(nextChart instanceof Chart)) {
                        throw new RuntimeException("Extension of point="
                                + extensionPointID
                                + " is expected to implement " + Chart.class);
                    }
                    result.add(nextChart);
                } catch (CoreException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }
            myCharts = (Chart[]) result.toArray(new Chart[0]);
        }
        return myCharts;
    }

    public Exporter[] getExporters() {
        if (myExporters == null) {
            List result = new ArrayList();
            String extensionPointID = EXTENSION_POINT_ID_EXPORTER;
            IExtensionRegistry extensionRegistry = Platform
                    .getExtensionRegistry();
            if (extensionRegistry == null) {
                return new Exporter[0];
            }
            IConfigurationElement[] configElements = extensionRegistry
                    .getConfigurationElementsFor(extensionPointID);
            for (int i = 0; i < configElements.length; i++) {
                try {
                    Object nextExporter = configElements[i]
                            .createExecutableExtension("class");
                    if (!(nextExporter instanceof Exporter)) {
                        throw new RuntimeException("Extension of point="
                                + extensionPointID
                                + " is expected to implement " + Exporter.class);
                    }
                    result.add(nextExporter);
                } catch (CoreException e) {
                    e.printStackTrace();
                    throw new RuntimeException(
                            "Failed to instantiate importer", e);
                }
            }
            myExporters = (Exporter[]) result.toArray(new Exporter[0]);
        }
        return myExporters;

    }
}
