package org.ganttproject.impex.msproject;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;
import java.util.Locale;

import javax.swing.Action;

import net.sourceforge.ganttproject.GanttProject;
import net.sourceforge.ganttproject.action.CancelAction;
import net.sourceforge.ganttproject.action.OkAction;
import net.sourceforge.ganttproject.document.Document;
import net.sourceforge.ganttproject.document.FileDocument;
import net.sourceforge.ganttproject.gui.GanttDialogInfo;
import net.sourceforge.ganttproject.importer.Importer;
import net.sourceforge.ganttproject.importer.ImporterBase;
import net.sourceforge.ganttproject.language.GanttLanguage;

public class ImporterFromMsProjectFile extends ImporterBase implements Importer {
    public String getFileNamePattern() {
        return "mpp|mpx|xml";
    }

    public String getFileTypeDescription() {
        return GanttLanguage.getInstance().getText("impex.msproject.description");
    }

    public void run(GanttProject project, File selectedFile) {
        if (acceptImport(project)) {
            super.run();
            Document document = getDocument(selectedFile);
            openDocument(project, document);
        }

    }

    protected Document getDocument(File selectedFile) {
        return new FileDocument(selectedFile);
    }

    GanttMPXJOpen open;

    protected void openDocument(final GanttProject project, Document document) {
        try {
            if (document.getPath().toLowerCase().endsWith(".mpp"))
                open = new GanttMPPOpen(project.getTree(), project);
            else if (document.getPath().toLowerCase().endsWith(".mpx")) {
                open = null;
                final LocaleChooserPanel localePanel = new LocaleChooserPanel();
                project.showDialog(localePanel, new Action[] { new OkAction() {
                    public void actionPerformed(ActionEvent arg0) {
                        Locale importlocale = null;
                        importlocale = localePanel.getSelectedLocale();
                        if (importlocale.equals(Locale.US))
                            importlocale = Locale.ENGLISH;
                        if (importlocale != null) {
                            open = new GanttMPXOpen(project.getTree(), project,
                                    importlocale);
                            project.getStatusBar().setFirstText("Import", 2000);
                        }
                    }
                }, new CancelAction() {
                    public void actionPerformed(ActionEvent arg0) {
                    }
                } });

            } else if (document.getPath().toLowerCase().endsWith(".xml"))
                open = new GanttMSPDIOpen(project.getTree(), project);
            else
                open = null;

            open.load(document.getInputStream());

        } catch (IOException e) {
            GanttDialogInfo gdi = new GanttDialogInfo(project,
                    GanttDialogInfo.ERROR, GanttDialogInfo.YES_OPTION, project
                            .getLanguage().getText("msg2")
                            + "\n" + document.getDescription(), project
                            .getLanguage().getText("error"));
            gdi.show();
        }
    }

}
